@@ -59,10 +59,14 @@ void Allocate_Arrays(int wherefrom)
       S_Snd_Num = (int*)malloc(sizeof(int)*Num_Procs);
       F_Rcv_Num = (int*)malloc(sizeof(int)*Num_Procs);
       S_Rcv_Num = (int*)malloc(sizeof(int)*Num_Procs);
+      F_Snd_Num_WK = (int*)malloc(sizeof(int)*Num_Procs);
+      F_Rcv_Num_WK = (int*)malloc(sizeof(int)*Num_Procs);
       F_TopMAN = (int*)malloc(sizeof(int)*Num_Procs);
       S_TopMAN = (int*)malloc(sizeof(int)*Num_Procs);
       Snd_DS_NL_Size = (int*)malloc(sizeof(int)*Num_Procs);
       Rcv_DS_NL_Size = (int*)malloc(sizeof(int)*Num_Procs);
+      Snd_HFS_Size = (int*)malloc(sizeof(int)*Num_Procs);
+      Rcv_HFS_Size = (int*)malloc(sizeof(int)*Num_Procs);
       Start_Grid1 = (int*)malloc(sizeof(int)*Num_Procs);
       End_Grid1   = (int*)malloc(sizeof(int)*Num_Procs);
       Start_Grid2 = (int*)malloc(sizeof(int)*Num_Procs);
@@ -94,6 +98,9 @@ void Allocate_Arrays(int wherefrom)
         GxyzHistoryIn[i] = (double**)malloc(sizeof(double*)*(atomnum+1));
         for(j=0; j<(atomnum+1); j++) {
           GxyzHistoryIn[i][j] = (double*)malloc(sizeof(double)*4);
+          for (k=0; k<4; k++){
+            GxyzHistoryIn[i][j][k] = 0.0;
+	  }
 	}
       }
 
@@ -102,6 +109,9 @@ void Allocate_Arrays(int wherefrom)
         GxyzHistoryR[i] = (double**)malloc(sizeof(double*)*(atomnum+1));
         for(j=0; j<(atomnum+1); j++) {
           GxyzHistoryR[i][j] = (double*)malloc(sizeof(double)*4);
+          for (k=0; k<4; k++){
+            GxyzHistoryR[i][j][k] = 0.0;
+          }
 	}
       }
 
@@ -193,14 +203,43 @@ void Allocate_Arrays(int wherefrom)
         OrbPol_flag = (int*)malloc(sizeof(int)*(atomnum+1));
       }
 
-      /* LBFGS */
+      /* EF */
 
-      if (MD_switch==10){
-        NDIM_LBFGS = 3*atomnum + 3;
-        NWORK_LBFGS = NDIM_LBFGS*(2*MSAVE_LBFGS +1)+2*MSAVE_LBFGS;
-  
-        DIAG_LBFGS = (double*)malloc(sizeof(double)*NDIM_LBFGS);
-        W_LBFGS = (double*)malloc(sizeof(double)*NWORK_LBFGS);
+      if (MD_switch==4){
+
+        Hessian = (double**)malloc(sizeof(double*)*(3*atomnum+2));
+	for (i=0; i<(3*atomnum+2); i++){
+          Hessian[i] = (double*)malloc(sizeof(double)*(3*atomnum+2));
+  	  for (j=0; j<(3*atomnum+2); j++){
+            Hessian[i][j] = 0.0;
+	  }
+	}
+      }
+
+      /* BFGS */
+
+      if (MD_switch==5){
+
+        InvHessian = (double**)malloc(sizeof(double*)*(3*atomnum+2));
+	for (i=0; i<(3*atomnum+2); i++){
+          InvHessian[i] = (double*)malloc(sizeof(double)*(3*atomnum+2));
+  	  for (j=0; j<(3*atomnum+2); j++){
+            InvHessian[i][j] = 0.0;
+	  }
+	}
+      }
+
+      /* RF added by hmweng */
+
+      if (MD_switch==6){
+
+        Hessian = (double**)malloc(sizeof(double*)*(3*atomnum+2));
+	for (i=0; i<(3*atomnum+2); i++){
+          Hessian[i] = (double*)malloc(sizeof(double)*(3*atomnum+2));
+  	  for (j=0; j<(3*atomnum+2); j++){
+            Hessian[i][j] = 0.0;
+	  }
+	}
       }
 
     break;
@@ -503,6 +542,96 @@ void Allocate_Arrays(int wherefrom)
       }
 
     break;
+    
+    case 8:  /* hmweng */
+
+      Wannier_ProSpeName = (char**)malloc(sizeof(char*)*Wannier_Num_Kinds_Projectors);
+      for (i=0; i<Wannier_Num_Kinds_Projectors; i++){
+        Wannier_ProSpeName[i] = (char*)malloc(sizeof(char)*YOUSO10);
+      }  
+
+      Wannier_ProName = (char**)malloc(sizeof(char*)*Wannier_Num_Kinds_Projectors);
+      for (i=0; i<Wannier_Num_Kinds_Projectors; i++){
+        Wannier_ProName[i] = (char*)malloc(sizeof(char)*YOUSO10);
+      }  
+
+      Wannier_Pos = (double**)malloc(sizeof(double*)*Wannier_Num_Kinds_Projectors);
+      for (i=0; i<Wannier_Num_Kinds_Projectors; i++){
+        Wannier_Pos[i] = (double*)malloc(sizeof(double)*4);
+      }
+
+      Wannier_X_Direction = (double**)malloc(sizeof(double*)*Wannier_Num_Kinds_Projectors);
+      for (i=0; i<Wannier_Num_Kinds_Projectors; i++){
+        Wannier_X_Direction[i] = (double*)malloc(sizeof(double)*4);
+      }
+
+      Wannier_Z_Direction = (double**)malloc(sizeof(double*)*Wannier_Num_Kinds_Projectors);
+      for (i=0; i<Wannier_Num_Kinds_Projectors; i++){
+        Wannier_Z_Direction[i] = (double*)malloc(sizeof(double)*4);
+      }
+
+      Wannier_Num_Pro = (int*)malloc(sizeof(int)*Wannier_Num_Kinds_Projectors);
+
+      Wannier_NumL_Pro = (int**)malloc(sizeof(int*)*Wannier_Num_Kinds_Projectors);
+      for (i=0; i<Wannier_Num_Kinds_Projectors; i++){
+        Wannier_NumL_Pro[i] = (int*)malloc(sizeof(int)*4);
+      }
+
+      WannierPro2SpeciesNum = (int*)malloc(sizeof(int)*Wannier_Num_Kinds_Projectors);
+      Wannier_Guide=(double**)malloc(sizeof(double*)*3);
+      for(i=0;i<3;i++){
+        Wannier_Guide[i]=(double*)malloc(sizeof(double)*Wannier_Func_Num);
+      }
+
+
+      Wannier_Euler_Rotation_Angle = (double**)malloc(sizeof(double*)*Wannier_Num_Kinds_Projectors);
+      for (i=0; i<Wannier_Num_Kinds_Projectors; i++){
+        Wannier_Euler_Rotation_Angle[i] = (double*)malloc(sizeof(double)*3);
+        for(j=0; j<3; j++){
+          Wannier_Euler_Rotation_Angle[i][j] =0.0; 
+        }
+      } 
+
+      Wannier_RotMat_for_Real_Func = (double****)malloc(sizeof(double***)*Wannier_Num_Kinds_Projectors);
+      for (i=0; i<Wannier_Num_Kinds_Projectors; i++){
+        Wannier_RotMat_for_Real_Func[i] = (double***)malloc(sizeof(double**)*4);
+        for(j=0; j<4; j++){
+          Wannier_RotMat_for_Real_Func[i][j] = (double**)malloc(sizeof(double*)*(2*j+1));
+          for(l=0; l<2*j+1; l++){
+            Wannier_RotMat_for_Real_Func[i][j][l] = (double*)malloc(sizeof(double)*(2*j+1));
+          }
+        }
+      }
+      
+      Wannier_ProName2Num=(int*)malloc(sizeof(int)*Wannier_Num_Kinds_Projectors);
+
+      alloc_first[24] = 0;
+
+      break;
+
+      /* hmweng */
+
+      case 9:
+
+      Wannier_Select_Matrix=(int**)malloc(sizeof(int*)*Wannier_Num_Kinds_Projectors);
+      for(i=0; i<Wannier_Num_Kinds_Projectors; i++){
+        Wannier_Select_Matrix[i]=(int*)malloc(sizeof(int)*Wannier_Num_Pro[i]);
+        for(j=0;j<Wannier_Num_Pro[i];j++){
+	  Wannier_Select_Matrix[i][j]=-1;
+        } 
+      }
+
+      Wannier_Projector_Hybridize_Matrix=(double***)malloc(sizeof(double**)*Wannier_Num_Kinds_Projectors);
+      for(i=0; i<Wannier_Num_Kinds_Projectors; i++){
+        Wannier_Projector_Hybridize_Matrix[i]=(double**)malloc(sizeof(double*)*Wannier_Num_Pro[i]);
+        for(j=0;j<Wannier_Num_Pro[i];j++){
+	  Wannier_Projector_Hybridize_Matrix[i][j]=(double*)malloc(sizeof(double)*Wannier_Num_Pro[i]);
+        }
+      }
+
+      alloc_first[25] = 0; 
+
+      break;
 
   }
 
