@@ -26,6 +26,13 @@
 #include "mpi.h"
 #endif
 
+#ifdef noomp
+#include "mimic_omp.h"
+#else
+#include <omp.h>
+#endif
+
+
 static double Band_collinear(
              int knum_i, int knum_j, int knum_k,
              int SpinP_switch,
@@ -120,7 +127,6 @@ double Band_DFT_Col(int SCF_iter,
 {
 
 
-
   return Band_collinear2( SCF_iter,
                           knum_i, knum_j, knum_k, SpinP_switch,
 			  nh, CntOLP, CDM, EDM, Eele0, Eele1,
@@ -198,8 +204,8 @@ static double Band_collinear2(int SCF_iter,
 {
   static int firsttime=1;
   int i,j,k,l,m,n,wan,MaxN,i0,ks;
-  int i1,j1,ia,jb,lmax,po,po1,spin,s1,e1;
-  int num2,RnB,l1,l2,l3,loop_num;
+  int i1,i1s,j1,ia,jb,lmax,po,po1,spin,s1,e1;
+  int num2,RnB,l1,l2,l3,loop_num,ns,ne;
   int ct_AN,h_AN,wanA,tnoA,wanB,tnoB;
   int MA_AN,GA_AN,Anum,num_kloop0;
   int T_knum,S_knum,E_knum,kloop,kloop0;
@@ -207,7 +213,7 @@ static double Band_collinear2(int SCF_iter,
   double time0;
   int LB_AN,GB_AN,Bnum;
   double k1,k2,k3,Fkw;
-  double sum,sumi;
+  double sum,sumi,sum_weights;
   double Num_State;
   double My_Num_State;
   double FermiF;
@@ -272,6 +278,9 @@ static double Band_collinear2(int SCF_iter,
   dcomplex *BLAS_H;
   dcomplex *BLAS_C;
 
+  /* for OpenMP */
+  int OMPID,Nthrds,Nprocs;
+
   /* for time */
   dtime(&TStime);
 
@@ -363,114 +372,147 @@ static double Band_collinear2(int SCF_iter,
   PrintMemory("Band_DFT: C",  sizeof(dcomplex)*(n+1)*(n+1),NULL);
   }
 
-  /**************************************************************
-   k_op[i][j][k]: weight of DOS 
+  /***********************************************
+              k-points by regular mesh 
+  ***********************************************/
+
+  if (way_of_kpoint==1){
+
+    /**************************************************************
+     k_op[i][j][k]: weight of DOS 
                  =0   no calc.
                  =1   G-point
                  =2   which has k<->-k point
         Now , only the relation, E(k)=E(-k), is used. 
 
     Future release: k_op will be used for symmetry operation 
-  *************************************************************/
+    *************************************************************/
 
-  for (i=0;i<=knum_i-1;i++) {
-    for (j=0;j<=knum_j-1;j++) {
-      for (k=0;k<=knum_k-1;k++) {
-	k_op[i][j][k]=-999;
+    for (i=0;i<=knum_i-1;i++) {
+      for (j=0;j<=knum_j-1;j++) {
+	for (k=0;k<=knum_k-1;k++) {
+	  k_op[i][j][k]=-999;
+	}
       }
     }
-  }
 
-  for (i=0;i<=knum_i-1;i++) {
-    for (j=0;j<=knum_j-1;j++) {
-      for (k=0;k<=knum_k-1;k++) {
-	if ( k_op[i][j][k]==-999 ) {
-	  k_inversion(i,j,k,knum_i,knum_j,knum_k,&ii,&ij,&ik);
-	  if ( i==ii && j==ij && k==ik ) {
-	    k_op[i][j][k]    = 1;
+    for (i=0;i<=knum_i-1;i++) {
+      for (j=0;j<=knum_j-1;j++) {
+	for (k=0;k<=knum_k-1;k++) {
+	  if ( k_op[i][j][k]==-999 ) {
+	    k_inversion(i,j,k,knum_i,knum_j,knum_k,&ii,&ij,&ik);
+	    if ( i==ii && j==ij && k==ik ) {
+	      k_op[i][j][k]    = 1;
+	    }
+
+	    else {
+	      k_op[i][j][k]    = 2;
+	      k_op[ii][ij][ik] = 0;
+	    }
 	  }
+	} /* k */
+      } /* j */
+    } /* i */
 
-          else {
-	    k_op[i][j][k]    = 2;
-	    k_op[ii][ij][ik] = 0;
+    /***********************************
+       one-dimentionalize for MPI
+    ************************************/
+
+    T_knum = 0;
+    for (i=0; i<=(knum_i-1); i++){
+      for (j=0; j<=(knum_j-1); j++){
+	for (k=0; k<=(knum_k-1); k++){
+	  if (0<k_op[i][j][k]){
+	    T_knum++;
 	  }
 	}
-      } /* k */
-    } /* j */
-  } /* i */
-
-  /***********************************
-       one-dimentionalize for MPI
-  ************************************/
-
-  T_knum = 0;
-  for (i=0; i<=(knum_i-1); i++){
-    for (j=0; j<=(knum_j-1); j++){
-      for (k=0; k<=(knum_k-1); k++){
-        if (0<k_op[i][j][k]){
-          T_knum++;
-        }
       }
     }
-  }
 
-  /* set T_KGrids1,2,3 and T_k_op */
+    /* set T_KGrids1,2,3 and T_k_op */
 
-  T_knum = 0;
-  for (i=0; i<knum_i; i++){
+    T_knum = 0;
+    for (i=0; i<knum_i; i++){
 
-    if (knum_i==1)  k1 = 0.0;
-    else            k1 = -0.5 + (2.0*(double)i+1.0)/(2.0*(double)knum_i) + Shift_K_Point;
-
-    for (j=0; j<knum_j; j++){
-
-      if (knum_j==1)  k2 = 0.0;
-      else            k2 = -0.5 + (2.0*(double)j+1.0)/(2.0*(double)knum_j) - Shift_K_Point;
-
-      for (k=0; k<knum_k; k++){
-
-	if (knum_k==1)  k3 = 0.0;
-	else            k3 = -0.5 + (2.0*(double)k+1.0)/(2.0*(double)knum_k) + 2.0*Shift_K_Point;
-
-        if (0<k_op[i][j][k]){
-
-	  T_KGrids1[T_knum] = k1;
-	  T_KGrids2[T_knum] = k2;
-	  T_KGrids3[T_knum] = k3;
-	  T_k_op[T_knum]    = k_op[i][j][k];
-
-          T_knum++;
-        }
-      }
-    }
-  }
-
-  if (myid0==Host_ID){
-
-    printf(" KGrids1: ");fflush(stdout);
-    for (i=0;i<=knum_i-1;i++){
       if (knum_i==1)  k1 = 0.0;
       else            k1 = -0.5 + (2.0*(double)i+1.0)/(2.0*(double)knum_i) + Shift_K_Point;
-      printf("%9.5f ",k1);fflush(stdout);
-    }
-    printf("\n");fflush(stdout);
 
-    printf(" KGrids2: ");fflush(stdout);
+      for (j=0; j<knum_j; j++){
 
-    for (i=0;i<=knum_j-1;i++){
-      if (knum_j==1)  k2 = 0.0;
-      else            k2 = -0.5 + (2.0*(double)i+1.0)/(2.0*(double)knum_j) - Shift_K_Point;
-      printf("%9.5f ",k2);fflush(stdout);
-    }
-    printf("\n");fflush(stdout);
+	if (knum_j==1)  k2 = 0.0;
+	else            k2 = -0.5 + (2.0*(double)j+1.0)/(2.0*(double)knum_j) - Shift_K_Point;
 
-    printf(" KGrids3: ");fflush(stdout);
-    for (i=0;i<=knum_k-1;i++){
-     if (knum_k==1)  k3 = 0.0;
-     else            k3 = -0.5 + (2.0*(double)i+1.0)/(2.0*(double)knum_k) + 2.0*Shift_K_Point;
-      printf("%9.5f ",k3);fflush(stdout);
+	for (k=0; k<knum_k; k++){
+
+	  if (knum_k==1)  k3 = 0.0;
+	  else            k3 = -0.5 + (2.0*(double)k+1.0)/(2.0*(double)knum_k) + 2.0*Shift_K_Point;
+
+	  if (0<k_op[i][j][k]){
+
+	    T_KGrids1[T_knum] = k1;
+	    T_KGrids2[T_knum] = k2;
+	    T_KGrids3[T_knum] = k3;
+	    T_k_op[T_knum]    = k_op[i][j][k];
+
+	    T_knum++;
+	  }
+	}
+      }
     }
-    printf("\n");fflush(stdout);
+
+    if (myid0==Host_ID){
+
+      printf(" KGrids1: ");fflush(stdout);
+      for (i=0;i<=knum_i-1;i++){
+	if (knum_i==1)  k1 = 0.0;
+	else            k1 = -0.5 + (2.0*(double)i+1.0)/(2.0*(double)knum_i) + Shift_K_Point;
+	printf("%9.5f ",k1);fflush(stdout);
+      }
+      printf("\n");fflush(stdout);
+
+      printf(" KGrids2: ");fflush(stdout);
+
+      for (i=0;i<=knum_j-1;i++){
+	if (knum_j==1)  k2 = 0.0;
+	else            k2 = -0.5 + (2.0*(double)i+1.0)/(2.0*(double)knum_j) - Shift_K_Point;
+	printf("%9.5f ",k2);fflush(stdout);
+      }
+      printf("\n");fflush(stdout);
+
+      printf(" KGrids3: ");fflush(stdout);
+      for (i=0;i<=knum_k-1;i++){
+	if (knum_k==1)  k3 = 0.0;
+	else            k3 = -0.5 + (2.0*(double)i+1.0)/(2.0*(double)knum_k) + 2.0*Shift_K_Point;
+	printf("%9.5f ",k3);fflush(stdout);
+      }
+      printf("\n");fflush(stdout);
+    }
+
+  }
+
+  /***********************************************
+                 Monkhorst-Pack k-points 
+  ***********************************************/
+
+  else if (way_of_kpoint==2){
+
+    T_knum = num_non_eq_kpt; 
+   
+    for (k=0; k<num_non_eq_kpt; k++){
+      T_KGrids1[k] = NE_KGrids1[k];
+      T_KGrids2[k] = NE_KGrids2[k];
+      T_KGrids3[k] = NE_KGrids3[k];
+      T_k_op[k]    = NE_T_k_op[k];
+    }
+  }
+
+  /***********************************************
+            calculate the sum of weights
+  ***********************************************/
+
+  sum_weights = 0.0;
+  for (k=0; k<T_knum; k++){
+    sum_weights += (double)T_k_op[k];
   }
 
   /***********************************************
@@ -819,7 +861,7 @@ diagonalize1:
     dtime(&Stime);
 
     if (parallel_mode==0){
-      EigenBand_lapack(S,ko,n);
+      EigenBand_lapack(S,ko,n,1);
     }
     else if (SCF_iter==1 || all_knum!=1){
       Eigen_PHH(MPI_CommWD2[myworld2],S,ko,n,n,1);
@@ -851,15 +893,25 @@ diagonalize1:
 
       /* S * 1.0/sqrt(ko[l]) */
 
-      for (i1=1; i1<=n; i1++){
-	for (j1=1; j1<=n; j1++){
+#pragma omp parallel shared(BLAS_S,ko,S,n) private(OMPID,Nthrds,Nprocs,i1,j1)
+      { 
 
-	  S[i1][j1].r = S[i1][j1].r*ko[j1];
-	  S[i1][j1].i = S[i1][j1].i*ko[j1];
+	/* get info. on OpenMP */ 
 
-          BLAS_S[(j1-1)*n+i1-1] = S[i1][j1];
+	OMPID = omp_get_thread_num();
+	Nthrds = omp_get_num_threads();
+	Nprocs = omp_get_num_procs();
+
+	for (i1=1+OMPID; i1<=n; i1+=Nthrds){
+	  for (j1=1; j1<=n; j1++){
+
+	    S[i1][j1].r = S[i1][j1].r*ko[j1];
+	    S[i1][j1].i = S[i1][j1].i*ko[j1];
+
+	    BLAS_S[(j1-1)*n+i1-1] = S[i1][j1];
+	  } 
 	} 
-      } 
+      } /* #pragma omp parallel */
     }
 
     /****************************************************
@@ -911,6 +963,38 @@ diagonalize1:
 
       /* note for BLAS, A[M*K] * B[K*N] = C[M*N] */
 
+#pragma omp parallel shared(myid2,ie1,is1,BLAS_S,BLAS_H,BLAS_C,n) private(OMPID,Nthrds,Nprocs,Ctmp1,Ctmp2,BM,BN,BK,ns,ne)
+      { 
+
+	/* get info. on OpenMP */ 
+
+	OMPID = omp_get_thread_num();
+	Nthrds = omp_get_num_threads();
+	Nprocs = omp_get_num_procs();
+
+        ns = is1[myid2] + OMPID*(ie1[myid2]-is1[myid2]+1)/Nthrds;
+        ne = is1[myid2] + (OMPID+1)*(ie1[myid2]-is1[myid2]+1)/Nthrds - 1;
+
+	BM = n;
+	BN = ne - ns + 1;
+	BK = n;
+
+	Ctmp1.r = 1.0;
+	Ctmp1.i = 0.0;
+	Ctmp2.r = 0.0;
+	Ctmp2.i = 0.0;
+
+	F77_NAME(zgemm,ZGEMM)("N","N", &BM,&BN,&BK, 
+			      &Ctmp1, 
+			      BLAS_H, &BM,
+			      &BLAS_S[(ns-1)*n], &BK,
+			      &Ctmp2, 
+			      &BLAS_C[(ns-1)*n], &BM);
+
+      } /* #pragma omp parallel */
+
+
+      /*
       BM = n;
       BN = ie1[myid2] - is1[myid2] + 1;  
       BK = n;
@@ -926,6 +1010,8 @@ diagonalize1:
 			    &BLAS_S[(is1[myid2]-1)*n], &BK,
                             &Ctmp2, 
 			    &BLAS_C[(is1[myid2]-1)*n], &BM);
+      */
+
 
       /* 1.0/sqrt(ko[l]) * U^+ H * U * 1.0/sqrt(ko[l]) */
       /* H is distributed by row in each processor */
@@ -951,27 +1037,41 @@ diagonalize1:
 
       /* note for BLAS, A[M*K] * B[K*N] = C[M*N] */
 
-      BM = n;
-      BN = ie1[myid2] - is1[myid2] + 1;  
-      BK = n;
+#pragma omp parallel shared(H,myid2,ie1,is1,BLAS_S,BLAS_H,BLAS_C,n) private(OMPID,Nthrds,Nprocs,Ctmp1,Ctmp2,BM,BN,BK,ns,ne,i1,j1)
+      { 
 
-      Ctmp1.r = 1.0;
-      Ctmp1.i = 0.0;
-      Ctmp2.r = 0.0;
-      Ctmp2.i = 0.0;
+	/* get info. on OpenMP */ 
 
-      F77_NAME(zgemm,ZGEMM)("C","N", &BM,&BN,&BK, 
-                            &Ctmp1,
-                            BLAS_S, &BM,
-			    &BLAS_C[(is1[myid2]-1)*n], &BK, 
-                            &Ctmp2, 
-			    &BLAS_H[(is1[myid2]-1)*n], &BM);
+	OMPID = omp_get_thread_num();
+	Nthrds = omp_get_num_threads();
+	Nprocs = omp_get_num_procs();
 
-      for (j1=is1[myid2]; j1<=ie1[myid2]; j1++){
-        for (i1=1; i1<=n; i1++){
-          H[j1][i1] = BLAS_H[(j1-1)*n+i1-1];            
+        ns = is1[myid2] + OMPID*(ie1[myid2]-is1[myid2]+1)/Nthrds;
+        ne = is1[myid2] + (OMPID+1)*(ie1[myid2]-is1[myid2]+1)/Nthrds - 1;
+
+	BM = n;
+	BN = ne - ns + 1;
+	BK = n;
+
+	Ctmp1.r = 1.0;
+	Ctmp1.i = 0.0;
+	Ctmp2.r = 0.0;
+	Ctmp2.i = 0.0;
+
+	F77_NAME(zgemm,ZGEMM)("C","N", &BM,&BN,&BK, 
+			      &Ctmp1,
+			      BLAS_S, &BM,
+			      &BLAS_C[(ns-1)*n], &BK, 
+			      &Ctmp2, 
+			      &BLAS_H[(ns-1)*n], &BM);
+
+	for (j1=ns; j1<=ne; j1++){
+	  for (i1=1; i1<=n; i1++){
+	    H[j1][i1] = BLAS_H[(j1-1)*n+i1-1];            
+	  }
 	}
-      }
+
+      } /* #pragma omp parallel */
 
       /* broadcast H */
 
@@ -1004,34 +1104,45 @@ diagonalize1:
 
       /* note for BLAS, A[M*K] * B[K*N] = C[M*N] */
 
-      BM = n;
-      BN = n;
-      BK = n;
+#pragma omp parallel shared(BLAS_S,BLAS_H,BLAS_C,n) private(OMPID,Nthrds,Nprocs,Ctmp1,Ctmp2,BM,BN,BK)
+      { 
 
-      Ctmp1.r = 1.0;
-      Ctmp1.i = 0.0;
-      Ctmp2.r = 0.0;
-      Ctmp2.i = 0.0;
+	/* get info. on OpenMP */ 
 
-      F77_NAME(zgemm,ZGEMM)("N","N", &BM,&BN,&BK, 
-                            &Ctmp1, 
-                            BLAS_H, &BM,
-			    BLAS_S, &BK,
-                            &Ctmp2, 
-			    BLAS_C, &BM);
+	OMPID = omp_get_thread_num();
+	Nthrds = omp_get_num_threads();
+	Nprocs = omp_get_num_procs();
+
+	BM = n;
+	BN = (OMPID+1)*n/Nthrds - (OMPID*n/Nthrds+1) + 1;
+	BK = n;
+
+	Ctmp1.r = 1.0;
+	Ctmp1.i = 0.0;
+	Ctmp2.r = 0.0;
+	Ctmp2.i = 0.0;
+
+	F77_NAME(zgemm,ZGEMM)("N","N", &BM,&BN,&BK, 
+			      &Ctmp1, 
+			      BLAS_H, &BM,
+			      &BLAS_S[(OMPID*n/Nthrds)*n], &BK,
+			      &Ctmp2, 
+			      &BLAS_C[(OMPID*n/Nthrds)*n], &BM);
+
+      } /* #pragma omp parallel */
 
       /* 1.0/sqrt(ko[l]) * U^+ H * U * 1.0/sqrt(ko[l]) */
 
       /*
       for (j1=1; j1<=n; j1++){
-        for (i1=1; i1<=n; i1++){
+	for (i1=1; i1<=n; i1++){
 
 	  sum  = 0.0;
 	  sumi = 0.0;
 
 	  for (l=1; l<=n; l++){
-	    sum  +=  S[i1][l].r*C[j1][l].r + S[i1][l].i*C[j1][l].i;
-	    sumi +=  S[i1][l].r*C[j1][l].i - S[i1][l].i*C[j1][l].r;
+	    sum  += S[i1][l].r*C[j1][l].r + S[i1][l].i*C[j1][l].i;
+	    sumi += S[i1][l].r*C[j1][l].i - S[i1][l].i*C[j1][l].r;
 	  }
 
 	  H[j1][i1].r = sum;
@@ -1042,27 +1153,38 @@ diagonalize1:
 
       /* note for BLAS, A[M*K] * B[K*N] = C[M*N] */
 
-      BM = n;
-      BN = n;
-      BK = n;
+#pragma omp parallel shared(H,BLAS_S,BLAS_H,BLAS_C,n) private(OMPID,Nthrds,Nprocs,Ctmp1,Ctmp2,BM,BN,BK,i1,j1)
+      { 
 
-      Ctmp1.r = 1.0;
-      Ctmp1.i = 0.0;
-      Ctmp2.r = 0.0;
-      Ctmp2.i = 0.0;
+	/* get info. on OpenMP */ 
 
-      F77_NAME(zgemm,ZGEMM)("C","N", &BM,&BN,&BK,
-                            &Ctmp1, 
-                            BLAS_S, &BM,
-			    BLAS_C, &BK, 
-                            &Ctmp2, 
-			    BLAS_H, &BM);
+	OMPID = omp_get_thread_num();
+	Nthrds = omp_get_num_threads();
+	Nprocs = omp_get_num_procs();
 
-      for (j1=1; j1<=n; j1++){
-        for (i1=1; i1<=n; i1++){
-          H[j1][i1] = BLAS_H[(j1-1)*n+i1-1];            
+	BM = n;
+	BN = (OMPID+1)*n/Nthrds - (OMPID*n/Nthrds+1) + 1;
+	BK = n;
+
+	Ctmp1.r = 1.0;
+	Ctmp1.i = 0.0;
+	Ctmp2.r = 0.0;
+	Ctmp2.i = 0.0;
+
+	F77_NAME(zgemm,ZGEMM)("C","N", &BM,&BN,&BK,
+			      &Ctmp1, 
+			      BLAS_S, &BM,
+			      &BLAS_C[(OMPID*n/Nthrds)*n], &BK, 
+			      &Ctmp2, 
+			      &BLAS_H[(OMPID*n/Nthrds)*n], &BM);
+
+	for (j1=(OMPID*n/Nthrds+1); j1<=(OMPID+1)*n/Nthrds; j1++){
+	  for (i1=1; i1<=n; i1++){
+	    H[j1][i1] = BLAS_H[(j1-1)*n+i1-1];            
+	  }
 	}
-      }
+
+      } /* #pragma omp parallel */
 
     } /* else */
 
@@ -1103,7 +1225,7 @@ diagonalize1:
     dtime(&Stime);
 
     if (parallel_mode==0){
-      EigenBand_lapack(C,ko,n);
+      EigenBand_lapack(C,ko,n,all_knum);
     }
     else{
       /*  The output C matrix is distributed by column. */
@@ -1186,27 +1308,41 @@ diagonalize1:
 
       /* note for BLAS, A[M*K] * B[K*N] = C[M*N] */
 
-      BM = n;
-      BN = ie2[myid2] - is2[myid2] + 1;  
-      BK = n;
+#pragma omp parallel shared(C,myid2,ie2,is2,BLAS_S,BLAS_H,BLAS_C,n) private(OMPID,Nthrds,Nprocs,Ctmp1,Ctmp2,BM,BN,BK,ns,ne,i1,j1)
+      { 
 
-      Ctmp1.r = 1.0;
-      Ctmp1.i = 0.0;
-      Ctmp2.r = 0.0;
-      Ctmp2.i = 0.0;
+	/* get info. on OpenMP */ 
 
-      F77_NAME(zgemm,ZGEMM)("N","N", &BM,&BN,&BK, 
-                            &Ctmp1, 
-                            BLAS_S, &BM,
-			    &BLAS_H[(is2[myid2]-1)*n], &BK,
-                            &Ctmp2, 
-			    &BLAS_C[(is2[myid2]-1)*n], &BM);
+	OMPID = omp_get_thread_num();
+	Nthrds = omp_get_num_threads();
+	Nprocs = omp_get_num_procs();
 
-      for (j1=is2[myid2]; j1<=ie2[myid2]; j1++){
-        for (i1=1; i1<=n; i1++){
-          C[j1][i1] = BLAS_C[(j1-1)*n+i1-1];            
+        ns = is2[myid2] + OMPID*(ie2[myid2]-is2[myid2]+1)/Nthrds;
+        ne = is2[myid2] + (OMPID+1)*(ie2[myid2]-is2[myid2]+1)/Nthrds - 1;
+
+	BM = n;
+	BN = ne - ns + 1;
+	BK = n;
+
+	Ctmp1.r = 1.0;
+	Ctmp1.i = 0.0;
+	Ctmp2.r = 0.0;
+	Ctmp2.i = 0.0;
+
+	F77_NAME(zgemm,ZGEMM)("N","N", &BM,&BN,&BK, 
+			      &Ctmp1, 
+			      BLAS_S, &BM,
+			      &BLAS_H[(ns-1)*n], &BK,
+			      &Ctmp2, 
+			      &BLAS_C[(ns-1)*n], &BM);
+
+	for (j1=ns; j1<=ne; j1++){
+	  for (i1=1; i1<=n; i1++){
+	    C[j1][i1] = BLAS_C[(j1-1)*n+i1-1];            
+	  }
 	}
-      }
+
+      } /* #pragma omp parallel */
 
       /* broadcast C:
        C is distributed by row in each processor
@@ -1306,9 +1442,9 @@ diagonalize1:
     } 
 
     if (SpinP_switch==0) 
-      Num_State = 2.0*Num_State/(double)knum_i/(double)knum_j/(double)knum_k;
+      Num_State = 2.0*Num_State/sum_weights;
     else 
-      Num_State = Num_State/(double)knum_i/(double)knum_j/(double)knum_k;
+      Num_State = Num_State/sum_weights;
 
     Dnum = TZ - Num_State - system_charge;
 
@@ -1342,12 +1478,12 @@ diagonalize1:
   } 
 
   if (SpinP_switch==0){
-    Eele0[0] = Eele0[0]/(double)knum_i/(double)knum_j/(double)knum_k;
+    Eele0[0] = Eele0[0]/sum_weights;
     Eele0[1] = Eele0[0];
   }
   else {
-    Eele0[0] = Eele0[0]/(double)knum_i/(double)knum_j/(double)knum_k;
-    Eele0[1] = Eele0[1]/(double)knum_i/(double)knum_j/(double)knum_k;
+    Eele0[0] = Eele0[0]/sum_weights;
+    Eele0[1] = Eele0[1]/sum_weights;
   }
 
   Uele = Eele0[0] + Eele0[1];
@@ -1421,40 +1557,72 @@ diagonalize1:
 
     dtime(&Stime0);
 
-    /* DM and EDM */
+    /* predetermination of k 
+       H is used as temporal array 
+    */ 
 
     k = 0;
-
     for (AN=1; AN<=atomnum; AN++){
+
       GA_AN = order_GA[AN];
       wanA = WhatSpecies[GA_AN];
       tnoA = Spe_Total_CNO[wanA];
-      Anum = MP[GA_AN];
+
+      H[0][AN].r = (double)k;
 
       for (LB_AN=0; LB_AN<=FNAN[GA_AN]; LB_AN++){
 	GB_AN = natn[GA_AN][LB_AN];
 	Rn = ncn[GA_AN][LB_AN];
 	wanB = WhatSpecies[GB_AN];
 	tnoB = Spe_Total_CNO[wanB];
-	Bnum = MP[GB_AN];
+	k += tnoA*tnoB;
+      }
+    }
 
-	l1 = atv_ijk[Rn][1];
-	l2 = atv_ijk[Rn][2];
-	l3 = atv_ijk[Rn][3];
-	kRn = k1*(double)l1 + k2*(double)l2 + k3*(double)l3;
+    /* DM and EDM */
 
-	si = sin(2.0*PI*kRn);
-	co = cos(2.0*PI*kRn);
+#pragma omp parallel shared(numprocs0,My_NZeros,SP_NZeros,MPI_CDM1_flag,EDM1,CDM1,VecFkwE,VecFkw,H,lmax,k1,k2,k3,atv_ijk,ncn,natn,FNAN,MP,Spe_Total_CNO,WhatSpecies,order_GA,atomnum) private(OMPID,Nthrds,Nprocs,AN,GA_AN,wanA,tnoA,Anum,k,LB_AN,GB_AN,Rn,wanB,tnoB,Bnum,l1,l2,l3,kRn,si,co,i,ia,j,jb,d1,d2,l,tmp,po,ID)
+    { 
 
-	for (i=0; i<tnoA; i++){
+      /* get info. on OpenMP */ 
 
-	  ia = Anum + i; 
+      OMPID = omp_get_thread_num();
+      Nthrds = omp_get_num_threads();
+      Nprocs = omp_get_num_procs();
 
-	  for (j=0; j<tnoB; j++){
+      for (AN=1+OMPID; AN<=atomnum; AN+=Nthrds){
 
-	    jb = Bnum + j;
+	GA_AN = order_GA[AN];
+	wanA = WhatSpecies[GA_AN];
+	tnoA = Spe_Total_CNO[wanA];
+	Anum = MP[GA_AN];
 
-	    /***************************************************************
+	k = (int)H[0][AN].r;
+
+	for (LB_AN=0; LB_AN<=FNAN[GA_AN]; LB_AN++){
+	  GB_AN = natn[GA_AN][LB_AN];
+	  Rn = ncn[GA_AN][LB_AN];
+	  wanB = WhatSpecies[GB_AN];
+	  tnoB = Spe_Total_CNO[wanB];
+	  Bnum = MP[GB_AN];
+
+	  l1 = atv_ijk[Rn][1];
+	  l2 = atv_ijk[Rn][2];
+	  l3 = atv_ijk[Rn][3];
+	  kRn = k1*(double)l1 + k2*(double)l2 + k3*(double)l3;
+
+	  si = sin(2.0*PI*kRn);
+	  co = cos(2.0*PI*kRn);
+
+	  for (i=0; i<tnoA; i++){
+
+	    ia = Anum + i; 
+
+	    for (j=0; j<tnoB; j++){
+
+	      jb = Bnum + j;
+
+	      /***************************************************************
              Note that the imagiary part is zero, 
              since
 
@@ -1464,42 +1632,44 @@ diagonalize1:
              B = (co - i si)(Re - i Im) = (co*Re - si*Im) - i (co*Im + si*Re) 
              Thus, Re(A+B) = 2*(co*Re - si*Im)
                    Im(A+B) = 0
-	    ***************************************************************/
+	      ***************************************************************/
 
-            po = 0;
-            ID = 0;
-            do { 
-              if (MPI_CDM1_flag[ID] && SP_NZeros[ID]<=k && k<(SP_NZeros[ID]+My_NZeros[ID])) po = 1;
-              ID++;
-            } while (po==0 && ID<numprocs0);
+	      po = 0;
+	      ID = 0;
+	      do { 
+		if (MPI_CDM1_flag[ID] && SP_NZeros[ID]<=k && k<(SP_NZeros[ID]+My_NZeros[ID])) po = 1;
+		ID++;
+	      } while (po==0 && ID<numprocs0);
 
-            if (po==1){
+	      if (po==1){
 
-	      d1 = 0.0;
-	      d2 = 0.0;
+		d1 = 0.0;
+		d2 = 0.0;
 	
-	      for (l=1; l<=lmax; l++){
+		for (l=1; l<=lmax; l++){
 
-		tmp = co*(H[ia][l].r*H[jb][l].r + H[ia][l].i*H[jb][l].i)
-		     -si*(H[ia][l].r*H[jb][l].i - H[ia][l].i*H[jb][l].r);
+		  tmp = co*(H[ia][l].r*H[jb][l].r + H[ia][l].i*H[jb][l].i)
+		       -si*(H[ia][l].r*H[jb][l].i - H[ia][l].i*H[jb][l].r);
 
-		d1 += VecFkw[l]*tmp;
-		d2 += VecFkwE[l]*tmp;;
+		  d1 += VecFkw[l]*tmp;
+		  d2 += VecFkwE[l]*tmp;;
+		}
+
+		CDM1[k] += d1;
+		EDM1[k] += d2; 
 	      }
 
-	      CDM1[k] += d1;
-              EDM1[k] += d2; 
+	      /* increment of k */
+
+	      k++;
+
 	    }
-
-	    /* increment of k */
-
-	    k++;
-
 	  }
-	}
 
-      } /* LB_AN */
-    } /* AN */
+	} /* LB_AN */
+      } /* AN */
+
+    } /* #pragma omp parallel */
 
     /* sum of CDM1 and EDM1 by Allreduce in MPI */
 
@@ -1813,7 +1983,7 @@ diagonalize1:
       dtime(&Stime);
 
       if (parallel_mode==0){
-	EigenBand_lapack(S,ko,n);
+	EigenBand_lapack(S,ko,n,1);
       }
       else{
 	Eigen_PHH(MPI_CommWD2[myworld2],S,ko,n,n,1);
@@ -1843,14 +2013,25 @@ diagonalize1:
 
       /* S * 1.0/sqrt(ko[l])  */
 
-      for (i1=1; i1<=n; i1++){
-	for (j1=1; j1<=n; j1++){
-	  S[i1][j1].r = S[i1][j1].r*ko[j1];
-	  S[i1][j1].i = S[i1][j1].i*ko[j1];
+#pragma omp parallel shared(BLAS_S,ko,S,n) private(OMPID,Nthrds,Nprocs,i1,j1)
+      { 
 
-          BLAS_S[(j1-1)*n+i1-1] = S[i1][j1];
+	/* get info. on OpenMP */ 
+
+	OMPID = omp_get_thread_num();
+	Nthrds = omp_get_num_threads();
+	Nprocs = omp_get_num_procs();
+
+	for (i1=1+OMPID; i1<=n; i1+=Nthrds){
+	  for (j1=1; j1<=n; j1++){
+
+	    S[i1][j1].r = S[i1][j1].r*ko[j1];
+	    S[i1][j1].i = S[i1][j1].i*ko[j1];
+	    BLAS_S[(j1-1)*n+i1-1] = S[i1][j1];
+	  } 
 	} 
-      } 
+
+      } /* #pragma omp parallel */
 
       /****************************************************
             1/sqrt(ko) * U^t * H * U * 1/sqrt(ko)
@@ -1868,10 +2049,10 @@ diagonalize1:
 	}
       }
       */
-        
+
       /* H * U * 1/sqrt(ko) */
 
-      /* 
+      /*
       for (j1=1; j1<=n; j1++){
 	for (i1=1; i1<=n; i1++){
 
@@ -1887,25 +2068,36 @@ diagonalize1:
 	  C[j1][i1].i = sumi;
 	}
       } 
-      */    
+      */
 
       /* note for BLAS, A[M*K] * B[K*N] = C[M*N] */
 
-      BM = n;
-      BN = n;
-      BK = n;
+#pragma omp parallel shared(BLAS_S,BLAS_H,BLAS_C,n) private(OMPID,Nthrds,Nprocs,Ctmp1,Ctmp2,BM,BN,BK)
+      { 
 
-      Ctmp1.r = 1.0;
-      Ctmp1.i = 0.0;
-      Ctmp2.r = 0.0;
-      Ctmp2.i = 0.0;
+	/* get info. on OpenMP */ 
 
-      F77_NAME(zgemm,ZGEMM)("N","N", &BM,&BN,&BK, 
-                            &Ctmp1, 
-                            BLAS_H, &BM,
-			    BLAS_S, &BK,
-                            &Ctmp2, 
-			    BLAS_C, &BM);
+	OMPID = omp_get_thread_num();
+	Nthrds = omp_get_num_threads();
+	Nprocs = omp_get_num_procs();
+
+	BM = n;
+	BN = (OMPID+1)*n/Nthrds - (OMPID*n/Nthrds+1) + 1;
+	BK = n;
+
+	Ctmp1.r = 1.0;
+	Ctmp1.i = 0.0;
+	Ctmp2.r = 0.0;
+	Ctmp2.i = 0.0;
+
+	F77_NAME(zgemm,ZGEMM)("N","N", &BM,&BN,&BK, 
+			      &Ctmp1, 
+			      BLAS_H, &BM,
+			      &BLAS_S[(OMPID*n/Nthrds)*n], &BK,
+			      &Ctmp2, 
+			      &BLAS_C[(OMPID*n/Nthrds)*n], &BM);
+
+      } /* #pragma omp parallel */
 
       /* 1/sqrt(ko) * U^+ H * U * 1/sqrt(ko) */
 
@@ -1926,27 +2118,38 @@ diagonalize1:
 
       /* note for BLAS, A[M*K] * B[K*N] = C[M*N] */
 
-      BM = n;
-      BN = n;
-      BK = n;
+#pragma omp parallel shared(C,BLAS_S,BLAS_H,BLAS_C,n) private(OMPID,Nthrds,Nprocs,Ctmp1,Ctmp2,BM,BN,BK,i1,j1)
+      { 
 
-      Ctmp1.r = 1.0;
-      Ctmp1.i = 0.0;
-      Ctmp2.r = 0.0;
-      Ctmp2.i = 0.0;
+	/* get info. on OpenMP */ 
 
-      F77_NAME(zgemm,ZGEMM)("C","N", &BM,&BN,&BK, 
-                            &Ctmp1,
-                            BLAS_S, &BM,
-			    BLAS_C, &BK, 
-                            &Ctmp2, 
-			    BLAS_H, &BM);
+	OMPID = omp_get_thread_num();
+	Nthrds = omp_get_num_threads();
+	Nprocs = omp_get_num_procs();
 
-      for (i1=1; i1<=n; i1++){
-        for (j1=1; j1<=n; j1++){
-          C[i1][j1] = BLAS_H[(j1-1)*n+i1-1];            
+	BM = n;
+	BN = (OMPID+1)*n/Nthrds - (OMPID*n/Nthrds+1) + 1;
+	BK = n;
+
+	Ctmp1.r = 1.0;
+	Ctmp1.i = 0.0;
+	Ctmp2.r = 0.0;
+	Ctmp2.i = 0.0;
+
+	F77_NAME(zgemm,ZGEMM)("C","N", &BM,&BN,&BK, 
+			      &Ctmp1,
+			      BLAS_S, &BM,
+			      &BLAS_C[(OMPID*n/Nthrds)*n], &BK, 
+			      &Ctmp2, 
+			      &BLAS_H[(OMPID*n/Nthrds)*n], &BM);
+
+	for (j1=(OMPID*n/Nthrds+1); j1<=(OMPID+1)*n/Nthrds; j1++){
+	  for (i1=1; i1<=n; i1++){
+	    C[i1][j1] = BLAS_H[(j1-1)*n+i1-1];            
+	  }
 	}
-      }
+
+      } /* #pragma omp parallel */
 
       /* H to C */
 
@@ -1984,7 +2187,7 @@ diagonalize1:
       dtime(&Stime);
 
       if (parallel_mode==0){
-	EigenBand_lapack(C,ko,n);
+	EigenBand_lapack(C,ko,n,1);
       }
       else{
 	/*  The C matrix is distributed by row */
@@ -2058,27 +2261,38 @@ diagonalize1:
 
       /* note for BLAS, A[M*K] * B[K*N] = C[M*N] */
 
-      BM = n;
-      BN = MaxN;
-      BK = n;
+#pragma omp parallel shared(H,BLAS_S,BLAS_H,BLAS_C,n,MaxN) private(OMPID,Nthrds,Nprocs,j1,i1,Ctmp1,Ctmp2,BM,BN,BK)
+      { 
 
-      Ctmp1.r = 1.0;
-      Ctmp1.i = 0.0;
-      Ctmp2.r = 0.0;
-      Ctmp2.i = 0.0;
+	/* get info. on OpenMP */ 
 
-      F77_NAME(zgemm,ZGEMM)("N","N", &BM,&BN,&BK, 
-                            &Ctmp1, 
-                            BLAS_S, &BM,
-			    BLAS_H, &BK,
-                            &Ctmp2, 
-			    BLAS_C, &BM);
+	OMPID = omp_get_thread_num();
+	Nthrds = omp_get_num_threads();
+	Nprocs = omp_get_num_procs();
 
-      for (j1=1; j1<=MaxN; j1++){
-        for (i1=1; i1<=n; i1++){
-          H[i1][j1] = BLAS_C[(j1-1)*n+i1-1];            
+	BM = n;
+	BN = (OMPID+1)*MaxN/Nthrds - (OMPID*MaxN/Nthrds+1) + 1;
+	BK = n;
+
+	Ctmp1.r = 1.0;
+	Ctmp1.i = 0.0;
+	Ctmp2.r = 0.0;
+	Ctmp2.i = 0.0;
+
+	F77_NAME(zgemm,ZGEMM)("N","N", &BM,&BN,&BK, 
+			      &Ctmp1, 
+			      BLAS_S, &BM,
+			      &BLAS_H[(OMPID*MaxN/Nthrds)*n], &BK,
+			      &Ctmp2, 
+			      &BLAS_C[(OMPID*MaxN/Nthrds)*n], &BM);
+
+	for (j1=(OMPID*MaxN/Nthrds+1); j1<=(OMPID+1)*MaxN/Nthrds; j1++){
+	  for (i1=1; i1<=n; i1++){
+	    H[i1][j1] = BLAS_C[(j1-1)*n+i1-1];            
+	  }
 	}
-      }
+
+      } /* #pragma omp parallel */
 
       /****************************************************
                    calculate DM and EDM
@@ -2117,40 +2331,72 @@ diagonalize1:
 
       if (po==0) lmax = MaxN;
 
-      /* DM and EDM */
+      /* predetermination of k 
+         H is used as temporal array 
+      */ 
 
       k = 0;
-
       for (AN=1; AN<=atomnum; AN++){
+
 	GA_AN = order_GA[AN];
 	wanA = WhatSpecies[GA_AN];
 	tnoA = Spe_Total_CNO[wanA];
-	Anum = MP[GA_AN];
+
+        H[0][AN].r = (double)k;
 
 	for (LB_AN=0; LB_AN<=FNAN[GA_AN]; LB_AN++){
 	  GB_AN = natn[GA_AN][LB_AN];
 	  Rn = ncn[GA_AN][LB_AN];
 	  wanB = WhatSpecies[GB_AN];
 	  tnoB = Spe_Total_CNO[wanB];
-	  Bnum = MP[GB_AN];
+          k += tnoA*tnoB;
+	}
+      }
 
-	  l1 = atv_ijk[Rn][1];
-	  l2 = atv_ijk[Rn][2];
-	  l3 = atv_ijk[Rn][3];
-	  kRn = k1*(double)l1 + k2*(double)l2 + k3*(double)l3;
+      /* DM and EDM */
 
-	  si = sin(2.0*PI*kRn);
-	  co = cos(2.0*PI*kRn);
+#pragma omp parallel shared(EDM1,CDM1,VecFkwE,VecFkw,H,lmax,k1,k2,k3,atv_ijk,ncn,natn,FNAN,MP,Spe_Total_CNO,WhatSpecies,order_GA,atomnum) private(OMPID,Nthrds,Nprocs,AN,GA_AN,wanA,tnoA,Anum,k,LB_AN,GB_AN,Rn,wanB,tnoB,Bnum,l1,l2,l3,kRn,si,co,i,ia,j,jb,d1,d2,l,tmp)
+      { 
 
-	  for (i=0; i<tnoA; i++){
+	/* get info. on OpenMP */ 
 
-	    ia = Anum + i; 
+	OMPID = omp_get_thread_num();
+	Nthrds = omp_get_num_threads();
+	Nprocs = omp_get_num_procs();
 
-	    for (j=0; j<tnoB; j++){
+	for (AN=1+OMPID; AN<=atomnum; AN+=Nthrds){
 
-              jb = Bnum + j;
+	  GA_AN = order_GA[AN];
+	  wanA = WhatSpecies[GA_AN];
+	  tnoA = Spe_Total_CNO[wanA];
+	  Anum = MP[GA_AN];
 
-	      /***************************************************************
+	  k = (int)H[0][AN].r;
+
+	  for (LB_AN=0; LB_AN<=FNAN[GA_AN]; LB_AN++){
+	    GB_AN = natn[GA_AN][LB_AN];
+	    Rn = ncn[GA_AN][LB_AN];
+	    wanB = WhatSpecies[GB_AN];
+	    tnoB = Spe_Total_CNO[wanB];
+	    Bnum = MP[GB_AN];
+
+	    l1 = atv_ijk[Rn][1];
+	    l2 = atv_ijk[Rn][2];
+	    l3 = atv_ijk[Rn][3];
+	    kRn = k1*(double)l1 + k2*(double)l2 + k3*(double)l3;
+
+	    si = sin(2.0*PI*kRn);
+	    co = cos(2.0*PI*kRn);
+
+	    for (i=0; i<tnoA; i++){
+
+	      ia = Anum + i; 
+
+	      for (j=0; j<tnoB; j++){
+
+		jb = Bnum + j;
+
+		/***************************************************************
                 Note that the imagiary part is zero, 
                  since
 
@@ -2160,32 +2406,34 @@ diagonalize1:
                  B = (co - i si)(Re - i Im) = (co*Re - si*Im) - i (co*Im + si*Re) 
                  Thus, Re(A+B) = 2*(co*Re - si*Im)
                        Im(A+B) = 0
-	      ***************************************************************/
+		***************************************************************/
 
-              d1 = 0.0;
-              d2 = 0.0;
+		d1 = 0.0;
+		d2 = 0.0;
 
-	      for (l=1; l<=lmax; l++){
+		for (l=1; l<=lmax; l++){
 
-                tmp = co*(H[ia][l].r*H[jb][l].r + H[ia][l].i*H[jb][l].i)
-		     -si*(H[ia][l].r*H[jb][l].i - H[ia][l].i*H[jb][l].r);
+		  tmp = co*(H[ia][l].r*H[jb][l].r + H[ia][l].i*H[jb][l].i)
+		       -si*(H[ia][l].r*H[jb][l].i - H[ia][l].i*H[jb][l].r);
 
-		d1 += VecFkw[l]*tmp;
-		d2 += VecFkwE[l]*tmp;;
+		  d1 += VecFkw[l]*tmp;
+		  d2 += VecFkwE[l]*tmp;;
+
+		}
+
+		CDM1[k] += d1; 
+		EDM1[k] += d2; 
+
+		/* increment of k */
+
+		k++;
 
 	      }
-
-              CDM1[k] += d1; 
-              EDM1[k] += d2; 
-
-	      /* increment of k */
-
-	      k++;
-
 	    }
 	  }
 	}
-      }
+
+      } /* #pragma omp parallel */
 
       dtime(&Etime);
       time11 += Etime - Stime;
@@ -2345,7 +2593,8 @@ diagonalize1:
     printf("<Band_DFT>  DM, time=%lf\n", EiloopTime-SiloopTime);fflush(stdout);
   }
 
-  dum = 1.0/(double)(knum_i*knum_j*knum_k);
+  dum = 1.0/sum_weights;
+
   for (spin=0; spin<=SpinP_switch; spin++){
     for (MA_AN=1; MA_AN<=Matomnum; MA_AN++){
       GA_AN = M2G[MA_AN];    
@@ -2396,6 +2645,18 @@ diagonalize1:
 	  }
 	}
       }
+
+      /*
+      printf("GA_AN=%2d LB_AN=%2d GB_AN=%2d RnB=%2d\n",GA_AN,LB_AN,GB_AN,ncn[GA_AN][j]);
+      for (k=0; k<tnoA; k++){
+	for (l=0; l<tnoB; l++){
+	  for (spin=0; spin<=SpinP_switch; spin++){
+	    printf("%15.12f ",CDM[spin][MA_AN][j][k][l]);
+	  }
+	}
+        printf("\n");
+      }
+      */
   
     }
   }
@@ -2949,7 +3210,7 @@ static double Band_collinear(
       }
 
       else{
-        EigenBand_lapack(S,ko[0],n);
+        EigenBand_lapack(S,ko[0],n,1);
       }
 
       if (3<=level_stdout){
@@ -3114,7 +3375,7 @@ static double Band_collinear(
 	}
 
 	else{
-	  EigenBand_lapack(C,ko[spin],n1);
+	  EigenBand_lapack(C,ko[spin],n1,1);
 	}
 
 	for (l=1; l<=n1; l++){
@@ -3191,7 +3452,7 @@ static double Band_collinear(
   loop_num = 0;
   ChemP_MAX = 10.0;  
   ChemP_MIN =-10.0;
-    
+
   do {
     loop_num++;
 
@@ -3357,7 +3618,7 @@ static double Band_collinear(
       }
 
       else{
-        EigenBand_lapack(S,ko[0],n);
+        EigenBand_lapack(S,ko[0],n,1);
       }
 
       if (3<=level_stdout && 0<=kloop){
@@ -3524,7 +3785,7 @@ static double Band_collinear(
 	}
 
 	else{
-	  EigenBand_lapack(C,ko[spin],n1);
+	  EigenBand_lapack(C,ko[spin],n1,1);
 	}
 
 	if (3<=level_stdout && 0<=kloop){
