@@ -188,16 +188,16 @@ static void Band_DFT_MO_Col(
 
     if (myid==Host_ID) printf("kpoint=%i /%i\n",kloop+1,nkpoint);
 
-    k1 = kpoint[kloop][1] + Shift_K_Point;
-    k2 = kpoint[kloop][2] - Shift_K_Point;
-    k3 = kpoint[kloop][3] + 2.0*Shift_K_Point;
+    k1 = kpoint[kloop][1];
+    k2 = kpoint[kloop][2];
+    k3 = kpoint[kloop][3];
 
     Overlap_Band(Host_ID,CntOLP,S,MP,k1,k2,k3);
 
     if (myid==Host_ID){
 
       n = S[0][0].r;
-      EigenBand_lapack(S,ko[0],n);
+      EigenBand_lapack(S,ko[0],n,1);
 
       if (3<=level_stdout){
         printf("  kloop %i, k1 k2 k3 %10.6f %10.6f %10.6f\n",
@@ -319,7 +319,7 @@ static void Band_DFT_MO_Col(
 	}
 
         n1 = n;
-        EigenBand_lapack(C[spin],ko[spin],n1);
+        EigenBand_lapack(C[spin],ko[spin],n1,1);
 
         for (i1=1; i1<=n1; i1++){
           EIGEN[kloop][spin][i1] = ko[spin][i1];
@@ -877,16 +877,16 @@ static void Band_DFT_MO_NonCol(
 
     printf("kpoint=%i /%i\n",kloop+1,nkpoint);
 
-    k1 = kpoint[kloop][1] + Shift_K_Point;
-    k2 = kpoint[kloop][2] - Shift_K_Point;
-    k3 = kpoint[kloop][3] + 2.0*Shift_K_Point;
+    k1 = kpoint[kloop][1];
+    k2 = kpoint[kloop][2];
+    k3 = kpoint[kloop][3];
 
     Overlap_Band(Host_ID,CntOLP, S, MP, k1, k2, k3);
 
     if (myid==Host_ID){
 
       n = S[0][0].r;
-      EigenBand_lapack(S,ko,n);
+      EigenBand_lapack(S,ko,n,1);
 
       if (2<=level_stdout){
         printf("  kloop %i, k1 k2 k3 %10.6f %10.6f %10.6f\n",
@@ -1019,7 +1019,7 @@ static void Band_DFT_MO_NonCol(
       /* solve eigenvalue problem */
 
       n1 = 2*n;
-      EigenBand_lapack(C, ko, n1);
+      EigenBand_lapack(C, ko, n1,1);
 
       for (i1=1; i1<=2*n; i1++){
         for (j1=1; j1<=n1; j1++){
@@ -1171,166 +1171,166 @@ static void Band_DFT_MO_NonCol(
     Bulk_Num_HOMOs[kloop] = nhomos;
     Bulk_Num_LUMOs[kloop] = nlumos;
 
-  }  /* kloop */
-
-  /****************************************************
+    /****************************************************
                         Output
-  ****************************************************/
+    ****************************************************/
 
-  if (myid==Host_ID){
+    if (myid==Host_ID){
 
-    strcpy(file_EV,".EV");
-    fnjoint(filepath,filename,file_EV);
+      strcpy(file_EV,".EV");
+      fnjoint(filepath,filename,file_EV);
 
-    if ((fp_EV = fopen(file_EV,"a")) != NULL){
+      if ((fp_EV = fopen(file_EV,"a")) != NULL){
 
 #ifdef xt3
-      setvbuf(fp_EV,buf,_IOFBF,fp_bsize);  /* setvbuf */
+	setvbuf(fp_EV,buf,_IOFBF,fp_bsize);  /* setvbuf */
 #endif
 
-      fprintf(fp_EV,"\n");
-      fprintf(fp_EV,"***********************************************************\n");
-      fprintf(fp_EV,"***********************************************************\n");
-      fprintf(fp_EV,"        Eigenvalues (Hartree) and LCAO coefficients        \n");
-      fprintf(fp_EV,"        at the k-points specified in the input file.       \n");
-      fprintf(fp_EV,"***********************************************************\n");
-      fprintf(fp_EV,"***********************************************************\n");
+	fprintf(fp_EV,"\n");
+	fprintf(fp_EV,"***********************************************************\n");
+	fprintf(fp_EV,"***********************************************************\n");
+	fprintf(fp_EV,"        Eigenvalues (Hartree) and LCAO coefficients        \n");
+	fprintf(fp_EV,"        at the k-points specified in the input file.       \n");
+	fprintf(fp_EV,"***********************************************************\n");
+	fprintf(fp_EV,"***********************************************************\n");
 
-      for (kloop=0; kloop<nkpoint; kloop++){
-        k1 = kpoint[kloop][1];
-        k2 = kpoint[kloop][2];
-        k3 = kpoint[kloop][3];
-        fprintf(fp_EV,"\n\n");
-        fprintf(fp_EV,"   # of k-point = %i\n",kloop);
-        fprintf(fp_EV,"   k1=%10.5f k2=%10.5f k3=%10.5f\n\n",k1,k2,k3);
-        fprintf(fp_EV,"   Chemical Potential (Hartree) = %18.14f\n",ChemP);
-        fprintf(fp_EV,"   HOMO = %i\n\n",Bulk_HOMO[kloop][0]);
+	for (kloop=0; kloop<nkpoint; kloop++){
+	  k1 = kpoint[kloop][1];
+	  k2 = kpoint[kloop][2];
+	  k3 = kpoint[kloop][3];
+	  fprintf(fp_EV,"\n\n");
+	  fprintf(fp_EV,"   # of k-point = %i\n",kloop);
+	  fprintf(fp_EV,"   k1=%10.5f k2=%10.5f k3=%10.5f\n\n",k1,k2,k3);
+	  fprintf(fp_EV,"   Chemical Potential (Hartree) = %18.14f\n",ChemP);
+	  fprintf(fp_EV,"   HOMO = %i\n\n",Bulk_HOMO[kloop][0]);
 
-	fprintf(fp_EV,"   Real (Re) and imaginary (Im) parts of LCAO coefficients\n\n");
+	  fprintf(fp_EV,"   Real (Re) and imaginary (Im) parts of LCAO coefficients\n\n");
 
-	num0 = 2;
-	num1 = 2*n/num0 + 1*((2*n)%num0!=0);
+	  num0 = 2;
+	  num1 = 2*n/num0 + 1*((2*n)%num0!=0);
   
-	for (i=1; i<=num1; i++){
+	  for (i=1; i<=num1; i++){
 
-	  fprintf(fp_EV,"\n");
-
-	  for (i1=-2; i1<=0; i1++){
-
-	    fprintf(fp_EV,"                     ");
-
-	    for (j=1; j<=num0; j++){
-
-	      j1 = num0*(i-1) + j;
-
-	      if (j1<=2*n){ 
-
-		if (i1==-2){
- 	          fprintf(fp_EV," %4d",j1);
-		  fprintf(fp_EV,"                                   ");
-		}
-
-		else if (i1==-1){
-		  fprintf(fp_EV,"   %8.5f",EIGEN[kloop][j1]);
-		  fprintf(fp_EV,"                             ");
-		}
-
-		else if (i1==0){
-		  fprintf(fp_EV,"     Re(U)");
-		  fprintf(fp_EV,"     Im(U)");
-		  fprintf(fp_EV,"     Re(D)");
-		  fprintf(fp_EV,"     Im(D)");
-		}
-	      }
-	    }
 	    fprintf(fp_EV,"\n");
-	    if (i1==-1)  fprintf(fp_EV,"\n");
-	    if (i1==0)   fprintf(fp_EV,"\n");
-	  }
 
-	  Name_Angular[0][0] = "s          ";
-	  Name_Angular[1][0] = "px         ";
-	  Name_Angular[1][1] = "py         ";
-	  Name_Angular[1][2] = "pz         ";
-	  Name_Angular[2][0] = "d3z^2-r^2  ";
-	  Name_Angular[2][1] = "dx^2-y^2   ";
-	  Name_Angular[2][2] = "dxy        ";
-	  Name_Angular[2][3] = "dxz        ";
-	  Name_Angular[2][4] = "dyz        ";
-	  Name_Angular[3][0] = "f5z^2-3r^2 ";
-	  Name_Angular[3][1] = "f5xz^2-xr^2";
-	  Name_Angular[3][2] = "f5yz^2-yr^2";
-	  Name_Angular[3][3] = "fzx^2-zy^2 ";
-	  Name_Angular[3][4] = "fxyz       ";
-	  Name_Angular[3][5] = "fx^3-3*xy^2";
-	  Name_Angular[3][6] = "f3yx^2-y^3 ";
-	  Name_Angular[4][0] = "g1         ";
-	  Name_Angular[4][1] = "g2         ";
-	  Name_Angular[4][2] = "g3         ";
-	  Name_Angular[4][3] = "g4         ";
-	  Name_Angular[4][4] = "g5         ";
-	  Name_Angular[4][5] = "g6         ";
-	  Name_Angular[4][6] = "g7         ";
-	  Name_Angular[4][7] = "g8         ";
-	  Name_Angular[4][8] = "g9         ";
+	    for (i1=-2; i1<=0; i1++){
 
-	  Name_Multiple[0] = "0";
-	  Name_Multiple[1] = "1";
-	  Name_Multiple[2] = "2";
-	  Name_Multiple[3] = "3";
-	  Name_Multiple[4] = "4";
-	  Name_Multiple[5] = "5";
+	      fprintf(fp_EV,"                     ");
 
-	  i1 = 1; 
+	      for (j=1; j<=num0; j++){
 
-	  for (Gc_AN=1; Gc_AN<=atomnum; Gc_AN++){
+		j1 = num0*(i-1) + j;
 
-	    wan1 = WhatSpecies[Gc_AN];
-            
-	    for (l=0; l<=Supported_MaxL; l++){
-	      for (mul=0; mul<Spe_Num_CBasis[wan1][l]; mul++){
-		for (m=0; m<(2*l+1); m++){
+		if (j1<=2*n){ 
 
-		  if (l==0 && mul==0 && m==0)
-		    fprintf(fp_EV,"%4d %3s %s %s", 
-			    Gc_AN,SpeName[wan1],Name_Multiple[mul],Name_Angular[l][m]);
-		  else
-		    fprintf(fp_EV,"         %s %s", 
-			    Name_Multiple[mul],Name_Angular[l][m]);
-
-		  for (j=1; j<=num0; j++){
-
-		    j1 = num0*(i-1) + j;
-
-		    if (0<i1 && j1<=2*n){
-		      fprintf(fp_EV,"  %8.5f",C[i1][j1].r);
-		      fprintf(fp_EV,"  %8.5f",C[i1][j1].i);
-		      fprintf(fp_EV,"  %8.5f",C[i1+n][j1].r);
-		      fprintf(fp_EV,"  %8.5f",C[i1+n][j1].i);
-		    }
+		  if (i1==-2){
+		    fprintf(fp_EV," %4d",j1);
+		    fprintf(fp_EV,"                                   ");
 		  }
 
-		  fprintf(fp_EV,"\n");
-		  if (i1==-1)  fprintf(fp_EV,"\n");
-		  if (i1==0)   fprintf(fp_EV,"\n");
+		  else if (i1==-1){
+		    fprintf(fp_EV,"   %8.5f",EIGEN[kloop][j1]);
+		    fprintf(fp_EV,"                             ");
+		  }
 
-		  i1++;
+		  else if (i1==0){
+		    fprintf(fp_EV,"     Re(U)");
+		    fprintf(fp_EV,"     Im(U)");
+		    fprintf(fp_EV,"     Re(D)");
+		    fprintf(fp_EV,"     Im(D)");
+		  }
+		}
+	      }
+	      fprintf(fp_EV,"\n");
+	      if (i1==-1)  fprintf(fp_EV,"\n");
+	      if (i1==0)   fprintf(fp_EV,"\n");
+	    }
+
+	    Name_Angular[0][0] = "s          ";
+	    Name_Angular[1][0] = "px         ";
+	    Name_Angular[1][1] = "py         ";
+	    Name_Angular[1][2] = "pz         ";
+	    Name_Angular[2][0] = "d3z^2-r^2  ";
+	    Name_Angular[2][1] = "dx^2-y^2   ";
+	    Name_Angular[2][2] = "dxy        ";
+	    Name_Angular[2][3] = "dxz        ";
+	    Name_Angular[2][4] = "dyz        ";
+	    Name_Angular[3][0] = "f5z^2-3r^2 ";
+	    Name_Angular[3][1] = "f5xz^2-xr^2";
+	    Name_Angular[3][2] = "f5yz^2-yr^2";
+	    Name_Angular[3][3] = "fzx^2-zy^2 ";
+	    Name_Angular[3][4] = "fxyz       ";
+	    Name_Angular[3][5] = "fx^3-3*xy^2";
+	    Name_Angular[3][6] = "f3yx^2-y^3 ";
+	    Name_Angular[4][0] = "g1         ";
+	    Name_Angular[4][1] = "g2         ";
+	    Name_Angular[4][2] = "g3         ";
+	    Name_Angular[4][3] = "g4         ";
+	    Name_Angular[4][4] = "g5         ";
+	    Name_Angular[4][5] = "g6         ";
+	    Name_Angular[4][6] = "g7         ";
+	    Name_Angular[4][7] = "g8         ";
+	    Name_Angular[4][8] = "g9         ";
+
+	    Name_Multiple[0] = "0";
+	    Name_Multiple[1] = "1";
+	    Name_Multiple[2] = "2";
+	    Name_Multiple[3] = "3";
+	    Name_Multiple[4] = "4";
+	    Name_Multiple[5] = "5";
+
+	    i1 = 1; 
+
+	    for (Gc_AN=1; Gc_AN<=atomnum; Gc_AN++){
+
+	      wan1 = WhatSpecies[Gc_AN];
+            
+	      for (l=0; l<=Supported_MaxL; l++){
+		for (mul=0; mul<Spe_Num_CBasis[wan1][l]; mul++){
+		  for (m=0; m<(2*l+1); m++){
+
+		    if (l==0 && mul==0 && m==0)
+		      fprintf(fp_EV,"%4d %3s %s %s", 
+			      Gc_AN,SpeName[wan1],Name_Multiple[mul],Name_Angular[l][m]);
+		    else
+		      fprintf(fp_EV,"         %s %s", 
+			      Name_Multiple[mul],Name_Angular[l][m]);
+
+		    for (j=1; j<=num0; j++){
+
+		      j1 = num0*(i-1) + j;
+
+		      if (0<i1 && j1<=2*n){
+			fprintf(fp_EV,"  %8.5f",C[i1][j1].r);
+			fprintf(fp_EV,"  %8.5f",C[i1][j1].i);
+			fprintf(fp_EV,"  %8.5f",C[i1+n][j1].r);
+			fprintf(fp_EV,"  %8.5f",C[i1+n][j1].i);
+		      }
+		    }
+
+		    fprintf(fp_EV,"\n");
+		    if (i1==-1)  fprintf(fp_EV,"\n");
+		    if (i1==0)   fprintf(fp_EV,"\n");
+
+		    i1++;
+		  }
 		}
 	      }
 	    }
+
 	  }
 
 	}
 
+	fclose(fp_EV);
       }
+      else{
+	printf("Failure of saving the EV file.\n");
+	fclose(fp_EV);
+      }
+    } /* if (myid==Host_ID) */
 
-      fclose(fp_EV);
-    }
-    else{
-      printf("Failure of saving the EV file.\n");
-      fclose(fp_EV);
-    }
-  } /* if (myid==Host_ID) */
+  }  /* kloop */
 
   /****************************************************
                        free arrays
