@@ -324,7 +324,7 @@ void Band_DFT_kpath_Col( int nkpath, int *n_perk,
 
     if (0<=kloop) {
 
-      EigenBand_lapack(S,ko[0],n);
+      EigenBand_lapack(S,ko[0],n,1);
 
       if (3<=level_stdout){
 	printf(" myid=%2d kloop %2d  k1 k2 k3 %10.6f %10.6f %10.6f\n",
@@ -481,7 +481,7 @@ void Band_DFT_kpath_Col( int nkpath, int *n_perk,
         /* solve eigenvalue problem */
 
 	n1 = n;
-        EigenBand_lapack(C,ko[spin],n1);
+        EigenBand_lapack(C,ko[spin],n1,0);
 
 	for (l=1; l<=n1; l++){
 	  EigenVal[ik][i_perk][spin][l-1] = (float)ko[spin][l];
@@ -916,7 +916,7 @@ void Band_DFT_kpath_NonCol( int nkpath, int *n_perk,
 
     if (0<=kloop){
 
-      EigenBand_lapack(S,ko,n);
+      EigenBand_lapack(S,ko,n,1);
 
       /* minus eigenvalues to 1.0e-14 */
       for (l=1; l<=n; l++){
@@ -1097,7 +1097,7 @@ void Band_DFT_kpath_NonCol( int nkpath, int *n_perk,
       /* solve eigenvalue problem */
 
       n1 = 2*n;
-      EigenBand_lapack(C, ko, n1);
+      EigenBand_lapack(C, ko, n1,0);
 
       for (l=1; l<=n1; l++){
         EigenVal[ik][i_perk][l-1] = (float)ko[l];
