@@ -307,6 +307,7 @@ double Cluster_Col_Dosout( int SpinP_switch,  double *****nh, double ****CntOLP)
     }
 
     Eigen_lapack(S,ko[0],n,n);
+
     /* S[i][j] contains jth eigenvector, not ith ! */
 
     /****************************************************
@@ -502,7 +503,7 @@ double Cluster_Col_Dosout( int SpinP_switch,  double *****nh, double ****CntOLP)
       if (iemax<iemax0) iemax = iemax0;
 
       /****************************************************
-          Transformation to the original eigen vectors.
+          Transformation to the original eigenvectors.
                         AIST NOTE 244P
       ****************************************************/
 
@@ -647,6 +648,7 @@ double Cluster_Col_Dosout( int SpinP_switch,  double *****nh, double ****CntOLP)
 
   for (spin=0; spin<=SpinP_switch; spin++){
     for (k=iemin; k<=iemax; k++){
+
       for (MA_AN=1; MA_AN<=Matomnum; MA_AN++){
         GA_AN = M2G[MA_AN];
 	wanA = WhatSpecies[GA_AN];
@@ -1145,7 +1147,7 @@ double Cluster_NonCol_Dosout( int SpinP_switch,
     /* solve eigenvalue problem */
 
     n1 = 2*n;
-    EigenBand_lapack(C,ko,n1);
+    EigenBand_lapack(C,ko,n1,1);
 
     for (i1=1; i1<=n1; i1++){
       for (j1=1; j1<=n1; j1++){
