@@ -22,31 +22,33 @@
 #include "mpi.h"
 #endif
 
-#ifdef TRAN
 #include "tran_prototypes.h"
-#endif
-
+ 
  
 static void Output_Energies_Forces(FILE *fp);
-
 static double dUele;
 
+
+
+
 double DFT(int MD_iter, int Cnt_Now)
 {
   static int firsttime=1;
   double ECE[15],Eele0[2],Eele1[2];
   double pUele,ChemP_e0[2];
+  double Norm1,Norm2,Norm3,Norm4,Norm5;
   double S_coordinate[3];
-  double tmp;
+  double tmp,tmp0;
   int Cnt_kind,Calc_CntOrbital_ON,spin,spinmax,m;
-  int SCF_iter,SCF_MAX,i,j,k,fft_charge_flag,MN;
+  int SCF_iter,SCF_iter_shift,SCF_MAX;
+  int i,j,k,fft_charge_flag,M1N;
   int orbitalOpt_iter,LSCF_iter,OrbOpt_end; 
-  int n,n2,wanA;
+  int n,n2,wanA,ETemp_controller;
   double time0,time1,time2,time3,time4;
   double time5,time6,time7,time8,time9;
   double time10,time11,time12;
   double x,y,z;
-  int po;
+  int po,TRAN_Poisson_flag2;
   int  SucceedReadingDMfile,My_SucceedReadingDMfile;
   char file_DFTSCF[YOUSO10] = ".DFTSCF";
   char file_OrbOpt[YOUSO10] = ".OrbOpt";
@@ -248,6 +250,9 @@ double DFT(int MD_iter, int Cnt_Now)
       }
     }
 
+    /* Monkhorst-Pack k-points */ 
+    if (way_of_kpoint==2) T_knum = num_non_eq_kpt; 
+
     T_KGrids1 = (double*)malloc(sizeof(double)*T_knum);
     T_KGrids2 = (double*)malloc(sizeof(double)*T_knum);
     T_KGrids3 = (double*)malloc(sizeof(double)*T_knum);
@@ -408,6 +413,7 @@ double DFT(int MD_iter, int Cnt_Now)
       ReRhoAtomk[i] = (double**)malloc(sizeof(double*)*Ngrid1); 
       for (j=0; j<Ngrid1; j++){
 	ReRhoAtomk[i][j] = (double*)malloc(sizeof(double)*Ngrid3); 
+        for (k=0; k<Ngrid3; k++) ReRhoAtomk[i][j][k] = 0.0;
       }
     }
 
@@ -416,6 +422,7 @@ double DFT(int MD_iter, int Cnt_Now)
       ImRhoAtomk[i] = (double**)malloc(sizeof(double*)*Ngrid1); 
       for (j=0; j<Ngrid1; j++){
 	ImRhoAtomk[i][j] = (double*)malloc(sizeof(double)*Ngrid3); 
+        for (k=0; k<Ngrid3; k++) ImRhoAtomk[i][j][k] = 0.0;
       }
     }
 
@@ -559,6 +566,7 @@ double DFT(int MD_iter, int Cnt_Now)
   for (i=0; i<15; i++) ECE[i] = 0.0;
 
   fft_charge_flag = 0;
+  TRAN_Poisson_flag2 = 0;
 
   /****************************************************
          Calculations of overlap and Hamiltonian
@@ -574,9 +582,7 @@ double DFT(int MD_iter, int Cnt_Now)
   if (ProExpn_VNA==1){
     if (myid0==Host_ID) printf("<MD=%2d>  Calculation of the VNA projector matrix\n",MD_iter);fflush(stdout);
     time12 = Set_ProExpn_VNA(HVNA, HVNA2, DS_VNA);  
-    /*
-    printf("time12=%10.5f\n",time12);fflush(stdout);
-    */
+    /* printf("time12=%10.5f\n",time12);fflush(stdout); */
   }
 
   /* SCF loop */
@@ -593,28 +599,18 @@ double DFT(int MD_iter, int Cnt_Now)
   OrbOpt_end = 0;
   SCF_iter = 0;
   LSCF_iter = 0;
+  ETemp_controller = 0;
+  SCF_iter_shift = 0;
+  NormRD[0] = 100.0;
+
   SCF_RENZOKU = -1;
   po = 0;
-  pUele = 10.0;
+  pUele  = 100.0;
+  Norm1 = 100.0;
+  Norm2 = 100.0;
+  Norm3 = 100.0;
+  Norm4 = 100.0;
 
-  /*****************************************************
-                    set grid for NEGF
-  *****************************************************/
-
-#ifdef TRAN
-   if (Solver==4) { /* transport */
-
-     TRAN_Set_Electrode_Grid(
-            mpi_comm_level1, 
-            Grid_Origin, tv, Left_tv, Right_tv, gtv,
-            Ngrid1, Ngrid2, Ngrid3,
-            Density_Grid[0]); /* work */
-
-     TRAN_Allocate_Lead_Region(mpi_comm_level1); 
-     TRAN_Allocate_Cregion(mpi_comm_level1, SpinP_switch,atomnum,WhatSpecies,Spe_Total_CNO);
-   }
-#endif
-  
   /*****************************************************
               read contraction coefficients
   *****************************************************/
@@ -624,7 +620,7 @@ double DFT(int MD_iter, int Cnt_Now)
   /****************************************************
             Start self consistent calculation
   ****************************************************/
-  
+
   do {
     
     SCF_iter++;
@@ -662,16 +658,35 @@ double DFT(int MD_iter, int Cnt_Now)
       else if (Cnt_switch==1)                             Cnt_kind = 1;
       else                                                Cnt_kind = 0;
 
-      time9   = Set_Aden_Grid(1);
+      time9 = Set_Aden_Grid(1);
 
-#ifdef TRAN
       if (Solver==4) {
-	/* output of Set_Density_Grid => Density_Grid */
-	TRAN_Overwrite_Densitygrid(mpi_comm_level1,
-				   SpinP_switch, Ngrid1, Ngrid2, Ngrid3,
-				   Num_Cells0, My_Cell0,My_Cell1, Density_Grid); 
+
+        /*****************************************************
+                          set grid for NEGF
+	*****************************************************/
+
+	TRAN_Set_Electrode_Grid(mpi_comm_level1, 
+                                &TRAN_Poisson_flag2,                                
+				Grid_Origin, tv, Left_tv, Right_tv, gtv,
+				Ngrid1, Ngrid2, Ngrid3,
+				Density_Grid[0]); /* work */
+     
+	TRAN_Allocate_Lead_Region(mpi_comm_level1); 
+	TRAN_Allocate_Cregion(mpi_comm_level1, SpinP_switch,atomnum,WhatSpecies,Spe_Total_CNO);
+
+        /*****************************************************
+                         add density from Leads
+ 	*****************************************************/
+
+	TRAN_Add_Density_Lead(mpi_comm_level1,
+			      SpinP_switch, Ngrid1, Ngrid2, Ngrid3,
+			      Num_Cells0, My_Cell0,My_Cell1, Density_Grid); 
+
+	TRAN_Add_ADensity_Lead(mpi_comm_level1,
+ 	 		       SpinP_switch, Ngrid1, Ngrid2, Ngrid3,
+			       Num_Cells0, My_Cell0,My_Cell1, ADensity_Grid); 
       }
-#endif
 
       time10 += Set_Orbitals_Grid(Cnt_kind);
 
@@ -684,6 +699,7 @@ double DFT(int MD_iter, int Cnt_Now)
 	  ((Cnt_switch==1 && Cnt_Now!=1) || Cnt_switch==0 ) ) {
 
 	My_SucceedReadingDMfile = RestartFileDFT("read",MD_iter,&Uele,H,CntH);
+
         MPI_Barrier(mpi_comm_level1);
 	MPI_Allreduce(&My_SucceedReadingDMfile, &SucceedReadingDMfile,
 		      1, MPI_INT, MPI_PROD, mpi_comm_level1);
@@ -696,6 +712,23 @@ double DFT(int MD_iter, int Cnt_Now)
             printf("<Restart>  Could not find restart files\n"); fflush(stdout);
 	  }
 	}
+
+        /***********************************************************************
+         If reading the restart files is terminated, the densities on grid are 
+         partially overwriten. So, Set_Aden_Grid(1) is called once more.  
+        ***********************************************************************/
+
+        if (SucceedReadingDMfile==0){
+
+          time9 += Set_Aden_Grid(1);
+
+          if (Solver==4) {
+	    TRAN_Add_ADensity_Lead(mpi_comm_level1,
+				   SpinP_switch, Ngrid1, Ngrid2, Ngrid3,
+				   Num_Cells0, My_Cell0,My_Cell1, ADensity_Grid); 
+	  }
+        } 
+
       }
 
       /*****************************************************
@@ -706,41 +739,61 @@ double DFT(int MD_iter, int Cnt_Now)
 
         /* non-spin polarization */
 	if (SpinP_switch==0){
-	  FFT_Density(1,ReV1,ImV1,ReRhok[1][0],ImRhok[1][0]);
+          if (Solver!=4 || TRAN_Poisson_flag2==4) FFT_Density(1,ReV1,ImV1,ReRhok[1][0],ImRhok[1][0]);
+          else                                    FFT2D_Density(1,ReV1,ImV1,ReRhok[1][0],ImRhok[1][0]);
 	}
 
 	/* collinear spin polarization */
 	else if (SpinP_switch==1) {
-	  FFT_Density(1,ReV1,ImV1,ReRhok[1][0],ImRhok[1][0]);
-	  FFT_Density(2,ReV1,ImV1,ReRhok[1][1],ImRhok[1][1]);
+	  if (Solver!=4 || TRAN_Poisson_flag2==4){
+            FFT_Density(1,ReV1,ImV1,ReRhok[1][0],ImRhok[1][0]);
+            FFT_Density(2,ReV1,ImV1,ReRhok[1][1],ImRhok[1][1]);
+	  }
+          else{
+            FFT2D_Density(1,ReV1,ImV1,ReRhok[1][0],ImRhok[1][0]);
+            FFT2D_Density(2,ReV1,ImV1,ReRhok[1][1],ImRhok[1][1]);
+          }
 	}
 
 	/* non-collinear spin polarization */
 	else if (SpinP_switch==3) {
-	  FFT_Density(1,ReV1,ImV1,ReRhok[1][0],ImRhok[1][0]);
-	  FFT_Density(2,ReV1,ImV1,ReRhok[1][1],ImRhok[1][1]);
-	  FFT_Density(4,ReV1,ImV1,ReRhok[1][2],ImRhok[1][2]);
+          if (Solver!=4 || TRAN_Poisson_flag2==4){
+  	    FFT_Density(1,ReV1,ImV1,ReRhok[1][0],ImRhok[1][0]);
+	    FFT_Density(2,ReV1,ImV1,ReRhok[1][1],ImRhok[1][1]);
+	    FFT_Density(4,ReV1,ImV1,ReRhok[1][2],ImRhok[1][2]);
+	  }
+          else{
+  	    FFT2D_Density(1,ReV1,ImV1,ReRhok[1][0],ImRhok[1][0]);
+	    FFT2D_Density(2,ReV1,ImV1,ReRhok[1][1],ImRhok[1][1]);
+	    FFT2D_Density(4,ReV1,ImV1,ReRhok[1][2],ImRhok[1][2]);
+          }
 	}
       }
 
       if (SpinP_switch==3) diagonalize_nc_density();
 
-      /* In case the restart file is not found */
+      /* In case the restart file is found */
 
       if (SucceedReadingDMfile && Cnt_switch==0) {
 
-#ifdef TRAN
-	if (Solver!=4) time4  += Poisson(1,ReV1,ImV1,ReV2,ImV2);
-	else           time4  += TRAN_Poisson(1,ReV1,ImV1,ReV2,ImV2); 
-#else
-	time4  += Poisson(1,ReV1,ImV1,ReV2,ImV2);
-#endif
-      }
+	if (Solver!=4 && ESM_switch==0)      time4  += Poisson(1,ReV1,ImV1,ReV2,ImV2);
+	else if (Solver!=4 && ESM_switch!=0) time4  += Poisson_ESM(1,ReV1,ImV1,ReV2,ImV2); /* added by Ohwaki */
+	else                                 time4  += TRAN_Poisson(ReV1,ImV1,ReV2,ImV2); 
 
-      else {
+        if (Correct_Position_flag)
         time3 += Set_Hamiltonian("nostdout",SCF_iter,SucceedReadingDMfile,Cnt_kind,H0,HNL,DM[0],H);
-      }
 
+      }
+      else{ 
+	/* 
+        Originally, TRAN_Poisson is called, but we modified so that Band_DFT is called 
+        in the beginning of SCF calculations, we eliminate the following function call: 
+        if (Solver==4) time4  += TRAN_Poisson(ReV1,ImV1,ReV2,ImV2); 
+        */
+
+        if (Solver==4) time4  += TRAN_Poisson(ReV1,ImV1,ReV2,ImV2); 
+        time3 += Set_Hamiltonian("nostdout",SCF_iter,SucceedReadingDMfile,Cnt_kind,H0,HNL,DM[0],H);
+      }  
 
       if (Cnt_switch==1 && Cnt_Now==1){
         if (MD_iter==1) Initial_CntCoes(H,OLP);
@@ -763,12 +816,9 @@ double DFT(int MD_iter, int Cnt_Now)
       else if (Cnt_switch==1)                             Cnt_kind = 1;
       else                                                Cnt_kind = 0;
 
-#ifdef TRAN
-      if (Solver!=4) time4  += Poisson(fft_charge_flag,ReV1,ImV1,ReV2,ImV2);
-      else           time4  += TRAN_Poisson(fft_charge_flag,ReV1,ImV1,ReV2,ImV2); 
-#else
-      time4  += Poisson(fft_charge_flag,ReV1,ImV1,ReV2,ImV2);
-#endif
+      if (Solver!=4 && ESM_switch==0)       time4 += Poisson(fft_charge_flag,ReV1,ImV1,ReV2,ImV2);
+      else if (Solver!=4 && ESM_switch!=0)  time4 += Poisson_ESM(fft_charge_flag,ReV1,ImV1,ReV2,ImV2); /* added by Ohwaki */
+      else                                  time4 += TRAN_Poisson(ReV1,ImV1,ReV2,ImV2); 
 
       /* construct matrix elements for LDA+U or Zeeman term */
 
@@ -796,7 +846,7 @@ double DFT(int MD_iter, int Cnt_Now)
     s_vec[3]="NEGF";      s_vec[4]="DC";      s_vec[5]="GDC";
     s_vec[6]="Cluster2";  s_vec[7]="Krylov";
 
-    if (myid0==Host_ID) printf("<%s>  Eigenvalue problem...\n",s_vec[Solver-1]);fflush(stdout);
+    if (myid0==Host_ID) printf("<%s>  Solving the eigenvalue problem...\n",s_vec[Solver-1]);fflush(stdout);
 
     if (Cnt_switch==0){
 
@@ -809,8 +859,10 @@ double DFT(int MD_iter, int Cnt_Now)
         break;
 
       case 2:
-        time5 += Cluster_DFT(LSCF_iter,SpinP_switch,Cluster_ReCoes,Cluster_ko,
+
+        time5 += Cluster_DFT("scf",LSCF_iter,SpinP_switch,Cluster_ReCoes,Cluster_ko,
                              H,iHNL,OLP[0],DM[0],EDM,Eele0,Eele1);
+
 	break;
 
       case 3:
@@ -846,29 +898,26 @@ double DFT(int MD_iter, int Cnt_Now)
         break;
 
       case 4:
-#ifdef TRAN
-        time5 += TRAN_DFT(mpi_comm_level1, level_stdout, LSCF_iter,SpinP_switch,H,iHNL,OLP[0],
+        time5 += TRAN_DFT(mpi_comm_level1, SucceedReadingDMfile, 
+                          level_stdout, LSCF_iter,SpinP_switch,H,iHNL,OLP[0],
                           atomnum,Matomnum,WhatSpecies, Spe_Total_CNO, FNAN, natn,ncn,
                           M2G, G2ID, F_G2M, atv_ijk, List_YOUSO,
                           DM[0],EDM,TRAN_DecMulP,Eele0,Eele1,ChemP_e0);
-#endif
         break;
 
       case 5:    
-        time5 += Divide_Conquer(LSCF_iter,H,iHNL,OLP[0],DM[0],EDM,Eele0,Eele1);
+        time5 += Divide_Conquer("scf",LSCF_iter,H,iHNL,OLP[0],DM[0],EDM,Eele0,Eele1);
         break;
 
       case 6:    
-
         time5 += GDivide_Conquer(LSCF_iter,H,iHNL,OLP[0],DM[0],EDM,Eele0,Eele1);
-
         break;       
 
       case 7:    
         break;       
 
       case 8:
-        time5 += Krylov(LSCF_iter,H,iHNL,OLP[0],DM[0],EDM,Eele0,Eele1);
+        time5 += Krylov("scf",LSCF_iter,H,iHNL,OLP[0],DM[0],EDM,Eele0,Eele1);
         break;       
 
       }
@@ -885,7 +934,7 @@ double DFT(int MD_iter, int Cnt_Now)
         break;
 
       case 2:
-        time5 += Cluster_DFT(LSCF_iter,SpinP_switch,Cluster_ReCoes,Cluster_ko,
+        time5 += Cluster_DFT("scf",LSCF_iter,SpinP_switch,Cluster_ReCoes,Cluster_ko,
                              CntH,iCntHNL,CntOLP[0],DM[0],EDM,Eele0,Eele1);
 	break;
 
@@ -922,16 +971,15 @@ double DFT(int MD_iter, int Cnt_Now)
         break;
 
       case 4:
-#ifdef TRAN
-        time5 += TRAN_DFT(mpi_comm_level1, level_stdout, LSCF_iter,SpinP_switch,H,iHNL,OLP[0],
+        time5 += TRAN_DFT(mpi_comm_level1, SucceedReadingDMfile, 
+                          level_stdout, LSCF_iter,SpinP_switch,H,iHNL,OLP[0],
                           atomnum,Matomnum,WhatSpecies, Spe_Total_CNO, FNAN, natn,ncn,
                           M2G, G2ID, F_G2M, atv_ijk, List_YOUSO,
                           DM[0],EDM,TRAN_DecMulP,Eele0,Eele1,ChemP_e0);
-#endif
         break;
 
       case 5:    
-        time5 += Divide_Conquer(LSCF_iter,CntH,iCntHNL,CntOLP[0],DM[0],EDM,Eele0,Eele1);
+        time5 += Divide_Conquer("scf",LSCF_iter,CntH,iCntHNL,CntOLP[0],DM[0],EDM,Eele0,Eele1);
         break;
 
       case 6: 
@@ -941,7 +989,7 @@ double DFT(int MD_iter, int Cnt_Now)
       case 7: break;       
 
       case 8:
-        time5 += Krylov(LSCF_iter,CntH,iCntHNL,CntOLP[0],DM[0],EDM,Eele0,Eele1);
+        time5 += Krylov("scf",LSCF_iter,CntH,iCntHNL,CntOLP[0],DM[0],EDM,Eele0,Eele1);
         break;       
 
       }
@@ -969,7 +1017,6 @@ double DFT(int MD_iter, int Cnt_Now)
                     check SCF-convergence 
     *****************************************************/
 
-#ifdef TRAN
     if (Solver==4) {
       dUele = 1.0; /* do not calculate Uele */
     }
@@ -980,9 +1027,6 @@ double DFT(int MD_iter, int Cnt_Now)
       else  
         dUele = fabs(Uele - pUele);
     }
-#else 
-    dUele = fabs(Uele - pUele);
-#endif
 
     if (
          2<=SCF_iter &&        
@@ -1010,6 +1054,8 @@ double DFT(int MD_iter, int Cnt_Now)
 
       orbitalOpt_iter++;
       LSCF_iter = 0;
+      SCF_iter_shift = 0;
+      ETemp_controller = 0;
 
       if ( (orbitalOpt_SCF*(orbitalOpt_MD+1)-1) < SCF_iter) po = 1;
       if (orbitalOpt_MD < orbitalOpt_iter){
@@ -1024,16 +1070,51 @@ double DFT(int MD_iter, int Cnt_Now)
 
     if (SCF_iter==1 || LSCF_iter==0)          Calc_CntOrbital_ON = 1;
     else                                      Calc_CntOrbital_ON = 0;
-    
+
     /********************************************************
-       simple, RMM-DIIS, or GR-Pulay mixing for density matrix
+      control the electric temperature 
+      for accelerating SCF convergence
+    ********************************************************/
+
+    if (Solver!=4 && SCF_Control_Temp==1){
+
+      Norm5 = Norm4;
+      Norm4 = Norm3;
+      Norm3 = Norm2;
+      Norm2 = Norm1;
+      Norm1 = NormRD[0];
+      tmp = (Norm1+Norm2+Norm3+Norm4+Norm5)/5.0;
+      tmp0 = sqrt(fabs(NormRD[0]));
+
+      if      (0.01<tmp0 && tmp<Norm1 && 6<LSCF_iter && ETemp_controller==0){
+        E_Temp = 10.0*Original_E_Temp;
+        n = LSCF_iter - Pulay_SCF + 2; 
+        if (0<=n && n<LSCF_iter) SCF_iter_shift = n;
+        ETemp_controller = 1;
+      }
+      else if (tmp0<=0.01 && ETemp_controller<=1){
+        E_Temp = 1.0*Original_E_Temp;
+        n = LSCF_iter - Pulay_SCF + 2; 
+        if (0<=n && n<LSCF_iter) SCF_iter_shift = n;
+        ETemp_controller = 2;
+      }
+
+      /* update Beta */ 
+      Beta = 1.0/kB/E_Temp;
+    }
+
+    /********************************************************
+     simple, RMM-DIIS, or GR-Pulay mixing for density matrix
     ********************************************************/
 
     if (    Mixing_switch==0
 	 || Mixing_switch==1
 	 || Mixing_switch==2 ){
 
-      time6  += Mixing_DM(1,LSCF_iter,SCF_iter,SucceedReadingDMfile,
+      time6  += Mixing_DM(1,
+                          LSCF_iter-SCF_iter_shift,
+                          SCF_iter-SCF_iter_shift,
+                          SucceedReadingDMfile,
 			  ReRhok,ImRhok,ReBestRhok,ImBestRhok,
 			  Residual_ReRhok,Residual_ImRhok,
 			  ReV1,ImV1,ReV2,ImV2,ReRhoAtomk,ImRhoAtomk);
@@ -1042,14 +1123,11 @@ double DFT(int MD_iter, int Cnt_Now)
 
       if (SpinP_switch==3) diagonalize_nc_density();
 
-#ifdef TRAN
       if (Solver==4) {
-	/* output of Set_Density_Grid => Density_Grid */
-	TRAN_Overwrite_Densitygrid(mpi_comm_level1,
-				   SpinP_switch, Ngrid1, Ngrid2, Ngrid3,
-				   Num_Cells0, My_Cell0,My_Cell1, Density_Grid); 
+	TRAN_Add_Density_Lead(mpi_comm_level1,
+			      SpinP_switch, Ngrid1, Ngrid2, Ngrid3,
+			      Num_Cells0, My_Cell0,My_Cell1, Density_Grid); 
       }
-#endif
 
       fft_charge_flag = 1;
     }
@@ -1062,50 +1140,60 @@ double DFT(int MD_iter, int Cnt_Now)
 
       time11 += Set_Density_Grid(Cnt_kind,Calc_CntOrbital_ON,DM[0]);
 
-#ifdef TRAN
       if (Solver==4) {
-	/* output of Set_Density_Grid => Density_Grid */
-	TRAN_Overwrite_Densitygrid(mpi_comm_level1,
-				   SpinP_switch, Ngrid1, Ngrid2, Ngrid3,
-				   Num_Cells0,My_Cell0,My_Cell1, Density_Grid); 
+	TRAN_Add_Density_Lead(mpi_comm_level1,
+			      SpinP_switch, Ngrid1, Ngrid2, Ngrid3,
+			      Num_Cells0, My_Cell0, My_Cell1, Density_Grid); 
       }
-#endif
 
       /* non-spin polarization */
       if (SpinP_switch==0){
-	FFT_Density(1,ReV1,ImV1,ReRhok[0][0],ImRhok[0][0]);
+        if (Solver!=4 || TRAN_Poisson_flag2==4) FFT_Density(1,ReV1,ImV1,ReRhok[0][0],ImRhok[0][0]);
+        else                                    FFT2D_Density(1,ReV1,ImV1,ReRhok[0][0],ImRhok[0][0]);
       }
 
       /* collinear spin polarization */
       else if (SpinP_switch==1) {
-	FFT_Density(1,ReV1,ImV1,ReRhok[0][0],ImRhok[0][0]);
-	FFT_Density(2,ReV1,ImV1,ReRhok[0][1],ImRhok[0][1]);
+        if (Solver!=4 || TRAN_Poisson_flag2==4){
+ 	  FFT_Density(1,ReV1,ImV1,ReRhok[0][0],ImRhok[0][0]);
+	  FFT_Density(2,ReV1,ImV1,ReRhok[0][1],ImRhok[0][1]);
+	}
+        else{
+ 	  FFT2D_Density(1,ReV1,ImV1,ReRhok[0][0],ImRhok[0][0]);
+	  FFT2D_Density(2,ReV1,ImV1,ReRhok[0][1],ImRhok[0][1]);
+        }
       }
       /* non-collinear spin polarization */
       else if (SpinP_switch==3) {
-	FFT_Density(1,ReV1,ImV1,ReRhok[0][0],ImRhok[0][0]);
-	FFT_Density(2,ReV1,ImV1,ReRhok[0][1],ImRhok[0][1]);
-	FFT_Density(4,ReV1,ImV1,ReRhok[0][2],ImRhok[0][2]);
+        if (Solver!=4 || TRAN_Poisson_flag2==4){
+  	  FFT_Density(1,ReV1,ImV1,ReRhok[0][0],ImRhok[0][0]);
+	  FFT_Density(2,ReV1,ImV1,ReRhok[0][1],ImRhok[0][1]);
+	  FFT_Density(4,ReV1,ImV1,ReRhok[0][2],ImRhok[0][2]);
+	}
+        else{
+  	  FFT2D_Density(1,ReV1,ImV1,ReRhok[0][0],ImRhok[0][0]);
+	  FFT2D_Density(2,ReV1,ImV1,ReRhok[0][1],ImRhok[0][1]);
+	  FFT2D_Density(4,ReV1,ImV1,ReRhok[0][2],ImRhok[0][2]);
+        }
       }
 
-      if (SCF_iter==1) FFT_Density(3,ReV1,ImV1,ReRhoAtomk,ImRhoAtomk);
+      if (SCF_iter==1){
+        if (Solver!=4 || TRAN_Poisson_flag2==4)  FFT_Density(3,ReV1,ImV1,ReRhoAtomk,ImRhoAtomk);
+        else                                     FFT2D_Density(3,ReV1,ImV1,ReRhoAtomk,ImRhoAtomk);  
+      }
 
-      time6 += Mixing_DM(1,LSCF_iter,SCF_iter,SucceedReadingDMfile,
+      time6 += Mixing_DM(1,
+                         LSCF_iter-SCF_iter_shift,
+                         SCF_iter-SCF_iter_shift,
+                         SucceedReadingDMfile,
 			 ReRhok,ImRhok,ReBestRhok,ImBestRhok,
 			 Residual_ReRhok,Residual_ImRhok,
 			 ReV1,ImV1,ReV2,ImV2,ReRhoAtomk,ImRhoAtomk);
-#ifdef TRAN
-      if (Solver==4) {
-	/* output of Set_Density_Grid => Density_Grid */
-	TRAN_Overwrite_Densitygrid(mpi_comm_level1,
-				   SpinP_switch, Ngrid1, Ngrid2, Ngrid3,
-				   Num_Cells0,My_Cell0,My_Cell1, Density_Grid); 
-      }
-#endif
-
+               
       /* call diagonalize_nc_density() in Set_Density_Grid.c */
 
       fft_charge_flag = 0;
+
     }
      
     else{
@@ -1177,8 +1265,10 @@ double DFT(int MD_iter, int Cnt_Now)
              sqrt(fabs(NormRD[0])),SCF_Criterion);fflush(stdout);
     }
 
-#ifdef TRAN
     if (Solver==4) {
+
+      /* check whether the SCF converges or not */
+
       if ( sqrt(fabs(NormRD[0]))< SCF_Criterion ) {
 
 	po = 1;
@@ -1186,7 +1276,6 @@ double DFT(int MD_iter, int Cnt_Now)
 	/* printf("TRAN  NormRD < SCF_Criterion, SCF is satisfied \n");	*/
       } 
     }
-#endif
 
     outputfile1(1,MD_iter,orbitalOpt_iter,Cnt_Now,SCF_iter,file_DFTSCF,ChemP_e0); 
 
@@ -1203,18 +1292,14 @@ double DFT(int MD_iter, int Cnt_Now)
   } while (po==0 && SCF_iter<SCF_MAX);
 
   /*****************************************************
-       move to the loop of MD iter in the future
+          making of the input data for TranMain
   *****************************************************/
 
-#ifdef TRAN
-  if (Solver==4)  {
-    TRAN_Output_Trans_HS( mpi_comm_level1, SpinP_switch, ChemP, H, OLP,
-                          atomnum, SpeciesNum, WhatSpecies, 
-                          Spe_Total_CNO, FNAN, natn, ncn, G2ID, atv_ijk,
-                          Max_FSNAN, ScaleSize, F_G2M, TCpyCell, List_YOUSO, 
-                          filepath,filename,"tranb");
-  }
-#endif
+  TRAN_Output_Trans_HS( mpi_comm_level1, Solver, SpinP_switch, ChemP, H, OLP,
+                        atomnum, SpeciesNum, WhatSpecies, 
+                        Spe_Total_CNO, FNAN, natn, ncn, G2ID, atv_ijk,
+                        Max_FSNAN, ScaleSize, F_G2M, TCpyCell, List_YOUSO, 
+                        filepath,filename,"tranb");
 
   /*****************************************************
               save contraction coefficients
@@ -1272,16 +1357,34 @@ double DFT(int MD_iter, int Cnt_Now)
   if ( Dos_fileout || DosGauss_fileout) {
 
     if ( Solver==2 && PeriodicGamma_flag==0 ){  /* cluster */
+
       if ( Cnt_switch==0 ){
-	Cluster_DFT_Dosout( SpinP_switch, H, iHNL, OLP[0] );
+
+        if (Opticalconductivity_fileout==1){
+  	  Cluster_DFT_Dosout( SpinP_switch, H, iHNL, OLP[0] );
+	}
+        else {      
+          time5 += Cluster_DFT("dos",LSCF_iter,SpinP_switch,Cluster_ReCoes,Cluster_ko,
+                               H,iHNL,OLP[0],DM[0],EDM,Eele0,Eele1);
+	}
       }
       else {
-	Cluster_DFT_Dosout( SpinP_switch, CntH, iCntHNL, CntOLP[0] );
+
+        if (Opticalconductivity_fileout==1){
+ 	  Cluster_DFT_Dosout( SpinP_switch, CntH, iCntHNL, CntOLP[0] );
+	}
+        else {
+          time5 += Cluster_DFT("dos",LSCF_iter,SpinP_switch,Cluster_ReCoes,Cluster_ko,
+                               CntH,iCntHNL,CntOLP[0],DM[0],EDM,Eele0,Eele1);
+        }
+
       }
     }
 
     if ( Solver==3 || PeriodicGamma_flag==1 ){  /* band */
       if (Cnt_switch==0){
+
+
 	Band_DFT_Dosout( Dos_Kgrid[0], Dos_Kgrid[1], Dos_Kgrid[2],
 			 SpinP_switch, H, iHNL, OLP[0] );
       }
@@ -1293,21 +1396,19 @@ double DFT(int MD_iter, int Cnt_Now)
 
     if (Solver==4){ /* NEGF */
 
-#ifdef TRAN
        TRAN_DFT_Dosout( mpi_comm_level1, level_stdout, LSCF_iter,SpinP_switch,H,iHNL,OLP[0],
                         atomnum,Matomnum,WhatSpecies, Spe_Total_CNO, FNAN, natn, ncn, 
                         M2G, G2ID, atv_ijk, List_YOUSO, Spe_Num_CBasis, SpeciesNum, filename, filepath,
                         DM[0],EDM,Eele0,Eele1 );
-#endif
 
     }
 
     if ( Solver==5 ){  /* divide-conquer */
       if (Cnt_switch==0){
-	Divide_Conquer_Dosout( H, iHNL, OLP[0] );
+        time5 += Divide_Conquer("dos",LSCF_iter,H,iHNL,OLP[0],DM[0],EDM,Eele0,Eele1);
       }
       else {
-        Divide_Conquer_Dosout( CntH, iCntHNL, CntOLP[0] );
+        time5 += Divide_Conquer("dos",LSCF_iter,CntH,iCntHNL,CntOLP[0],DM[0],EDM,Eele0,Eele1);
       }
     }
 
@@ -1322,10 +1423,10 @@ double DFT(int MD_iter, int Cnt_Now)
 
     if ( Solver==8 ){  /* Krylov subspace method */
       if (Cnt_switch==0){
-        Krylov_Dosout( H, iHNL, OLP[0] );
+        time5 += Krylov("dos",LSCF_iter,H,iHNL,OLP[0],DM[0],EDM,Eele0,Eele1);
       }
       else {
-        Krylov_Dosout( CntH, iCntHNL, CntOLP[0] );
+        time5 += Krylov("dos",LSCF_iter,CntH,iCntHNL,CntOLP[0],DM[0],EDM,Eele0,Eele1);
       }
     }
   }
@@ -1362,14 +1463,11 @@ double DFT(int MD_iter, int Cnt_Now)
   if (Cnt_switch==1 && Cnt_Now==1) time10 += Set_Orbitals_Grid(1);
   time11 += Set_Density_Grid(1, 0, DM[0]);
 
-#ifdef TRAN
   if (Solver==4) {
-    /* output of Set_Density_Grid => Density_Grid */
-    TRAN_Overwrite_Densitygrid(mpi_comm_level1,
-			       SpinP_switch, Ngrid1, Ngrid2, Ngrid3,
-			       Num_Cells0,My_Cell0,My_Cell1, Density_Grid); 
+    TRAN_Add_Density_Lead(mpi_comm_level1,
+			  SpinP_switch, Ngrid1, Ngrid2, Ngrid3,
+			  Num_Cells0,My_Cell0,My_Cell1, Density_Grid); 
   }
-#endif
 
   /* to save mixing densities between the previous and current ones
      do charge mixing */
@@ -1378,34 +1476,42 @@ double DFT(int MD_iter, int Cnt_Now)
 
     /* non-spin polarization */
     if (SpinP_switch==0){
-      FFT_Density(1,ReV1,ImV1,ReRhok[0][0],ImRhok[0][0]);
+      if (Solver!=4 || TRAN_Poisson_flag2==4)  FFT_Density(1,ReV1,ImV1,ReRhok[0][0],ImRhok[0][0]);
+      else                                     FFT2D_Density(1,ReV1,ImV1,ReRhok[0][0],ImRhok[0][0]);
     }
 
     /* collinear spin polarization */
     else if (SpinP_switch==1) {
-      FFT_Density(1,ReV1,ImV1,ReRhok[0][0],ImRhok[0][0]);
-      FFT_Density(2,ReV1,ImV1,ReRhok[0][1],ImRhok[0][1]);
+      if (Solver!=4 || TRAN_Poisson_flag2==4){
+        FFT_Density(1,ReV1,ImV1,ReRhok[0][0],ImRhok[0][0]);
+        FFT_Density(2,ReV1,ImV1,ReRhok[0][1],ImRhok[0][1]);
+      }
+      else{
+        FFT2D_Density(1,ReV1,ImV1,ReRhok[0][0],ImRhok[0][0]);
+        FFT2D_Density(2,ReV1,ImV1,ReRhok[0][1],ImRhok[0][1]);
+      }
     }
     /* non-collinear spin polarization */
     else if (SpinP_switch==3) {
-      FFT_Density(1,ReV1,ImV1,ReRhok[0][0],ImRhok[0][0]);
-      FFT_Density(2,ReV1,ImV1,ReRhok[0][1],ImRhok[0][1]);
-      FFT_Density(4,ReV1,ImV1,ReRhok[0][2],ImRhok[0][2]);
+      if (Solver!=4 || TRAN_Poisson_flag2==4){
+        FFT_Density(1,ReV1,ImV1,ReRhok[0][0],ImRhok[0][0]);
+        FFT_Density(2,ReV1,ImV1,ReRhok[0][1],ImRhok[0][1]);
+        FFT_Density(4,ReV1,ImV1,ReRhok[0][2],ImRhok[0][2]);
+      }
+      else{
+        FFT2D_Density(1,ReV1,ImV1,ReRhok[0][0],ImRhok[0][0]);
+        FFT2D_Density(2,ReV1,ImV1,ReRhok[0][1],ImRhok[0][1]);
+        FFT2D_Density(4,ReV1,ImV1,ReRhok[0][2],ImRhok[0][2]);
+      }
     }
 
-    time6 += Mixing_DM(1,LSCF_iter,SCF_iter,SucceedReadingDMfile,
+    time6 += Mixing_DM(1,
+                       LSCF_iter-SCF_iter_shift,
+                       SCF_iter-SCF_iter_shift,
+                       SucceedReadingDMfile,
 		       ReRhok,ImRhok,ReBestRhok,ImBestRhok,
 		       Residual_ReRhok,Residual_ImRhok,
 		       ReV1,ImV1,ReV2,ImV2,ReRhoAtomk,ImRhoAtomk);
-#ifdef TRAN
-    if (Solver==4) {
-      /* output of Set_Density_Grid => Density_Grid */
-      TRAN_Overwrite_Densitygrid(mpi_comm_level1,
-				 SpinP_switch, Ngrid1, Ngrid2, Ngrid3,
-				 Num_Cells0,My_Cell0,My_Cell1, Density_Grid); 
-    }
-#endif
-
   }
 
   /* write a restart file, it is important to save
@@ -1481,7 +1587,7 @@ double DFT(int MD_iter, int Cnt_Now)
     printf("  Uzs   = %20.12f\n",Uzs);
     printf("  Uzo   = %20.12f\n",Uzo);
     printf("  Uef   = %20.12f\n",Uef);
-    printf("  Utot  = %20.12f\n\n",Utot);
+    printf("  Utot  = %20.14f\n\n",Utot);
 
     printf("  Note:\n\n");
     printf("  Utot = Ukin+UH0+UH1+Una+Unl+Uxc0+Uxc1+Ucore+Uhub+Ucs+Uzs+Uzo+Uef\n\n");
@@ -1844,12 +1950,11 @@ double DFT(int MD_iter, int Cnt_Now)
 
   }
 
-#ifdef TRAN
   if (Solver==4)  {
     TRAN_Deallocate_Lead_Region();
     TRAN_Deallocate_Cregion( SpinP_switch );
+    TRAN_Deallocate_Electrode_Grid(Ngrid2);
   }
-#endif
 
   return time0;
 }
@@ -1858,6 +1963,11 @@ double DFT(int MD_iter, int Cnt_Now)
 
 
 
+
+
+
+
+
 void Output_Energies_Forces(FILE *fp)
 { 
   int ct_AN;
