@@ -746,7 +746,7 @@ static double DC_Dosout_Col(double *****Hks, double ****OLP0)
       Eigen_lapack(D,ko,NUM1,NUM1);
 
       /***********************************************
-        transformation to the original eigen vectors.
+        transformation to the original eigenvectors.
                         NOTE 244P
       ***********************************************/
 
@@ -1750,7 +1750,7 @@ static double DC_Dosout_NonCol(double *****Hks,
 
     NUM1 = 2*NUM - (P_min - 1);
     Msize1[Mc_AN] = NUM1;
-    EigenBand_lapack(C, ko, NUM1);
+    EigenBand_lapack(C, ko, NUM1, 1);
 
     for (i1=1; i1<=NUM1; i1++){
       for (j1=1; j1<=NUM1; j1++){
