@@ -9,6 +9,7 @@
 
  
 #include <stdio.h>
+#include <stdlib.h>
 #include <string.h>
 #include <math.h>
 #include "Inputtools.h"
@@ -817,8 +818,7 @@ void Spectra_Gaussian( int pdos_n, int *pdos_atoms, char *basename, int Dos_N,
       if (Spe_Num_CBasis[wanA][L]>0) {
         for (M=0;M<2*L+1;M++) {
           LM=100*L+M+1;
-	  sprintf(file_Dos,"%s.%s.atom%d.%s%d",basename,extension,pdos_atoms[iatom],
-		  Lname[L],M+1);
+	  sprintf(file_Dos,"%s.%s.atom%d.%s%d",basename,extension,pdos_atoms[iatom],Lname[L],M+1);
 	  if ( (fp_Dos=fopen(file_Dos,"w")) ==NULL )  {
 
 #ifdef xt3
@@ -854,13 +854,14 @@ void Spectra_Gaussian( int pdos_n, int *pdos_atoms, char *basename, int Dos_N,
   } /* iatom */
 
   /* atom projection */
-  for (iatom=0;iatom<pdos_n;iatom++) {
+  for (iatom=0; iatom<pdos_n; iatom++) {
+
     GA_AN = pdos_atoms[iatom]-1;
     wanA = WhatSpecies[GA_AN];
     tnoA = Spe_Total_CNO[wanA];
 
-    sprintf(file_Dos,"%s.%s.atom%d",basename,extension,pdos_atoms[iatom],
-	    Lname[L]);
+    sprintf(file_Dos,"%s.%s.atom%d",basename,extension,pdos_atoms[iatom]);
+
     if ( (fp_Dos=fopen(file_Dos,"w")) ==NULL )  {
 
 #ifdef xt3
@@ -1095,8 +1096,7 @@ void SpectraDC_Gaussian( int pdos_n, int *pdos_atoms, char *basename, int Dos_N,
     wanA = WhatSpecies[GA_AN];
     tnoA = Spe_Total_CNO[wanA];
 
-    sprintf(file_Dos,"%s.%s.atom%d",basename,extension,pdos_atoms[iatom],
-	    Lname[L]);
+    sprintf(file_Dos,"%s.%s.atom%d",basename,extension,pdos_atoms[iatom]);
     if ( (fp_Dos=fopen(file_Dos,"w")) ==NULL )  {
 
 #ifdef xt3
@@ -1404,11 +1404,13 @@ void Spectra_Tetrahedron( int pdos_n, int *pdos_atoms, char *basename, int Dos_N
 		}
 	      }
 	      if (SpinP_switch==1) {
-		fprintf(fp_Dos,"%lf %lf %lf\n", (DosE[ie])*eV2Hartree, 
-			dossum[0],dossum[1]);
+		fprintf(fp_Dos,"%lf %lf %lf\n", 
+                        (DosE[ie])*eV2Hartree, 
+			dossum[0],-dossum[1]);
 	      }
 	      else {
-		fprintf(fp_Dos,"%lf %lf\n", (DosE[ie])*eV2Hartree,
+		fprintf(fp_Dos,"%lf %lf\n", 
+                        (DosE[ie])*eV2Hartree,
 			dossum[0]*2.0);
 	      }
 
@@ -1426,8 +1428,7 @@ void Spectra_Tetrahedron( int pdos_n, int *pdos_atoms, char *basename, int Dos_N
     wanA = WhatSpecies[GA_AN];
     tnoA = Spe_Total_CNO[wanA];
 
-    sprintf(file_Dos,"%s.%s.atom%d",basename,extension,pdos_atoms[iatom],
-	    Lname[L]);
+    sprintf(file_Dos,"%s.%s.atom%d",basename,extension,pdos_atoms[iatom]);
     if ( (fp_Dos=fopen(file_Dos,"w")) ==NULL )  {
 
 #ifdef xt3
@@ -1622,8 +1623,7 @@ void Spectra_NEGF( int pdos_n, int *pdos_atoms, char *basename, char *extension,
     wanA = WhatSpecies[GA_AN];
     tnoA = Spe_Total_CNO[wanA];
 
-    sprintf(file_Dos,"%s.PDOS.%s.atom%d",basename,extension,pdos_atoms[iatom],
-	    Lname[L]);
+    sprintf(file_Dos,"%s.PDOS.%s.atom%d",basename,extension,pdos_atoms[iatom]);
     if ( (fp_Dos=fopen(file_Dos,"w")) ==NULL )  {
 
 #ifdef xt3
