@@ -48,7 +48,7 @@ void Eigen_Improved_PHH(MPI_Comm MPI_Current_Comm_WD,
                         dcomplex **ac, double *ko, int n, int EVmax, int bcast_flag)
 {
   double ABSTOL=1.0e-14;
-  dcomplex **ad,*u,*b1,*p,*q,tmp1,tmp2,u1,u2,p1,ss;
+  dcomplex **ad,*u,*b1,*p,*q,tmp0,tmp1,tmp2,u1,u2,p1,ss,ss0,ss1;
   double *uu,*alphar,*alphai,
          s1,s2,s3,r,
          sum,ar,ai,br,bi,e,
@@ -56,8 +56,8 @@ void Eigen_Improved_PHH(MPI_Comm MPI_Current_Comm_WD,
          r3,x1,x2,xap,my_r,
          bb,bb1,ui,uj,uij;
 
-  int jj,jj1,jj2,k,ii,ll,i3,i2,j2,
-      i,j,i1,j1,n1,n2,ik,k0,k1,num0,num1,nump,
+  int jj,jj1,jj2,k,ks,ii,ll,i3,i2,j2,
+      i,j,i1,i1s,j1,n1,n2,ik,k0,k1,num0,num1,nump,
       jk,po1,nn,count,is0,num,ID0,ID1,IDS,IDR;
 
   double Stime,Etime;
@@ -205,7 +205,18 @@ void Eigen_Improved_PHH(MPI_Comm MPI_Current_Comm_WD,
     u[i+1].r = ac[i][i+1].r;
     u[i+1].i =-ac[i][i+1].i;
 
-    for (i1=i+2; i1<=n; i1++){
+    for (i1=i+2; i1<=(n-1); i1+=2){
+      u[i1+0].r = ac[i][i1+0].r; 
+      u[i1+0].i =-ac[i][i1+0].i; 
+      u[i1+1].r = ac[i][i1+1].r; 
+      u[i1+1].i =-ac[i][i1+1].i; 
+
+      s2 += u[i1+0].r*u[i1+0].r + u[i1+0].i*u[i1+0].i
+          + u[i1+1].r*u[i1+1].r + u[i1+1].i*u[i1+1].i;
+    }
+
+    i1s = n + 1 - (n+1-(i+2))%2;
+    for (i1=i1s; ((i+2)<=i1 && i1<=n); i1++){
       u[i1].r = ac[i][i1].r; 
       u[i1].i =-ac[i][i1].i; 
       s2 += u[i1].r*u[i1].r + u[i1].i*u[i1].i;
@@ -523,7 +534,7 @@ void Eigen_Improved_PHH(MPI_Comm MPI_Current_Comm_WD,
 
   if      (dste_flag==0) lapack_dstegr2(n,EVmax,&a1d[0],&a1d[n],ko,ac);
   else if (dste_flag==1) lapack_dstedc2(n,&a1d[0],&a1d[n],ko,ac);
-  else if (dste_flag==2) lapack_dstevx2(n,EVmax,&a1d[0],&a1d[n],ko,ac);
+  else if (dste_flag==2) lapack_dstevx2(n,EVmax,&a1d[0],&a1d[n],ko,ac,1);
 
   if (measure_time==1){
     dtime(&Etime);
@@ -544,7 +555,41 @@ void Eigen_Improved_PHH(MPI_Comm MPI_Current_Comm_WD,
     ad[i][i-1].i = b1[i-1].i;
   }
 
-  for (k=is1[myid]; k<=ie1[myid]; k++){
+  for (k=is1[myid]; k<=(ie1[myid]-1); k+=2){
+    for (nn=1; nn<=n-1; nn++){
+
+      if ( (1.0e-3*ABSTOL)<fabs(uu[n-nn])){
+
+	tmp0.r = 0.0;	tmp0.i = 0.0;
+	tmp1.r = 0.0;	tmp1.i = 0.0;
+
+	for (i=n-nn+1; i<=n; i++){
+	  tmp0.r += ad[n-nn][i].r * ac[k+0][i].r - ad[n-nn][i].i * ac[k+0][i].i;
+	  tmp0.i += ad[n-nn][i].i * ac[k+0][i].r + ad[n-nn][i].r * ac[k+0][i].i;
+	  tmp1.r += ad[n-nn][i].r * ac[k+1][i].r - ad[n-nn][i].i * ac[k+1][i].i;
+	  tmp1.i += ad[n-nn][i].i * ac[k+1][i].r + ad[n-nn][i].r * ac[k+1][i].i;
+	}
+
+	ss0.r = (alphar[n-nn]*tmp0.r - alphai[n-nn]*tmp0.i) / uu[n-nn];
+	ss0.i = (alphar[n-nn]*tmp0.i + alphai[n-nn]*tmp0.r) / uu[n-nn];
+
+	ss1.r = (alphar[n-nn]*tmp1.r - alphai[n-nn]*tmp1.i) / uu[n-nn];
+	ss1.i = (alphar[n-nn]*tmp1.i + alphai[n-nn]*tmp1.r) / uu[n-nn];
+
+	for (i=n-nn+1; i<=n; i++){
+	  ac[k+0][i].r -= ss0.r * ad[n-nn][i].r + ss0.i * ad[n-nn][i].i;
+	  ac[k+0][i].i -=-ss0.r * ad[n-nn][i].i + ss0.i * ad[n-nn][i].r;
+	  ac[k+1][i].r -= ss1.r * ad[n-nn][i].r + ss1.i * ad[n-nn][i].i;
+	  ac[k+1][i].i -=-ss1.r * ad[n-nn][i].i + ss1.i * ad[n-nn][i].r;
+	}
+
+      }
+    }
+  }
+
+  ks = ie1[myid] + 1 - (ie1[myid]+1-is1[myid])%2;
+
+  for (k=ks; (is1[myid]<=k && k<=ie1[myid]); k++){
     for (nn=1; nn<=n-1; nn++){
 
       if ( (1.0e-3*ABSTOL)<fabs(uu[n-nn])){
@@ -955,7 +1000,7 @@ void Eigen_Original_PHH(MPI_Comm MPI_Current_Comm_WD,
 
   if      (dste_flag==0) lapack_dstegr2(n,EVmax,D,E,ko,ac);
   else if (dste_flag==1) lapack_dstedc2(n,D,E,ko,ac);
-  else if (dste_flag==2) lapack_dstevx2(n,EVmax,D,E,ko,ac);
+  else if (dste_flag==2) lapack_dstevx2(n,EVmax,D,E,ko,ac,1);
 
   if (measure_time==1){
     dtime(&Etime);
