@@ -15,6 +15,8 @@
 #include <math.h>
 #include <stdlib.h>
 #include "openmx_common.h"
+#include "lapack_prototypes.h"
+
 
 #ifdef nompi
 #include "mimic_mpi.h"
@@ -31,6 +33,18 @@ static void Eigen_Original_PReHH(MPI_Comm MPI_Curret_Comm_WD,
 static void Eigen_Improved_PReHH(MPI_Comm MPI_Curret_Comm_WD, 
 			double **ac, double *ko, int n, int EVmax, int bcast_flag);
 
+static void myHH( MPI_Comm MPI_Curret_Comm_WD, int numprocs, int myid, int n, double **ac, double **ad, 
+		  double *uu, double *b1, 
+		  MPI_Request *request_send, 
+		  MPI_Request *request_recv, 
+		  MPI_Status *stat_send);
+
+static void call_dsytrd( MPI_Comm MPI_Curret_Comm_WD, int numprocs, int myid, int n, double **ac, double **ad, 
+			 double *uu, double *p, double *q, double *b1, 
+			 MPI_Request *request_send, 
+			 MPI_Request *request_recv, 
+			 MPI_Status *stat_send);
+
 
 void Eigen_PReHH(MPI_Comm MPI_Curret_Comm_WD, 
                  double **ac, double *ko, int n, int EVmax, int bcast_flag)
@@ -51,19 +65,21 @@ void Eigen_Improved_PReHH(MPI_Comm MPI_Curret_Comm_WD,
                           double **ac, double *ko, int n, int EVmax, int bcast_flag)
 {
   double ABSTOL=1.0e-13;
-  double **ad,*b1,*u,*uu,*q,*p,
+  double **ad,*b1,*uu,*q,*p,
     s1,s2,s3,ss,u1,u2,r,p1,my_r,my_s2, 
     xsum,bunbo,si,co,sum,
     a1,a2,a3,a4,a5,a6,b7,r1,r2,
     r3,x1,x2,xap,tmp1,tmp2,
-    bb,bb1,ui,uj,uij;
+    bb,bb1,ui,uj,uij,
+    ss0,ss1,ss2,ss3,
+    s20,s21,s22,s23;
 
   int jj,jj1,jj2,k,k0,k1,ii,ll,i3,i2,j2,
-    i,j,i1,j1,n1,n2,ik,
+    i,j,i1,j1,n1,n2,ik,ks,i1s,
     jk,po1,nn,count,num,num0,num1;
 
   int is0;
-  int *is1,*ie1,*is2,*ie2,*row_flag;
+  int *is1,*ie1;
   double av_num;
   double Stime, Etime;
   int numprocs,nump,myid,tag=999,ID,IDS,IDR,ID0,ID1;
@@ -122,18 +138,12 @@ void Eigen_Improved_PReHH(MPI_Comm MPI_Curret_Comm_WD,
 
   n2 = n + 5;
 
-  is2 = (int*)malloc(sizeof(int)*numprocs);
-  ie2 = (int*)malloc(sizeof(int)*numprocs);
-
-  row_flag = (int*)malloc(sizeof(int)*n2);
-
   ad = (double**)malloc(sizeof(double*)*n2);
   for (i=0; i<n2; i++){
     ad[i] = (double*)malloc(sizeof(double)*n2);
   }
 
   b1 = (double*)malloc(sizeof(double)*n2);
-  u = (double*)malloc(sizeof(double)*n2);
   uu = (double*)malloc(sizeof(double)*n2);
   q = (double*)malloc(sizeof(double)*n2);
   p = (double*)malloc(sizeof(double)*2*n2);
@@ -144,278 +154,47 @@ void Eigen_Improved_PReHH(MPI_Comm MPI_Curret_Comm_WD,
   }
 
   if (measure_time==1) printf("size n=%3d EVmax=%2d\n",n,EVmax);
-  if (measure_time==1) dtime(&Stime);
 
   time1 = 0.0;
   time2 = 0.0;
-  time1a = 0.0;
-  time1b = 0.0;
-  time1c = 0.0;
-  time1d = 0.0;
-  time1e = 0.0;
 
   /****************************************************
                    Householder method
   ****************************************************/
 
-  /* make is2 and ie2 */
+  myHH(MPI_Curret_Comm_WD, numprocs, myid, n, ac, ad, uu, b1, request_send, request_recv, stat_send);
 
-  if ( numprocs<=(n-1) ){
-
-    av_num = (double)(n-1)/(double)numprocs;
-
-    for (ID=0; ID<numprocs; ID++){
-      is2[ID] = (int)(av_num*(double)ID) + 2;
-      ie2[ID] = (int)(av_num*(double)(ID+1)) + 1;
-    }
-
-    is2[0] = 2;
-    ie2[numprocs-1] = n;
-  }
-
-  else{
-    for (ID=0; ID<(n-1); ID++){
-      is2[ID] = ID + 2;
-      ie2[ID] = ID + 2;
-    }
-    for (ID=(n-1); ID<numprocs; ID++){
-      is2[ID] = 2;
-      ie2[ID] = 0;
-    }
-  }
-
-  row_flag[1] = 0;
-  for (ID=0; ID<numprocs; ID++){
-    for (i=is2[ID]; i<=ie2[ID]; i++){
-      row_flag[i] = ID;
-    }
-  }
-
-  /* loop for i */
-
-  for (i=1; i<=(n-2); i++){
-
-    if (measure_time==1) dtime(&Stime1);
-
-    /* make u vector */
-
-    s2 = 0.0;
-
-    u[i+1] = ac[i][i+1];
-    for (i1=i+2; i1<=n; i1++){
-      u[i1] = ac[i][i1];
-      s2 += u[i1]*u[i1];
-    }
-
-    u[i] = s2;
-      
-    ID = row_flag[i];
-    count = n - i + 1;
-
-    if (myid==ID){
-      for (IDS=ID+1; IDS<numprocs; IDS++){
-        MPI_Isend(&u[i], count, MPI_DOUBLE, IDS, tag, MPI_Curret_Comm_WD, &request_send[IDS-(ID+1)]);
-      }
-
-      num = (numprocs-1) - (ID+1) + 1;  
-      if (1<=num){
-        MPI_Waitall(num,request_send,stat_send);
-      }
-    }
-    else if ( (ID+1)<=myid ) {
-      MPI_Recv(&u[i], count, MPI_DOUBLE, ID, tag, MPI_Curret_Comm_WD, &stat);
-    }
-
-    s2 = u[i];
-
-    if (measure_time==1){ 
-      dtime(&Etime1);
-      time1a += Etime1 - Stime1;      
-    }
-
-    s1 = u[i+1]*u[i+1];
-    s3 = fabs(s1 + s2);
-
-    if (ABSTOL<fabs(u[i+1])){
-      if (u[i+1]<0.0)    s3 =  sqrt(s3);
-      else               s3 = -sqrt(s3);
-    }
-    else{
-      s3 = sqrt(s3);
-    }
-
-    if (ABSTOL<fabs(s2)){
-
-      if (measure_time==1) dtime(&Stime1);
-
-      ss = u[i+1];
-      ac[i+1][i] = s3;
-      ac[i][i+1] = s3;
-      u[i+1] = u[i+1] - s3;
-      u1 = s3 * s3 - ss * s3;
-      u2 = 2.0 * u1;
-
-      uu[i] = u2;
-      b1[i] = ss - s3;
-
-      my_r = 0.0;
-      ID0 = row_flag[i+1];
-
-      if ( (i+1)<=is2[myid] || (i+1)<=ie2[myid] ){
-
-	if (is2[myid]<(i+1)) is0 = i + 1;
-	else                 is0 = is2[myid];
-
-	for (i1=is0; i1<=ie2[myid]; i1++){
- 
-          i2 = i1 + (myid - ID0);
-	  p[i2] = 0.0;
-
-	  for (j=i+1; j<=n; j++){
-	    p[i2] += ac[i1][j] * u[j];
-	  }
-
-	  p[i2] = p[i2] / u1;
-	  my_r += u[i1] * p[i2];
-	}
-        
-        p[ie2[myid]+(myid-ID0)+1] = my_r; 
-      }
-
-      if (measure_time==1){
-	dtime(&Etime1);
-	time1b += Etime1 - Stime1;      
-      }
-      
-      if (measure_time==1) dtime(&Stime1);
-
-      /* MPI: p */
-
-      if (ID0<=myid){ 
-
-        /* the number of working processors */
-
-	nump = numprocs - ID0;
-
-        /* sending */
-
-	num0 = ie2[myid] - is2[myid] + 2;
-	k0 = is2[myid] + (myid - ID0);
-
-	for (ID=0; ID<nump; ID++){
-	  IDS = (myid + ID) % nump + ID0; 
-	  MPI_Isend(&p[k0], num0, MPI_DOUBLE, IDS, tag, MPI_Curret_Comm_WD, &request_send[IDS-ID0]);
-	}
-
-        /* receiving */
-
-	for (ID=0; ID<nump; ID++){
-
-	  IDR = (myid - ID + nump) % nump + ID0;
-	  num1 = ie2[IDR] - is2[IDR] + 2;
-	  k1 = is2[IDR] + (IDR - ID0);
-	  MPI_Irecv(&p[k1], num1, MPI_DOUBLE, IDR, tag, MPI_Curret_Comm_WD, &request_recv[IDR-ID0]);
-	}
-
-        /* waitall */
-
-	MPI_Waitall(nump,request_recv,stat_send);
-	MPI_Waitall(nump,request_send,stat_send);
-      }
-
-      if (measure_time==1){
-	dtime(&Etime1);
-	time1c += Etime1-Stime1;      
-      }
-
-      if (measure_time==1) dtime(&Stime1);
-
-      /* calculate r */
-
-      r = 0.0;
-      for (ID=ID0; ID<numprocs; ID++){
-        r += p[ie2[ID]+(ID-ID0)+1];
-      }
-
-      /* shift p */
-      
-      for (ID=(ID0+1); ID<numprocs; ID++){
-        ID1 = ID - ID0;
-	for (i1=is2[ID]; i1<=ie2[ID]; i1++){
-          p[i1] = p[i1+ID1];
-	}
-      }
-
-      /* calculate q */
-
-      if ( (i+1)<=is2[myid] || (i+1)<=ie2[myid] ){
-	r = r / u2;
-	for (i1=i+1; i1<=n; i1++){
-	  q[i1] = p[i1] - r * u[i1];
-	}
-      }
-
-      if ( (i+1)<=is2[myid] || (i+1)<=ie2[myid] ){
-
-	if (is2[myid]<(i+1)) is0 = i + 1;
-	else                 is0 = is2[myid];
-
-	for (i1=is0; i1<=ie2[myid]; i1++){
-
-	  tmp1 = u[i1];
-	  tmp2 = q[i1];
-
-	  for (j1=i+1; j1<=n; j1++){
-	    ac[i1][j1] -= tmp1 * q[j1] + tmp2 * u[j1];
-	  }
-	}
-      }
-
-      if (measure_time==1){
-	dtime(&Etime1);
-	time1d += Etime1 - Stime1;      
-      }
-
-    }
-
-    if (myid<row_flag[i+1]) break;
-  }
-
-  MPI_Barrier(MPI_Curret_Comm_WD);
-
-  if (measure_time==1) dtime(&Stime1);
-
-  /* broadcast uu, b1, and ac */
-
-  MPI_Bcast(&uu[0],n+1,MPI_DOUBLE,numprocs-1,MPI_Curret_Comm_WD);
-  MPI_Bcast(&b1[0],n+1,MPI_DOUBLE,numprocs-1,MPI_Curret_Comm_WD);
-
-  BroadCast_ReMatrix(MPI_Curret_Comm_WD,ac,n,is2,ie2,myid,numprocs,
-                     stat_send,request_send,request_recv);
-
-  if (measure_time==1){
-    dtime(&Etime1);
-    time1e += Etime1-Stime1;      
-  }
+  /*
+  call_dsytrd(MPI_Curret_Comm_WD, numprocs, myid, n, ac, ad, uu, p, q, b1, request_send, request_recv, stat_send);
+  */
 
+  /*
+  printf("ad\n");
   for (i=1; i<=n; i++){
     for (j=1; j<=n; j++){
-      ad[i][j] = ac[i][j];
+      printf("%10.5f ",ad[i][j]);
     }
+    printf("\n");
   }
 
-  if (measure_time==1){
-    printf("T0  myid=%2d time1a=%15.12f\n",myid,time1a);
-    printf("T0  myid=%2d time1b=%15.12f\n",myid,time1b);
-    printf("T0  myid=%2d time1c=%15.12f\n",myid,time1c);
-    printf("T0  myid=%2d time1d=%15.12f\n",myid,time1d);
-    printf("T0  myid=%2d time1e=%15.12f\n",myid,time1e);
+  printf("uu\n");
+  for (i=1; i<=n; i++){
+     printf("%10.5f ",uu[i]);
   }
+  printf("\n");
 
-  if (measure_time==1){
-    dtime(&Etime);
-    printf("T1 myid=%2d   %15.12f\n",myid,Etime-Stime);
+  printf("b1\n");
+  for (i=1; i<=n; i++){
+     printf("%10.5f ",b1[i]);
   }
+  printf("\n");
+  */
+
+  /*
+  MPI_Finalize();
+  exit(0);
+  */
+
 
   /****************************************************
                    call a lapack routine
@@ -450,7 +229,42 @@ void Eigen_Improved_PReHH(MPI_Comm MPI_Curret_Comm_WD,
     ad[i-1][i] = b1[i-1];
   }
 
-  for (k=is1[myid]; k<=ie1[myid]; k++){
+  for (k=is1[myid]; k<=(ie1[myid]-3); k+=4){
+    for (nn=2; nn<=(n-1); nn++){
+
+      if ( (1.0e-3*ABSTOL)<fabs(uu[n-nn])){
+
+	ss0 = 0.0;
+	ss1 = 0.0;
+	ss2 = 0.0;
+	ss3 = 0.0;
+
+	for (i=n-nn+1; i<=n; i++){
+	  ss0 += ad[n-nn][i] * ac[k+0][i];
+	  ss1 += ad[n-nn][i] * ac[k+1][i];
+	  ss2 += ad[n-nn][i] * ac[k+2][i];
+	  ss3 += ad[n-nn][i] * ac[k+3][i];
+	}
+
+	ss0 = 2.0*ss0/uu[n-nn];
+	ss1 = 2.0*ss1/uu[n-nn];
+	ss2 = 2.0*ss2/uu[n-nn];
+	ss3 = 2.0*ss3/uu[n-nn];
+
+	for (i=n-nn+1; i<=n; i++){
+	  ac[k+0][i] -= ss0 * ad[n-nn][i];
+	  ac[k+1][i] -= ss1 * ad[n-nn][i];
+	  ac[k+2][i] -= ss2 * ad[n-nn][i];
+	  ac[k+3][i] -= ss3 * ad[n-nn][i];
+	}
+
+      }
+    }
+  }
+
+  ks = ie1[myid] + 1 - (ie1[myid]+1-is1[myid])%4;
+
+  for (k=ks; (is1[myid]<=k && k<=ie1[myid]); k++){
     for (nn=2; nn<=(n-1); nn++){
 
       if ( (1.0e-3*ABSTOL)<fabs(uu[n-nn])){
@@ -542,9 +356,6 @@ void Eigen_Improved_PReHH(MPI_Comm MPI_Curret_Comm_WD,
 
   free(is1);
   free(ie1);
-  free(is2);
-  free(ie2);
-  free(row_flag);
 
   for (i=0; i<n2; i++){
     free(ad[i]);
@@ -552,7 +363,6 @@ void Eigen_Improved_PReHH(MPI_Comm MPI_Curret_Comm_WD,
   free(ad);
 
   free(b1);
-  free(u);
   free(uu);
   free(q);
   free(p);
@@ -560,6 +370,393 @@ void Eigen_Improved_PReHH(MPI_Comm MPI_Curret_Comm_WD,
 
 
 
+void call_dsytrd( MPI_Comm MPI_Curret_Comm_WD, int numprocs, int myid, int n, double **ac, double **ad, 
+		  double *uu, double *p, double *q, double *b1, 
+		  MPI_Request *request_send, 
+		  MPI_Request *request_recv, 
+		  MPI_Status *stat_send)
+{
+  char  *UPLO="U";
+  int i,j,lwork,info;
+  double *A,*work;
+
+  lwork = 2*n;
+
+  A = (double*)malloc(sizeof(double)*n*n) ;
+  work = (double*)malloc(sizeof(double)*lwork) ;
+
+  for (i=1; i<=n; i++){
+    for (j=1; j<=n; j++){
+       A[(j-1)*n+(i-1)]= ac[i][j];
+    }
+  }
+       
+  dsytrd_( UPLO, &n, A, &n, p, q, b1, work, &lwork, &info);
+
+  for (j=0; j<n; j++) {
+    for (i=0; i<=j; i++) {
+      ad[i+1][j+1] = A[j*n+i];
+      ad[j+1][i+1] = A[j*n+i];
+    }
+  }
+
+
+
+}
+
+
+
+
+void myHH( MPI_Comm MPI_Curret_Comm_WD, int numprocs, int myid, int n, double **ac, double **ad, 
+           double *uu, double *b1, 
+           MPI_Request *request_send, 
+           MPI_Request *request_recv, 
+           MPI_Status *stat_send)
+{
+  double ABSTOL=1.0e-13; 
+  int jj1,i,j,i1,j1,i1s,is0,ID1,ID0,k1,i2,n2;
+  int nump,num0,k0,num,count,ID;
+  int IDS,IDR,num1,tag=999;
+  int *is2,*ie2,*row_flag;
+  double *p,*q,*u;
+  double u1,u2,ss,my_r,r;
+  double s1,s2,s3,s20,s21,s22,s23,tmp1,tmp2;
+  double time1a,time1b,time1c,time1d,time1e;
+  double Stime1, Etime1,av_num;
+  double Stime, Etime;
+
+  MPI_Status stat;
+  MPI_Request request;
+
+  n2 = n + 5;
+
+  /* allocation of arrays */
+  
+  u = (double*)malloc(sizeof(double)*n2);
+  is2 = (int*)malloc(sizeof(int)*numprocs);
+  ie2 = (int*)malloc(sizeof(int)*numprocs);
+  row_flag = (int*)malloc(sizeof(int)*n2);
+  q = (double*)malloc(sizeof(double)*n2);
+  p = (double*)malloc(sizeof(double)*2*n2);
+
+  time1a = 0.0;
+  time1b = 0.0;
+  time1c = 0.0;
+  time1d = 0.0;
+  time1e = 0.0;
+
+  if (measure_time==1) dtime(&Stime);
+
+  /****************************************************
+                   Householder method
+  ****************************************************/
+
+  /* make is2 and ie2 */
+
+  if ( numprocs<=(n-1) ){
+
+    av_num = (double)(n-1)/(double)numprocs;
+
+    for (ID=0; ID<numprocs; ID++){
+      is2[ID] = (int)(av_num*(double)ID) + 2;
+      ie2[ID] = (int)(av_num*(double)(ID+1)) + 1;
+    }
+
+    is2[0] = 2;
+    ie2[numprocs-1] = n;
+  }
+
+  else{
+    for (ID=0; ID<(n-1); ID++){
+      is2[ID] = ID + 2;
+      ie2[ID] = ID + 2;
+    }
+    for (ID=(n-1); ID<numprocs; ID++){
+      is2[ID] = 2;
+      ie2[ID] = 0;
+    }
+  }
+
+  row_flag[1] = 0;
+  for (ID=0; ID<numprocs; ID++){
+    for (i=is2[ID]; i<=ie2[ID]; i++){
+      row_flag[i] = ID;
+    }
+  }
+
+  /* loop for i */
+
+  for (i=1; i<=(n-2); i++){
+
+    if (measure_time==1) dtime(&Stime1);
+
+    /* make u vector */
+
+    u[i+1] = ac[i][i+1];
+
+    /* original version */    
+    /*
+    s2 = 0.0;
+    for (i1=i+2; i1<=n; i1++){
+      u[i1] = ac[i][i1];
+      s2 += u[i1]*u[i1];
+    }
+    */
+
+    /* unrolling version */    
+
+    s20 = 0.0;
+    s21 = 0.0;
+    s22 = 0.0;
+    s23 = 0.0;
+
+    for (i1=i+2; i1<=(n-3); i1+=4){
+
+      s20 += ac[i][i1+0]*ac[i][i1+0];
+      s21 += ac[i][i1+1]*ac[i][i1+1];
+      s22 += ac[i][i1+2]*ac[i][i1+2];
+      s23 += ac[i][i1+3]*ac[i][i1+3];
+
+      u[i1+0] = ac[i][i1+0];
+      u[i1+1] = ac[i][i1+1];
+      u[i1+2] = ac[i][i1+2];
+      u[i1+3] = ac[i][i1+3];
+    }
+
+    i1s = n + 1 - (n+1-(i+2))%4;
+    for (i1=i1s; ((i+2)<=i1 && i1<=n); i1++){
+      s20 += ac[i][i1]*ac[i][i1];
+      u[i1] = ac[i][i1];
+    }
+
+    s2 = s20 + s21 + s22 + s23;
+
+    u[i] = s2;
+      
+    ID = row_flag[i];
+    count = n - i + 1;
+
+    if (myid==ID){
+      for (IDS=ID+1; IDS<numprocs; IDS++){
+        MPI_Isend(&u[i], count, MPI_DOUBLE, IDS, tag, MPI_Curret_Comm_WD, &request_send[IDS-(ID+1)]);
+      }
+
+      num = (numprocs-1) - (ID+1) + 1;  
+      if (1<=num){
+        MPI_Waitall(num,request_send,stat_send);
+      }
+    }
+    else if ( (ID+1)<=myid ) {
+      MPI_Recv(&u[i], count, MPI_DOUBLE, ID, tag, MPI_Curret_Comm_WD, &stat);
+    }
+
+    s2 = u[i];
+
+    if (measure_time==1){ 
+      dtime(&Etime1);
+      time1a += Etime1 - Stime1;      
+    }
+
+    s1 = u[i+1]*u[i+1];
+    s3 = fabs(s1 + s2);
+
+    if (ABSTOL<fabs(u[i+1])){
+      if (u[i+1]<0.0)    s3 =  sqrt(s3);
+      else               s3 = -sqrt(s3);
+    }
+    else{
+      s3 = sqrt(s3);
+    }
+
+    if (ABSTOL<fabs(s2)){
+
+      if (measure_time==1) dtime(&Stime1);
+
+      ss = u[i+1];
+      ac[i+1][i] = s3;
+      ac[i][i+1] = s3;
+      u[i+1] = u[i+1] - s3;
+      u1 = s3 * s3 - ss * s3;
+      u2 = 2.0 * u1;
+
+      uu[i] = u2;
+      b1[i] = ss - s3;
+
+      my_r = 0.0;
+      ID0 = row_flag[i+1];
+
+      if ( (i+1)<=is2[myid] || (i+1)<=ie2[myid] ){
+
+	if (is2[myid]<(i+1)) is0 = i + 1;
+	else                 is0 = is2[myid];
+
+	for (i1=is0; i1<=ie2[myid]; i1++){
+ 
+          i2 = i1 + (myid - ID0);
+	  p[i2] = 0.0;
+
+	  for (j=i+1; j<=n; j++){
+	    p[i2] += ac[i1][j] * u[j];
+	  }
+
+	  p[i2] = p[i2] / u1;
+	  my_r += u[i1] * p[i2];
+	}
+
+        p[ie2[myid]+(myid-ID0)+1] = my_r; 
+      }
+
+      if (measure_time==1){
+	dtime(&Etime1);
+	time1b += Etime1 - Stime1;      
+      }
+      
+      if (measure_time==1) dtime(&Stime1);
+
+      /* MPI: p */
+
+      if (ID0<=myid){ 
+
+        /* the number of working processors */
+
+	nump = numprocs - ID0;
+
+        /* sending */
+
+	num0 = ie2[myid] - is2[myid] + 2;
+	k0 = is2[myid] + (myid - ID0);
+
+	for (ID=0; ID<nump; ID++){
+	  IDS = (myid + ID) % nump + ID0; 
+	  MPI_Isend(&p[k0], num0, MPI_DOUBLE, IDS, tag, MPI_Curret_Comm_WD, &request_send[IDS-ID0]);
+	}
+
+        /* receiving */
+
+	for (ID=0; ID<nump; ID++){
+
+	  IDR = (myid - ID + nump) % nump + ID0;
+	  num1 = ie2[IDR] - is2[IDR] + 2;
+	  k1 = is2[IDR] + (IDR - ID0);
+	  MPI_Irecv(&p[k1], num1, MPI_DOUBLE, IDR, tag, MPI_Curret_Comm_WD, &request_recv[IDR-ID0]);
+	}
+
+        /* waitall */
+
+	MPI_Waitall(nump,request_recv,stat_send);
+	MPI_Waitall(nump,request_send,stat_send);
+      }
+
+      if (measure_time==1){
+	dtime(&Etime1);
+	time1c += Etime1-Stime1;      
+      }
+
+      if (measure_time==1) dtime(&Stime1);
+
+      /* calculate r */
+
+      r = 0.0;
+      for (ID=ID0; ID<numprocs; ID++){
+        r += p[ie2[ID]+(ID-ID0)+1];
+      }
+
+      /* shift p */
+      
+      for (ID=(ID0+1); ID<numprocs; ID++){
+        ID1 = ID - ID0;
+	for (i1=is2[ID]; i1<=ie2[ID]; i1++){
+          p[i1] = p[i1+ID1];
+	}
+      }
+
+      /* calculate q */
+
+      if ( (i+1)<=is2[myid] || (i+1)<=ie2[myid] ){
+	r = r / u2;
+	for (i1=i+1; i1<=n; i1++){
+	  q[i1] = p[i1] - r * u[i1];
+	}
+      }
+      
+      if ( (i+1)<=is2[myid] || (i+1)<=ie2[myid] ){
+        
+	if (is2[myid]<(i+1)) is0 = i + 1;
+	else                 is0 = is2[myid];
+        
+	for (i1=is0; i1<=ie2[myid]; i1++){
+          
+	  tmp1 = u[i1];
+	  tmp2 = q[i1];
+
+	  for (j1=i+1; j1<=n; j1++){
+	    ac[i1][j1] -= tmp1 * q[j1] + tmp2 * u[j1];
+	  }
+
+	}
+      }
+
+      if (measure_time==1){
+	dtime(&Etime1);
+	time1d += Etime1 - Stime1;      
+      }
+
+    }
+
+    if (myid<row_flag[i+1]) break;
+  }
+
+  MPI_Barrier(MPI_Curret_Comm_WD);
+
+  if (measure_time==1) dtime(&Stime1);
+
+  /* broadcast uu, b1, and ac */
+
+  MPI_Bcast(&uu[0],n+1,MPI_DOUBLE,numprocs-1,MPI_Curret_Comm_WD);
+  MPI_Bcast(&b1[0],n+1,MPI_DOUBLE,numprocs-1,MPI_Curret_Comm_WD);
+
+  BroadCast_ReMatrix(MPI_Curret_Comm_WD,ac,n,is2,ie2,myid,numprocs,
+                     stat_send,request_send,request_recv);
+
+  if (measure_time==1){
+    dtime(&Etime1);
+    time1e += Etime1-Stime1;      
+  }
+
+  for (i=1; i<=n; i++){
+    for (j=1; j<=n; j++){
+      ad[i][j] = ac[i][j];
+    }
+  }
+
+  if (measure_time==1){
+    printf("T0  myid=%2d time1a=%15.12f\n",myid,time1a);
+    printf("T0  myid=%2d time1b=%15.12f\n",myid,time1b);
+    printf("T0  myid=%2d time1c=%15.12f\n",myid,time1c);
+    printf("T0  myid=%2d time1d=%15.12f\n",myid,time1d);
+    printf("T0  myid=%2d time1e=%15.12f\n",myid,time1e);
+  }
+  
+  if (measure_time==1){
+    dtime(&Etime);
+    printf("T1 myid=%2d   %15.12f\n",myid,Etime-Stime);
+  }
+
+  /* freeing of arrays */
+
+  free(row_flag);
+  free(is2);
+  free(ie2);
+  free(u);
+  free(q);
+  free(p);
+
+}
+
+
+
+
+
 
 
 
