@@ -330,8 +330,8 @@ void Eigen_HH(double **ac, double *ko, int n, int EVmax)
     Eigen_HH:
 
     Eigen_HH.c is a subroutine to solve a seqular equation without an
-    overlap matrix using Householder method and lapack's dstegr_() or
-    F77_NAME(dstedc,DSTEDC)().
+    overlap matrix using Householder method and lapack's dstevx, dstegr, 
+    or dstedc.
 
     Log of Eigen_HH.c:
 
@@ -339,19 +339,25 @@ void Eigen_HH(double **ac, double *ko, int n, int EVmax)
 
   ***********************************************************************/
 
-  double ABSTOL=1.0e-14;
+  double ABSTOL=1.0e-13;
   double **ad,*D,*E,
                 *b1,*u,*uu,
                 *p,*q,*s,*c,
                 s1,s2,s3,ss,u1,u2,r,p1,
+                s20,s21,s22,s23,
                 xsum,bunbo,si,co,sum,
-                a1,a2,a3,a4,a5,a6,b7,r1,r2,
-                r3,x1,x2,xap,tmp1,tmp2,
+                a1,a2,a3,a4,a5,a6,b7,
+                x1,x2,xap,tmp1,tmp2,
                 bb,bb1,ui,uj,uij;
 
-  int jj,jj1,jj2,k,ii,ll,i3,i2,j2,
+  double ss0,ss1,ss2,ss3;
+  double r0,r1,r2,r3,p10,p11,p12,p13;
+  double tmp10,tmp11,tmp12,tmp13;
+  double tmp20,tmp21,tmp22,tmp23;
+ 
+  int jj,jj1,jj2,k,ii,ll,i3,i2,j2,i1s,
              i,j,i1,j1,n1,n2,ik,
-             jk,po1,nn,count;
+             jk,po1,nn,count,ks;
 
   double Stime, Etime;
   double Stime1, Etime1;
@@ -393,6 +399,9 @@ void Eigen_HH(double **ac, double *ko, int n, int EVmax)
 
   for (i=1; i<=(n-2); i++){
 
+    /* original version */
+
+    /*
     s1 = ac[i+1][i] * ac[i+1][i];
     s2 = 0.0;
     u[i+1] = ac[i+1][i];
@@ -401,7 +410,41 @@ void Eigen_HH(double **ac, double *ko, int n, int EVmax)
       s2 += tmp1*tmp1;
       u[i1] = tmp1;
     }
+    s3 = fabs(s1 + s2);
+    */
 
+    /* unrolling version */
+
+    s1 = ac[i+1][i] * ac[i+1][i];
+    u[i+1] = ac[i+1][i];
+
+    s20 = 0.0;
+    s21 = 0.0;
+    s22 = 0.0;
+    s23 = 0.0;
+
+    for (i1=i+2; i1<=(n-3); i1+=4){
+
+      s20 += ac[i][i1+0]*ac[i][i1+0];
+      s21 += ac[i][i1+1]*ac[i][i1+1];
+      s22 += ac[i][i1+2]*ac[i][i1+2];
+      s23 += ac[i][i1+3]*ac[i][i1+3];
+
+      u[i1+0] = ac[i][i1+0];
+      u[i1+1] = ac[i][i1+1];
+      u[i1+2] = ac[i][i1+2];
+      u[i1+3] = ac[i][i1+3];
+    }
+
+    i1s = n + 1 - (n+1-(i+2))%4;
+
+    for (i1=i1s; ((i+2)<=i1 && i1<=n); i1++){
+      tmp1 = ac[i][i1]; 
+      s20 += tmp1*tmp1;
+      u[i1] = tmp1;
+    }
+
+    s2 = s20 + s21 + s22 + s23;
     s3 = fabs(s1 + s2);
 
     if (ABSTOL<fabs(ac[i+1][i])){
@@ -423,6 +466,9 @@ void Eigen_HH(double **ac, double *ko, int n, int EVmax)
       uu[i] = u2;
       b1[i] = ss - s3;
 
+      /* original version */
+
+      /*
       r = 0.0;
       for (i1=i+1; i1<=n; i1++){
 	p1 = 0.0;
@@ -433,11 +479,79 @@ void Eigen_HH(double **ac, double *ko, int n, int EVmax)
 	r += u[i1] * p[i1];
       }
       r = r / u2;
+      */
 
+      /* unrolling version */
+
+      r0 = 0.0;
+      r1 = 0.0;
+      r2 = 0.0;
+      r3 = 0.0;
+
+      for (i1=i+1; i1<=(n-3); i1+=4){
+
+	p10 = 0.0;
+	p11 = 0.0;
+	p12 = 0.0;
+	p13 = 0.0;
+
+	for (j=i+1; j<=n; j++){
+	  p10 += ac[i1+0][j] * u[j];
+	  p11 += ac[i1+1][j] * u[j];
+	  p12 += ac[i1+2][j] * u[j];
+	  p13 += ac[i1+3][j] * u[j];
+	}
+
+	p[i1+0] = p10 / u1;
+	p[i1+1] = p11 / u1;
+	p[i1+2] = p12 / u1;
+	p[i1+3] = p13 / u1;
+
+	r0 += u[i1+0] * p[i1+0];
+	r1 += u[i1+1] * p[i1+1];
+	r2 += u[i1+2] * p[i1+2];
+	r3 += u[i1+3] * p[i1+3];
+      }
+
+      i1s = n + 1 - (n+1-(i+1))%4;
+
+      for (i1=i1s; ((i+1)<=i1 && i1<=n); i1++){
+	p1 = 0.0;
+	for (j=i+1; j<=n; j++){
+	  p1 += ac[i1][j] * u[j];
+	}
+	p[i1] = p1 / u1;
+	r0 += u[i1] * p[i1];
+      }
+
+      r = (r0+r1+r2+r3) / u2;
+
+      /* original version */
+
+      /*
       for (i1=i+1; i1<=n; i1++){
 	q[i1] = p[i1] - r * u[i1];
       }
+      */
 
+      /* unrolling version */
+
+      for (i1=i+1; i1<=(n-3); i1+=4){
+	q[i1+0] = p[i1+0] - r * u[i1+0];
+	q[i1+1] = p[i1+1] - r * u[i1+1];
+	q[i1+2] = p[i1+2] - r * u[i1+2];
+	q[i1+3] = p[i1+3] - r * u[i1+3];
+      }
+
+      i1s = n + 1 - (n+1-(i+1))%4;
+
+      for (i1=i1s; ((i+1)<=i1 && i1<=n); i1++){
+	q[i1] = p[i1] - r * u[i1];
+      }
+
+      /* original version */
+
+      /*
       for (i1=i+1; i1<=n; i1++){
         tmp1 = u[i1];
         tmp2 = q[i1]; 
@@ -445,6 +559,39 @@ void Eigen_HH(double **ac, double *ko, int n, int EVmax)
 	  ac[i1][j1] -= tmp1 * q[j1] + tmp2 * u[j1];
 	}
       }
+      */
+
+      /* unrolling version */
+
+      for (i1=i+1; i1<=(n-3); i1+=4){
+
+        tmp10 = u[i1+0];
+        tmp11 = u[i1+1];
+        tmp12 = u[i1+2];
+        tmp13 = u[i1+3];
+
+        tmp20 = q[i1+0]; 
+        tmp21 = q[i1+1]; 
+        tmp22 = q[i1+2]; 
+        tmp23 = q[i1+3]; 
+
+	for (j1=i+1; j1<=n; j1++){
+	  ac[i1+0][j1] -= tmp10 * q[j1] + tmp20 * u[j1];
+	  ac[i1+1][j1] -= tmp11 * q[j1] + tmp21 * u[j1];
+	  ac[i1+2][j1] -= tmp12 * q[j1] + tmp22 * u[j1];
+	  ac[i1+3][j1] -= tmp13 * q[j1] + tmp23 * u[j1];
+	}
+      }
+
+      i1s = n + 1 - (n+1-(i+1))%4;
+
+      for (i1=i1s; ((i+1)<=i1 && i1<=n); i1++){
+        tmp1 = u[i1];
+        tmp2 = q[i1]; 
+	for (j1=i+1; j1<=n; j1++){
+	  ac[i1][j1] -= tmp1 * q[j1] + tmp2 * u[j1];
+	}
+      }
 
     }
   }
@@ -496,6 +643,9 @@ void Eigen_HH(double **ac, double *ko, int n, int EVmax)
     ad[i-1][i] = b1[i-1];
   }
 
+  /* original version */
+
+  /*
   for (k=1; k<=EVmax; k++){
     for (nn=2; nn<=n-1; nn++){
       if ( (1.0e-3*ABSTOL)<fabs(uu[n-nn])){
@@ -510,6 +660,59 @@ void Eigen_HH(double **ac, double *ko, int n, int EVmax)
       }
     }
   }
+  */
+
+  /* unrolling version */
+
+  for (k=1; k<=(EVmax-3); k+=4){
+
+    for (nn=2; nn<=n-1; nn++){
+      if ( (1.0e-3*ABSTOL)<fabs(uu[n-nn])){
+
+	ss0 = 0.0;
+	ss1 = 0.0;
+	ss2 = 0.0;
+	ss3 = 0.0;
+
+	for (i=n-nn+1; i<=n; i++){
+	  ss0 += ad[n-nn][i] * ac[k+0][i];
+	  ss1 += ad[n-nn][i] * ac[k+1][i];
+	  ss2 += ad[n-nn][i] * ac[k+2][i];
+	  ss3 += ad[n-nn][i] * ac[k+3][i];
+	}
+
+	ss0 = 2.0*ss0/uu[n-nn];
+	ss1 = 2.0*ss1/uu[n-nn];
+	ss2 = 2.0*ss2/uu[n-nn];
+	ss3 = 2.0*ss3/uu[n-nn];
+
+	for (i=n-nn+1; i<=n; i++){
+	  ac[k+0][i] -= ss0 * ad[n-nn][i];
+	  ac[k+1][i] -= ss1 * ad[n-nn][i];
+	  ac[k+2][i] -= ss2 * ad[n-nn][i];
+	  ac[k+3][i] -= ss3 * ad[n-nn][i];
+	}
+
+      }
+    }
+  }
+
+  ks = EVmax - EVmax%4 + 1;
+
+  for (k=ks; k<=EVmax; k++){
+    for (nn=2; nn<=n-1; nn++){
+      if ( (1.0e-3*ABSTOL)<fabs(uu[n-nn])){
+	ss = 0.0;
+	for (i=n-nn+1; i<=n; i++){
+	  ss += ad[n-nn][i] * ac[k][i];
+	}
+	ss = 2.0*ss/uu[n-nn];
+	for (i=n-nn+1; i<=n; i++){
+	  ac[k][i] -= ss * ad[n-nn][i];
+	}
+      }
+    }
+  }
 
   if (measure_time==1){
     dtime(&Etime);
