@@ -27,7 +27,7 @@ void array0()
 {
   int i,j,k,ii,l,L,n,ct_AN,h_AN,wan,al,tno,Cwan;
   int tno0,tno1,Mc_AN,Gc_AN,Gh_AN,Hwan,m,so,s1,s2;
-  int q_AN,Gq_AN,Qwan,Lmax,spe,ns,nc,spin;
+  int q_AN,Gq_AN,Qwan,Lmax,spe,ns,nc,spin,fan;
   int num,n2,wanA,Gi,Mc_AN_GDC,MAnum;
   int Anum,p,vsize,NUM;
   int numprocs,myid,ID;
@@ -41,6 +41,7 @@ void array0()
   /* allocate in truncation.c */
 
   if (alloc_first[0]==0){
+
     FNAN[0] = 0; 
     for (Mc_AN=0; Mc_AN<=Matomnum; Mc_AN++){
 
@@ -48,23 +49,32 @@ void array0()
       else          Gc_AN = M2G[Mc_AN];
 
       for (h_AN=0; h_AN<=FNAN[Gc_AN]; h_AN++){
-        free(GListTAtoms0[Mc_AN][h_AN]);
-        free(GListTCells0[Mc_AN][h_AN]);
-        free(GListTAtoms1[Mc_AN][h_AN]);
         free(GListTAtoms2[Mc_AN][h_AN]);
-        free(GListTAtoms3[Mc_AN][h_AN]);
+        free(GListTAtoms1[Mc_AN][h_AN]);
+
+        if (Allocate_TAtoms0==1){
+          free(GListTCells0[Mc_AN][h_AN]);
+          free(GListTAtoms3[Mc_AN][h_AN]);
+          free(GListTAtoms0[Mc_AN][h_AN]);
+	}
       }
-      free(GListTAtoms0[Mc_AN]);
-      free(GListTCells0[Mc_AN]);
-      free(GListTAtoms1[Mc_AN]);
       free(GListTAtoms2[Mc_AN]);
-      free(GListTAtoms3[Mc_AN]);
+      free(GListTAtoms1[Mc_AN]);
+
+      if (Allocate_TAtoms0==1){
+        free(GListTCells0[Mc_AN]);
+        free(GListTAtoms3[Mc_AN]);
+        free(GListTAtoms0[Mc_AN]);
+      }
     }
-    free(GListTAtoms0);
-    free(GListTCells0);
-    free(GListTAtoms1);
     free(GListTAtoms2);
-    free(GListTAtoms3);
+    free(GListTAtoms1);
+
+    if (Allocate_TAtoms0==1){
+      free(GListTCells0);
+      free(GListTAtoms3);
+      free(GListTAtoms0);
+    }
   }
 
   if (alloc_first[1]==0){
@@ -73,25 +83,34 @@ void array0()
   /* Allocation in UCell_Box() of truncation.c */
 
   if (alloc_first[2]==0){
+
     for (Mc_AN=0; Mc_AN<=(Matomnum+MatomnumF); Mc_AN++){
-      free(GridListAtom[Mc_AN]);
-      free(CellListAtom[Mc_AN]);
       free(MGridListAtom[Mc_AN]);
     }
-    free(GridListAtom);
-    free(CellListAtom);
     free(MGridListAtom);
+
+    for (Mc_AN=1; Mc_AN<=Matomnum; Mc_AN++){
+      free(GridListAtom[Mc_AN]);
+    }
+    free(GridListAtom);
+
+    for (Mc_AN=1; Mc_AN<=(Matomnum+MatomnumF); Mc_AN++){
+      free(CellListAtom[Mc_AN]);
+    }
+    free(CellListAtom);
   }
 
   /* Allocation in truncation.c */
-
+  
   if (alloc_first[3]==0){
 
     if (SpinP_switch==3){ /* spin non-collinear */
+
       for (k=0; k<=3; k++){
         free(Density_Grid[k]);
       }
       free(Density_Grid);
+
     }
     else{ 
       for (k=0; k<=1; k++){
@@ -1367,23 +1386,34 @@ void array0()
 
       for (k=0; k<4; k++){
 
-        if (k<=1)  MAnum = Matomnum + MatomnumF + 1;
-        else       MAnum = Matomnum + 1;
-
 	FNAN[0] = 0;
-	for (Mc_AN=0; Mc_AN<MAnum; Mc_AN++){
+	for (Mc_AN=0; Mc_AN<(Matomnum+2); Mc_AN++){
           
 	  if (Mc_AN==0){
 	    Gc_AN = 0;
 	    tno0 = 1;
+	    fan = FNAN[Gc_AN];
+	  }
+  	  else if ( (Matomnum+1)<=Mc_AN ){
+	    fan = List_YOUSO[8];
+	    tno0 = List_YOUSO[7];
 	  }
 	  else{
 	    Gc_AN = F_M2G[Mc_AN];
 	    Cwan = WhatSpecies[Gc_AN];
 	    tno0 = Spe_Total_NO[Cwan];  
+	    fan = FNAN[Gc_AN];
 	  }
           
-	  for (h_AN=0; h_AN<=FNAN[Gc_AN]; h_AN++){
+	  for (h_AN=0; h_AN<(fan+1); h_AN++){
+
+	    if (Mc_AN==0){
+	      tno1 = 1;  
+	    }
+	    else{
+	      tno1 = (List_YOUSO[35]+1)*(List_YOUSO[35]+1)*List_YOUSO[34];
+	    } 
+
 	    for (i=0; i<tno0; i++){
 	      free(DS_VNA[k][Mc_AN][h_AN][i]);
 	    }
@@ -1395,32 +1425,44 @@ void array0()
       }
       free(DS_VNA);
 
+      /* CntDS_VNA */  
+
       if (Cnt_switch==1){
 
 	for (k=0; k<4; k++){
 
-          if (k<=1)  MAnum = Matomnum + MatomnumF + 1;
-          else       MAnum = Matomnum + 1;
-
 	  FNAN[0] = 0;
-  	  for (Mc_AN=0; Mc_AN<MAnum; Mc_AN++){
+	  for (Mc_AN=0; Mc_AN<(Matomnum+2); Mc_AN++){
           
 	    if (Mc_AN==0){
 	      Gc_AN = 0;
 	      tno0 = 1;
+	      fan = FNAN[Gc_AN];
+	    }
+	    else if ( Mc_AN==(Matomnum+1) ){
+	      fan = List_YOUSO[8];
+	      tno0 = List_YOUSO[7];
 	    }
 	    else{
 	      Gc_AN = F_M2G[Mc_AN];
 	      Cwan = WhatSpecies[Gc_AN];
-	      tno0 = Spe_Total_CNO[Cwan];  
+	      tno0 = Spe_Total_CNO[Cwan]; 
+              fan = FNAN[Gc_AN];
 	    }
-          
-	    for (h_AN=0; h_AN<=FNAN[Gc_AN]; h_AN++){
+
+	    for (h_AN=0; h_AN<(fan+1); h_AN++){
+
+	      if (Mc_AN==0){
+		tno1 = 1;  
+	      }
+	      else{
+		tno1 = (List_YOUSO[35]+1)*(List_YOUSO[35]+1)*List_YOUSO[34];
+	      } 
 
 	      for (i=0; i<tno0; i++){
 		free(CntDS_VNA[k][Mc_AN][h_AN][i]);
 	      }
-              free(CntDS_VNA[k][Mc_AN][h_AN]);
+ 	      free(CntDS_VNA[k][Mc_AN][h_AN]);
 	    }
             free(CntDS_VNA[k][Mc_AN]);
 	  }
@@ -1620,6 +1662,7 @@ void array0()
   }
 
   if (alloc_first[12]==0){
+
     for (ID=0; ID<numprocs; ID++){
       free(Rcv_GAN[ID]); 
     }
@@ -1675,6 +1718,7 @@ void array0()
     free(Num_Rcv_FNAN2_Grid);
     free(Num_Snd_FNAN2_Grid);
     free(Rcv_FNAN2_MN);
+    free(Rcv_FNAN2_GRc);
     free(Rcv_FNAN2_GA);
     free(TopMAN2_Grid);
 
@@ -1689,6 +1733,29 @@ void array0()
     free(Snd_FNAN2_Nc);
   }
 
+  if (alloc_first[22]==0){
+
+    for (ID=0; ID<numprocs; ID++){
+      free(Pro_Snd_GAtom[ID]);
+    }
+    free(Pro_Snd_GAtom);
+
+    for (ID=0; ID<numprocs; ID++){
+      free(Pro_Snd_MAtom[ID]);
+    }
+    free(Pro_Snd_MAtom);
+
+    for (ID=0; ID<numprocs; ID++){
+      free(Pro_Snd_LAtom[ID]);
+    }
+    free(Pro_Snd_LAtom);
+
+    for (ID=0; ID<numprocs; ID++){
+      free(Pro_Snd_LAtom2[ID]);
+    }
+    free(Pro_Snd_LAtom2);
+  }
+
   /* allocation in Set_BasisPara() of SetPara_DFT.c*/
 
   for (i=0; i<List_YOUSO[18]; i++){
@@ -1735,6 +1802,13 @@ void array0()
 
   }
 
+  if (alloc_first[23]==0){
+    free(NE_T_k_op);
+    free(NE_KGrids3);
+    free(NE_KGrids2);
+    free(NE_KGrids1);
+  }
+
   /* Allocation_Arrays(0) */
 
   /* arrays for LDA+U added by MJ */
@@ -1789,10 +1863,14 @@ void array0()
   free(S_Snd_Num);
   free(F_Rcv_Num);
   free(S_Rcv_Num);
+  free(F_Snd_Num_WK);
+  free(F_Rcv_Num_WK);
   free(F_TopMAN);
   free(S_TopMAN);
   free(Snd_DS_NL_Size);
   free(Rcv_DS_NL_Size);
+  free(Snd_HFS_Size);
+  free(Rcv_HFS_Size);
   free(Start_Grid1);
   free(End_Grid1);
   free(Start_Grid2);
@@ -1804,36 +1882,6 @@ void array0()
   }
   free(EH0_scaling);
 
-  /* exchange splitting matrix */
-  if (ESM_flag==1){
-
-    for (k=0; k<Num_ESM; k++){
-      Gc_AN = ESM_Atoms[k];
-
-      for (h_AN=0; h_AN<=FNAN[Gc_AN]; h_AN++){
-	Gh_AN = natn[Gc_AN][h_AN];
-	Hwan = WhatSpecies[Gh_AN];
-	tno0 = Spe_Total_CNO[Hwan];
-
-	for (q_AN=0; q_AN<=FNAN[Gc_AN]; q_AN++){
-	  Gq_AN = natn[Gc_AN][q_AN];
-	  Qwan = WhatSpecies[Gq_AN];
-	  tno1 = Spe_Total_CNO[Qwan];
-
-	  for (i=0; i<tno0; i++){
-	    free(ESM[k][h_AN][q_AN][i]);
-	  }
-          free(ESM[k][h_AN][q_AN]);
-	}
-        free(ESM[k][h_AN]);
-      }
-      free(ESM[k]);
-    }
-    free(ESM);
-
-    free(ESM_Atoms);
-  }
-
   /* Allocation_Arrays(1) */
 
   for (i=0; i<(atomnum+1); i++){
@@ -1887,9 +1935,33 @@ void array0()
   free(S_G2M);
   free(time_per_atom);
 
-  if (MD_switch==10){
-    free(DIAG_LBFGS);
-    free(W_LBFGS);
+   /* EF */
+
+  if (MD_switch==4){
+    for (i=0; i<(3*atomnum+2); i++){
+      free(Hessian[i]);
+    }
+    free(Hessian);
+  }
+
+  /* BFGS */
+
+  if (MD_switch==5){
+
+    for (i=0; i<(3*atomnum+2); i++){
+      free(InvHessian[i]);
+    }
+    free(InvHessian);
+  }
+
+  /* RF by hmweng */
+
+  if (MD_switch==6){
+
+    for (i=0; i<(3*atomnum+2); i++){
+      free(Hessian[i]);
+    }
+    free(Hessian);
   }
 
   /* spin non-collinear */
@@ -2215,6 +2287,88 @@ void array0()
     free(Band_kname);
   }
 
+  /* hmweng */
+  /* Those arrays were allocated in Allocate_Array(9). */ 
+
+  if (alloc_first[25]==0){
+
+    for(i=0; i<Wannier_Num_Kinds_Projectors; i++){
+      free(Wannier_Select_Matrix[i]);
+    }
+    free(Wannier_Select_Matrix);
+
+    for(i=0; i<Wannier_Num_Kinds_Projectors; i++){
+      for(j=0;j<Wannier_Num_Pro[i];j++){
+	free(Wannier_Projector_Hybridize_Matrix[i][j]);
+      }
+      free(Wannier_Projector_Hybridize_Matrix[i]);
+    }
+    free(Wannier_Projector_Hybridize_Matrix);
+  }
+
+  /* hmweng */
+  /* Those arrays were allocated in Allocate_Array(8). */ 
+
+  if (alloc_first[24]==0){
+
+    for (i=0; i<Wannier_Num_Kinds_Projectors; i++){
+      free(Wannier_ProSpeName[i]);
+    }  
+    free(Wannier_ProSpeName);
+
+    for (i=0; i<Wannier_Num_Kinds_Projectors; i++){
+      free(Wannier_ProName[i]);
+    }  
+    free(Wannier_ProName);
+
+    for (i=0; i<Wannier_Num_Kinds_Projectors; i++){
+      free(Wannier_Pos[i]);
+    }
+    free(Wannier_Pos);
+
+    for (i=0; i<Wannier_Num_Kinds_Projectors; i++){
+      free(Wannier_X_Direction[i]);
+    }
+    free(Wannier_X_Direction);
+
+    for (i=0; i<Wannier_Num_Kinds_Projectors; i++){
+      free(Wannier_Z_Direction[i]);
+    }
+    free(Wannier_Z_Direction);
+
+    free(Wannier_Num_Pro);
+
+    for (i=0; i<Wannier_Num_Kinds_Projectors; i++){
+      free(Wannier_NumL_Pro[i]);
+    }
+    free(Wannier_NumL_Pro);
+
+    free(WannierPro2SpeciesNum);
+
+    for(i=0;i<3;i++){
+      free(Wannier_Guide[i]);
+    }
+    free(Wannier_Guide);
+
+    for (i=0; i<Wannier_Num_Kinds_Projectors; i++){
+      free(Wannier_Euler_Rotation_Angle[i]);
+    } 
+    free(Wannier_Euler_Rotation_Angle);
+
+    for (i=0; i<Wannier_Num_Kinds_Projectors; i++){
+      for(j=0; j<4; j++){
+	for(l=0; l<2*j+1; l++){
+	  free(Wannier_RotMat_for_Real_Func[i][j][l]);
+	}
+	free(Wannier_RotMat_for_Real_Func[i][j]);
+      }
+      free(Wannier_RotMat_for_Real_Func[i]);
+    }
+    free(Wannier_RotMat_for_Real_Func);
+      
+    free(Wannier_ProName2Num);
+  }
+
 }
 
 void array1()
