@@ -2309,7 +2309,7 @@ static double GDC_NonCol(
     ************************************************/
 
     NUM1 = 2*NUM - (P_min - 1);
-    EigenBand_lapack(C, ko, NUM1);
+    EigenBand_lapack(C, ko, NUM1, 1);
 
     /* C to H (transposition) */
 
