@@ -1860,7 +1860,7 @@ static double GDC_Dosout_NonCol(double *****Hks,
       Msize1[Mh_AN_GDC] = NUM1;
     }
 
-    EigenBand_lapack(C, ko, NUM1);
+    EigenBand_lapack(C, ko, NUM1, 1);
 
     for (i1=1; i1<=NUM1; i1++){
       for (j1=1; j1<=NUM1; j1++){
