@@ -32,7 +32,7 @@ void Mulliken_Charge( char *mode )
   double MulP[4],My_Total_SpinS;
   double MulP_LA[4],MulP_LB[4],MagML;
   double summx,summy,summz;
-  double x,y,z;
+  double x,y,z,TZ;
   double My_Total_SpinSx;
   double My_Total_SpinSy;
   double My_Total_SpinSz;
@@ -40,7 +40,8 @@ void Mulliken_Charge( char *mode )
   double My_Total_OrbitalMomentx;
   double My_Total_OrbitalMomenty;
   double My_Total_OrbitalMomentz;
-  double sden,tmp,tmp0,tmp1,tmp2;
+  double sden,tmp,tmp0,tmp1,tmp2;  
+  double Total_Mul_up,Total_Mul_dn;
   double tmpA0,tmpA1,tmpA2;
   double tmpB0,tmpB1,tmpB2;
   double Re11,Re22,Re12,Im12,d1,d2,d3;
@@ -57,6 +58,14 @@ void Mulliken_Charge( char *mode )
   int numprocs,myid,ID;
   char buf[fp_bsize];          /* setvbuf */
 
+  /* calculate TZ */
+
+  TZ = 0.0;
+  for (i=1; i<=atomnum; i++){
+    wan1 = WhatSpecies[i];
+    TZ = TZ + Spe_Core_Charge[wan1];
+  }
+
   /* allocation of arrays */
 
   DecMulP = (double***)malloc(sizeof(double**)*(SpinP_switch+1));
@@ -231,7 +240,6 @@ void Mulliken_Charge( char *mode )
         
       theta[0] = Angle0_Spin[Gc_AN];
       phi[0]   = Angle1_Spin[Gc_AN];
-      
       sden = 0.5*(InitN_USpin[Gc_AN] - InitN_DSpin[Gc_AN]);
       My_Total_SpinSx += sden*sin(theta[0])*cos(phi[0]);
       My_Total_SpinSy += sden*sin(theta[0])*sin(phi[0]);
@@ -322,10 +330,17 @@ void Mulliken_Charge( char *mode )
   ****************************************/
 
   if (myid==Host_ID && stdout_MulP && strcasecmp(mode,"stdout")==0 ) {
+
+    Total_Mul_up = 0.0;
+    Total_Mul_dn = 0.0;
+
     for (Gc_AN=1; Gc_AN<=atomnum; Gc_AN++){
 
       if (SpinP_switch==0){
 
+        Total_Mul_up += InitN_USpin[Gc_AN];
+        Total_Mul_dn += InitN_USpin[Gc_AN];
+
         printf(" Atom%4d MulP %7.4f%7.4f sum %7.4f\n",
                   Gc_AN,InitN_USpin[Gc_AN],InitN_USpin[Gc_AN],
                   InitN_USpin[Gc_AN]+InitN_USpin[Gc_AN]);
@@ -339,6 +354,8 @@ void Mulliken_Charge( char *mode )
       }
       else if (SpinP_switch==1){
 
+        Total_Mul_up += InitN_USpin[Gc_AN];
+        Total_Mul_dn += InitN_DSpin[Gc_AN];
 
         printf(" Atom%4d MulP %7.4f %7.4f sum %7.4f diff %7.4f\n",
                   Gc_AN,InitN_USpin[Gc_AN],InitN_DSpin[Gc_AN],
@@ -371,6 +388,9 @@ void Mulliken_Charge( char *mode )
 
         xyz2spherical( x,y,z, 0.0,0.0,0.0, S_coordinate ); 
 
+        Total_Mul_up += InitN_USpin[Gc_AN];
+        Total_Mul_dn += InitN_DSpin[Gc_AN];
+
         printf(" Atom%4d MulP%5.2f%5.2f sum %5.2f diff %5.2f (%6.2f %6.2f)  Ml %4.2f (%6.2f %6.2f)  Ml+s %4.2f (%6.2f %6.2f)\n",
                   Gc_AN,InitN_USpin[Gc_AN],InitN_DSpin[Gc_AN],
                   InitN_USpin[Gc_AN]+InitN_DSpin[Gc_AN],
@@ -383,6 +403,12 @@ void Mulliken_Charge( char *mode )
 
       }
     }
+
+    printf(" Sum of MulP: up   =%12.5f down          =%12.5f\n",
+             Total_Mul_up,Total_Mul_dn);
+    printf("              total=%12.5f ideal(neutral)=%12.5f\n",
+                          Total_Mul_up+Total_Mul_dn,TZ);     
+
   }
 
   /****************************************
@@ -391,6 +417,9 @@ void Mulliken_Charge( char *mode )
 
   if ( myid==Host_ID && strcasecmp(mode,"write")==0 ){
 
+    Total_Mul_up = 0.0;
+    Total_Mul_dn = 0.0;
+
     sprintf(file_MC,"%s%s.MC",filepath,filename);
 
     if ((fp_MC = fopen(file_MC,"w")) != NULL){
@@ -421,8 +450,18 @@ void Mulliken_Charge( char *mode )
                   InitN_USpin[Gc_AN]+InitN_DSpin[Gc_AN],
                   InitN_USpin[Gc_AN]-InitN_DSpin[Gc_AN],
                   Angle0_Spin[Gc_AN]/PI*180.0, Angle1_Spin[Gc_AN]/PI*180.0 );
+
+          Total_Mul_up += InitN_USpin[Gc_AN];
+          Total_Mul_dn += InitN_DSpin[Gc_AN];
+
         }
 
+        fprintf(fp_MC,"\n");
+        fprintf(fp_MC," Sum of MulP: up   =%12.5f down          =%12.5f\n",
+                 Total_Mul_up,Total_Mul_dn);
+        fprintf(fp_MC,"              total=%12.5f ideal(neutral)=%12.5f\n",
+                              Total_Mul_up+Total_Mul_dn,TZ);     
+
       }
       else{
 
@@ -435,7 +474,16 @@ void Mulliken_Charge( char *mode )
                   Gc_AN,SpeName[wan1],InitN_USpin[Gc_AN],InitN_DSpin[Gc_AN],
                   InitN_USpin[Gc_AN]+InitN_DSpin[Gc_AN],
                   InitN_USpin[Gc_AN]-InitN_DSpin[Gc_AN]);
+
+          Total_Mul_up += InitN_USpin[Gc_AN];
+          Total_Mul_dn += InitN_DSpin[Gc_AN];
         }
+
+        fprintf(fp_MC,"\n");
+        fprintf(fp_MC," Sum of MulP: up   =%12.5f down          =%12.5f\n",
+                 Total_Mul_up,Total_Mul_dn);
+        fprintf(fp_MC,"              total=%12.5f ideal(neutral)=%12.5f\n",
+                              Total_Mul_up+Total_Mul_dn,TZ);     
       }
 
       /* decomposed Mulliken charge */
