@@ -106,8 +106,8 @@ double Poisson(int fft_charge_flag,
         find the Hartree potential in real space
   ****************************************************/
   
-  Get_Value_inReal(0,ReV2,ImV2,ReV1,ImV1,dVHart_Grid,dVHart_Grid); 
-  
+  Get_Value_inReal(0,ReV2,ImV2,ReV1,ImV1,dVHart_Grid,dVHart_Grid);
+
   /* for time */
   MPI_Barrier(mpi_comm_level1);
   dtime(&TEtime);
@@ -716,6 +716,15 @@ void FFT_Density(int den_flag,
                        FFT of Dens
   ****************************************************/
 
+  /*
+  n2 = Ngrid2/2;
+  n3 = Ngrid3/2;
+
+  for (n1=0; n1<Ngrid1; n1++){
+    printf("%2d %15.12f %15.12f\n",n1,ReV1[n1][n2][n3],ImV2[n1][n2][n3]);
+  }
+  */
+
   FFT_Poisson(1, 2, Ngrid1, Ngrid2, Ngrid3,
               My_NGrid1_Poisson, My_NGrid2_Poisson,
               -1, -1, -1, ReV1, ImV1, ReV2, ImV2);
@@ -882,4 +891,10 @@ void Get_Value_inReal(int complex_flag,
     }
   }
 
+  /*
+  for (n1=0; n1<Ngrid1; n1++){
+    printf("%2d %18.15f %18.15f\n",n1,ReV1[n1][Ngrid2/2][Ngrid3/2],ImV1[n1][Ngrid2/2][Ngrid3/2]);
+  }
+  */
+
 }
