@@ -166,7 +166,7 @@ void Extp_Charge(int MD_iter, double *extpln_coes)
 
       B[i] = sum; 
     }
-
+  
     /*
     printf("A\n");fflush(stdout);
     for (i=0; i<NumHis; i++){
@@ -276,7 +276,7 @@ void Inverse(int n, double **a, double **ia)
   Eigen_lapack(a0,ko,n+1,n+1);
 
   for (i=1; i<=(n+1); i++){
-    if (fabs(ko[i])<1.0e-10) 
+    if (fabs(ko[i])<1.0e-12) 
       iko[i] = 0.0;
     else    
       iko[i] = 1.0/ko[i];
@@ -378,7 +378,7 @@ int Input_HKS( int MD_iter, double *Uele, double *****CH )
            i_vec[7]!=TNO1           
          )
       {
-        printf("Failed (1) in roading the restart file %s\n",fileHKS); fflush(stdout);     
+        printf("Failed (1) in reading the restart file %s\n",fileHKS); fflush(stdout);     
         my_check = 0;
       }
       
@@ -526,7 +526,7 @@ int Input_HKS( int MD_iter, double *Uele, double *****CH )
       fclose(fp);
     }
     else{
-      printf("Failed (2) in roading the restart file %s\n",fileHKS); fflush(stdout);     
+      printf("Failed (2) in reading the restart file %s\n",fileHKS); fflush(stdout);     
       my_check = 0;
     }
   }
@@ -750,9 +750,12 @@ void Output_HKS(int MD_iter, double *Uele, double *****CH )
 	fclose(fp);
 
 	po = 1;
+
       }
       else{
 	printf("Failed in saving the restart file %s\n",fileHKS);      
+        /* openmx gives up to save the restart file. */
+        po = 1;
       }
 
     } while (po==0);
