@@ -32,6 +32,8 @@ static double ****OLPpox;
 static double ****OLPpoy;
 static double ****OLPpoz;
 
+
+
 void SCF2File(char *mode, char *inputfile)
 {
   int Mc_AN,Gc_AN,tno0,tno1;
@@ -851,55 +853,6 @@ void Output( FILE *fp, char *inputfile )
     }
   }
 
-  /****************************************************
-      exchange splitting matrix
-  ****************************************************/
-
-  if (ESM_flag==1 && myid==Host_ID){
-
-    i_vec[0] = Num_ESM;
-    fwrite(i_vec, sizeof(int), 1, fp);
-    fwrite(ESM_Atoms, sizeof(int), Num_ESM, fp);
-
-    Tmp_Vec = (double*)malloc(sizeof(double)*Num_ESM*List_YOUSO[8]*List_YOUSO[8]
-                                            *List_YOUSO[7]*List_YOUSO[7]);
-    num = 0; 
-    for (k=0; k<Num_ESM; k++){
-      Gc_AN = ESM_Atoms[k];
-
-      for (h_AN=0; h_AN<=FNAN[Gc_AN]; h_AN++){
-
-	Gh_AN = natn[Gc_AN][h_AN];
-	Mh_AN = F_G2M[Gh_AN];
-	Rnh = ncn[Gc_AN][h_AN];
-	Hwan = WhatSpecies[Gh_AN];
-	NO0 = Spe_Total_CNO[Hwan];
-
-	for (q_AN=0; q_AN<=FNAN[Gc_AN]; q_AN++){
-
-	  Gq_AN = natn[Gc_AN][q_AN];
-	  Mq_AN = F_G2M[Gq_AN];
-	  Rnq = ncn[Gc_AN][q_AN];
-	  Qwan = WhatSpecies[Gq_AN];
-	  NO1 = Spe_Total_CNO[Qwan];
-
-	  for (i=0; i<NO0; i++){
-	    for (j=0; j<NO1; j++){
-              Tmp_Vec[num] = ESM[k][h_AN][q_AN][i][j];
-              num++;
-	    }
-	  }
-	}
-      }      
-    }
-
-    fwrite(Tmp_Vec, sizeof(double), num, fp);
-    free(Tmp_Vec);
-  }
-  else if (myid==Host_ID){
-    i_vec[0] = 0;
-    fwrite(i_vec, sizeof(int), 1, fp);
-  }
 }
 
 
