@@ -38,6 +38,7 @@ static void Inverse(int n, double **a, double **ia);
 static void Generate_PAO_Mixed_Basis(int spe);
 static void Generate_VPS_Mixed_Basis(int spe);
 
+static void output_structures();
 
 void SetPara_DFT()
 {
@@ -80,6 +81,15 @@ void SetPara_DFT()
 
   Check_InitDensity();
 
+  Set_Atom_Weight();
+  Set_Atom_Symbol();
+  Set_Comp2Real();
+  Set_BasisPara();
+
+  /* output structures */
+
+  if (myid==Host_ID) output_structures();
+
   if (myid==Host_ID){
     printf("\n");
     printf("*******************************************************\n");   fflush(stdout);
@@ -95,11 +105,6 @@ void SetPara_DFT()
     FT_ProductPAO();
   }
 
-  Set_Atom_Weight();
-  Set_Atom_Symbol();
-  Set_Comp2Real();
-  Set_BasisPara();
-
   /***************************************************
    allocation of arrays:
 
@@ -436,7 +441,11 @@ void Read_PAO(int spe, char *file)
                        read data
   ****************************************************/
 
-  input_int("AtomSpecies",&Spe_WhatAtom[spe],0);
+  input_double("AtomSpecies",&dum,0.0);
+  Spe_WhatAtom[spe] = (int)dum; 
+  /* dv_EH0 temporaliry is used. */
+  dv_EH0[spe] = dum;
+  
   input_int("grid.num.output",&Spe_Num_Mesh_PAO[spe],0);
 
   if (2<=level_stdout){
@@ -527,6 +536,8 @@ void Read_PAO(int spe, char *file)
   ****************************************************/
 
   input_close();
+
+
 }
 
 
@@ -537,7 +548,7 @@ void Read_VPS(int spe, char *file)
   int m,n,k,t,i,j,L,ii,jj,NVPS,LVPS,tmp0,maxL,po;
   int number_vps,local_part_vps,charge_pcc_calc;
   double te,ve,VPS_Rcut,dum,dum0,dum1,r,dumping;
-  double VNA_width,rmin,rmax,Sr,Dr,sum;
+  double VNA_width,rmin,rmax,Sr,Dr,sum,dx;
   double *Vcore,**TmpVNL,**TmpMat;
   double **phi,*phi2,*pe;
   int numprocs,myid;
@@ -579,20 +590,41 @@ void Read_VPS(int spe, char *file)
                        read data
   ****************************************************/
 
-  input_int("AtomSpecies",&tmp0,0);
-  
-  if (tmp0!=Spe_WhatAtom[spe]){
+  input_double("AtomSpecies",&dum0,0.0);
+  tmp0 = (int)dum0;
+
+  if (dv_EH0[spe]!=dum0){
     printf("Not the same atom in PAO and VPS for spe=%2d\n",spe);
+    MPI_Finalize();
     exit(1);
   }
 
-  input_double("total.electron",&te,(double)tmp0);
+  input_double("total.electron",&te,(double)dum0);
   input_double("valence.electron",&ve,(double)0.0);
-  Spe_Core_Charge[spe] = ve + tmp0 - te;
+  Spe_Core_Charge[spe] = ve + dum0 - te;
   input_int("grid.num.output",&Spe_Num_Mesh_VPS[spe],0);
 
-  if (List_YOUSO[22]<=(Spe_Num_Mesh_VPS[spe]+2))
+  if (List_YOUSO[22]<=(Spe_Num_Mesh_VPS[spe]+2)){
     List_YOUSO[22] = Spe_Num_Mesh_VPS[spe] + 2;
+  }
+
+  /****************************************************
+        re-normalization of atomic charge density 
+  ****************************************************/
+
+  if (remake_headfile==0){
+
+    sum = 0.0;
+    dx = Spe_PAO_XV[spe][1] - Spe_PAO_XV[spe][0];
+    for (i=0; i<Spe_Num_Mesh_PAO[spe]; i++){
+      sum += Spe_Atomic_Den[spe][i]*exp(3.0*Spe_PAO_XV[spe][i]);
+    }
+    sum *= 4.0*PI*dx; 
+
+    for (i=0; i<Spe_Num_Mesh_PAO[spe]; i++){
+      Spe_Atomic_Den[spe][i] *= (Spe_Core_Charge[spe]/sum);
+    }
+  }
 
   /**************************************************
       check j-dependency 
@@ -942,7 +974,12 @@ void Read_VPS(int spe, char *file)
 
     for (i=0; i<Spe_Num_Mesh_VPS[spe]; i++){
       r = Spe_VPS_RV[spe][i];
-      Spe_VH_Atom[spe][i] = V_Hart_atom(spe,r);
+
+      /* empty atom */ 
+      if (Spe_WhatAtom[spe]==0)
+        Spe_VH_Atom[spe][i] = 0.0;
+      else 
+        Spe_VH_Atom[spe][i] = V_Hart_atom(spe,r);
 
       /*
      printf("A1 %2d %15.12f %15.12f %15.12f %15.12f\n",
@@ -983,7 +1020,12 @@ void Read_VPS(int spe, char *file)
     for (i=0; i<Spe_Num_Mesh_VPS[spe]; i++){
       r = Spe_VPS_RV[spe][i];
       dumping = 1.0/(1.0+exp( 20.0*(r-Spe_Atom_Cut1[spe])) );
-      Spe_Vna[spe][i] = dumping*(Vcore[i] + Spe_VH_Atom[spe][i]);
+
+      /* empty atom */ 
+      if (Spe_WhatAtom[spe]==0)
+        Spe_Vna[spe][i] = 0.0;
+      else 
+        Spe_Vna[spe][i] = dumping*(Vcore[i] + Spe_VH_Atom[spe][i]);
 
       /*
       printf("%2d %15.12f %15.12f %15.12f %15.12f\n",
@@ -1520,16 +1562,16 @@ void Set_Comp2Real()
 
   /* p */
 
-  Comp2Real[1][0][0] = Complex(-1.0/sqrt(2.0), 0.0);
+  Comp2Real[1][0][0] = Complex( 1.0/sqrt(2.0), 0.0);
   Comp2Real[1][0][1] = Complex( 0.0,           0.0);
-  Comp2Real[1][0][2] = Complex( 1.0/sqrt(2.0), 0.0);
+  Comp2Real[1][0][2] = Complex(-1.0/sqrt(2.0), 0.0);
 
-  Comp2Real[1][1][0] = Complex( 0.0, -1.0/sqrt(2.0));
-  Comp2Real[1][1][1] = Complex( 0.0,           0.0);
-  Comp2Real[1][1][2] = Complex( 0.0, -1.0/sqrt(2.0));
+  Comp2Real[1][1][0] = Complex( 0.0, 1.0/sqrt(2.0));
+  Comp2Real[1][1][1] = Complex( 0.0,          0.0);
+  Comp2Real[1][1][2] = Complex( 0.0, 1.0/sqrt(2.0));
 
   Comp2Real[1][2][0] = Complex( 0.0, 0.0);
-  Comp2Real[1][2][1] = Complex(-1.0, 0.0);
+  Comp2Real[1][2][1] = Complex( 1.0, 0.0);
   Comp2Real[1][2][2] = Complex( 0.0, 0.0);
 
   /* d */
@@ -1569,58 +1611,58 @@ void Set_Comp2Real()
   Comp2Real[3][0][0] = Complex( 0.0,           0.0);
   Comp2Real[3][0][1] = Complex( 0.0,           0.0);
   Comp2Real[3][0][2] = Complex( 0.0,           0.0);
-  Comp2Real[3][0][3] = Complex(-1.0,           0.0);
+  Comp2Real[3][0][3] = Complex( 1.0,           0.0);
   Comp2Real[3][0][4] = Complex( 0.0,           0.0);
   Comp2Real[3][0][5] = Complex( 0.0,           0.0);
   Comp2Real[3][0][6] = Complex( 0.0,           0.0);
 
   Comp2Real[3][1][0] = Complex( 0.0,           0.0);
   Comp2Real[3][1][1] = Complex( 0.0,           0.0);
-  Comp2Real[3][1][2] = Complex(-1.0/sqrt(2.0), 0.0);
+  Comp2Real[3][1][2] = Complex( 1.0/sqrt(2.0), 0.0);
   Comp2Real[3][1][3] = Complex( 0.0,           0.0);
-  Comp2Real[3][1][4] = Complex( 1.0/sqrt(2.0), 0.0);
+  Comp2Real[3][1][4] = Complex(-1.0/sqrt(2.0), 0.0);
   Comp2Real[3][1][5] = Complex( 0.0,           0.0);
   Comp2Real[3][1][6] = Complex( 0.0,           0.0);
 
   Comp2Real[3][2][0] = Complex( 0.0,           0.0);
   Comp2Real[3][2][1] = Complex( 0.0,           0.0);
-  Comp2Real[3][2][2] = Complex( 0.0,-1.0/sqrt(2.0));
+  Comp2Real[3][2][2] = Complex( 0.0, 1.0/sqrt(2.0));
   Comp2Real[3][2][3] = Complex( 0.0,           0.0);
-  Comp2Real[3][2][4] = Complex( 0.0,-1.0/sqrt(2.0));
+  Comp2Real[3][2][4] = Complex( 0.0, 1.0/sqrt(2.0));
   Comp2Real[3][2][5] = Complex( 0.0,           0.0);
   Comp2Real[3][2][6] = Complex( 0.0,           0.0);
 
   Comp2Real[3][3][0] = Complex( 0.0,           0.0);
-  Comp2Real[3][3][1] = Complex(-1.0/sqrt(2.0), 0.0);
+  Comp2Real[3][3][1] = Complex( 1.0/sqrt(2.0), 0.0);
   Comp2Real[3][3][2] = Complex( 0.0,           0.0);
   Comp2Real[3][3][3] = Complex( 0.0,           0.0);
   Comp2Real[3][3][4] = Complex( 0.0,           0.0);
-  Comp2Real[3][3][5] = Complex(-1.0/sqrt(2.0), 0.0);
+  Comp2Real[3][3][5] = Complex( 1.0/sqrt(2.0), 0.0);
   Comp2Real[3][3][6] = Complex( 0.0,           0.0);
 
   Comp2Real[3][4][0] = Complex( 0.0,           0.0);
-  Comp2Real[3][4][1] = Complex( 0.0,-1.0/sqrt(2.0));
+  Comp2Real[3][4][1] = Complex( 0.0, 1.0/sqrt(2.0));
   Comp2Real[3][4][2] = Complex( 0.0,           0.0);
   Comp2Real[3][4][3] = Complex( 0.0,           0.0);
   Comp2Real[3][4][4] = Complex( 0.0,           0.0);
-  Comp2Real[3][4][5] = Complex( 0.0, 1.0/sqrt(2.0));
+  Comp2Real[3][4][5] = Complex( 0.0,-1.0/sqrt(2.0));
   Comp2Real[3][4][6] = Complex( 0.0,           0.0);
 
-  Comp2Real[3][5][0] = Complex(-1.0/sqrt(2.0), 0.0);
+  Comp2Real[3][5][0] = Complex( 1.0/sqrt(2.0), 0.0);
   Comp2Real[3][5][1] = Complex( 0.0,           0.0);
   Comp2Real[3][5][2] = Complex( 0.0,           0.0);
   Comp2Real[3][5][3] = Complex( 0.0,           0.0);
   Comp2Real[3][5][4] = Complex( 0.0,           0.0);
   Comp2Real[3][5][5] = Complex( 0.0,           0.0);
-  Comp2Real[3][5][6] = Complex( 1.0/sqrt(2.0), 0.0);
+  Comp2Real[3][5][6] = Complex(-1.0/sqrt(2.0), 0.0);
 
-  Comp2Real[3][6][0] = Complex( 0.0,-1.0/sqrt(2.0));
+  Comp2Real[3][6][0] = Complex( 0.0, 1.0/sqrt(2.0));
   Comp2Real[3][6][1] = Complex( 0.0,           0.0);
   Comp2Real[3][6][2] = Complex( 0.0,           0.0);
   Comp2Real[3][6][3] = Complex( 0.0,           0.0);
   Comp2Real[3][6][4] = Complex( 0.0,           0.0);
   Comp2Real[3][6][5] = Complex( 0.0,           0.0);
-  Comp2Real[3][6][6] = Complex( 0.0,-1.0/sqrt(2.0));
+  Comp2Real[3][6][6] = Complex( 0.0, 1.0/sqrt(2.0));
 
   /* g(4), h(5), i(6), j(7), k(8), l(9), and m(10) */
 
@@ -1632,34 +1674,34 @@ void Set_Comp2Real()
       }
     }
 
-    Comp2Real[L][0][L] = Complex(-1.0, 0.0);
+    Comp2Real[L][0][L] = Complex(1.0, 0.0);
 
     j = -1;
     for (i=1; i<(L*2+1); i=i+4){
       j++; 
-      Comp2Real[L][i][L-(2*j+1)]  = Complex(-1.0/sqrt(2.0), 0.0);
-      Comp2Real[L][i][L+(2*j+1)]  = Complex( 1.0/sqrt(2.0), 0.0);
+      Comp2Real[L][i][L-(2*j+1)]  = Complex( 1.0/sqrt(2.0), 0.0);
+      Comp2Real[L][i][L+(2*j+1)]  = Complex(-1.0/sqrt(2.0), 0.0);
     }
 
     j = 0;
     for (i=3; i<(L*2+1); i=i+4){
       j++;
-      Comp2Real[L][i][L-2*j]  = Complex(-1.0/sqrt(2.0), 0.0);
-      Comp2Real[L][i][L+2*j]  = Complex(-1.0/sqrt(2.0), 0.0);
+      Comp2Real[L][i][L-2*j]  = Complex(1.0/sqrt(2.0), 0.0);
+      Comp2Real[L][i][L+2*j]  = Complex(1.0/sqrt(2.0), 0.0);
     }
 
     j = -1;
     for (i=2; i<(L*2+1); i=i+4){
       j++;
-      Comp2Real[L][i][L-(2*j+1)]  = Complex(0.0, -1.0/sqrt(2.0));
-      Comp2Real[L][i][L+(2*j+1)]  = Complex(0.0, -1.0/sqrt(2.0));
+      Comp2Real[L][i][L-(2*j+1)]  = Complex(0.0, 1.0/sqrt(2.0));
+      Comp2Real[L][i][L+(2*j+1)]  = Complex(0.0, 1.0/sqrt(2.0));
     }
 
     j = 0;
     for (i=4; i<(L*2+1); i=i+4){
       j++;
-      Comp2Real[L][i][L-2*j]  = Complex(0.0, -1.0/sqrt(2.0));
-      Comp2Real[L][i][L+2*j]  = Complex(0.0,  1.0/sqrt(2.0));
+      Comp2Real[L][i][L-2*j]  = Complex(0.0,  1.0/sqrt(2.0));
+      Comp2Real[L][i][L+2*j]  = Complex(0.0, -1.0/sqrt(2.0));
     }
   }
 
@@ -2119,3 +2161,172 @@ static void Generate_VPS_Mixed_Basis(int spe)
   }
 
 }
+
+
+
+
+
+void output_structures()
+{
+  int i,j,Gc_AN,k,itmp;
+  double tmp[4],Cxyz[4];
+  double lena,lenb,lenc,t1;
+  double alpha,beta,gamma;
+  double CellV,Cell_Volume;
+  char fname1[YOUSO10];
+  FILE *fp;
+
+  /********************************************
+              making of a CIF file
+  *********************************************/
+  
+  sprintf(fname1,"%s%s.cif",filepath,filename);
+
+  Cross_Product(tv[2],tv[3],tmp);
+  CellV = Dot_Product(tv[1],tmp); 
+  Cell_Volume = fabs(CellV);
+
+  lena = sqrt(fabs(Dot_Product(tv[1],tv[1]))); 
+  lenb = sqrt(fabs(Dot_Product(tv[2],tv[2]))); 
+  lenc = sqrt(fabs(Dot_Product(tv[3],tv[3]))); 
+
+  Cross_Product(tv[2],tv[3],tmp);
+  rtv[1][1] = 2.0*PI*tmp[1]/CellV;
+  rtv[1][2] = 2.0*PI*tmp[2]/CellV;
+  rtv[1][3] = 2.0*PI*tmp[3]/CellV;
+  
+  Cross_Product(tv[3],tv[1],tmp);
+  rtv[2][1] = 2.0*PI*tmp[1]/CellV;
+  rtv[2][2] = 2.0*PI*tmp[2]/CellV;
+  rtv[2][3] = 2.0*PI*tmp[3]/CellV;
+  
+  Cross_Product(tv[1],tv[2],tmp);
+  rtv[3][1] = 2.0*PI*tmp[1]/CellV;
+  rtv[3][2] = 2.0*PI*tmp[2]/CellV;
+  rtv[3][3] = 2.0*PI*tmp[3]/CellV;
+
+  /* alpha: angle between b and c in Deg. */
+
+  t1 = Dot_Product(tv[2],tv[3]);
+  if (fabs(t1)<1.0e-14) 
+    alpha = 90.0;
+  else 
+    alpha = acos(t1/(lenb*lenc))/PI*180.0;
+
+  /* beta: angle between c and a in Deg. */
+
+  t1 = Dot_Product(tv[3],tv[1]);
+  if (fabs(t1)<1.0e-14) 
+    beta = 90.0;
+  else 
+    beta = acos(t1/(lenc*lena))/PI*180.0;
+
+  /* gamma: angle between a and b in Deg. */
+
+  t1 = Dot_Product(tv[1],tv[2]);
+  if (fabs(t1)<1.0e-14) 
+    gamma = 90.0;
+  else 
+    gamma = acos(t1/(lena*lenb))/PI*180.0;
+
+  if ((fp = fopen(fname1,"w")) != NULL){
+
+    /* write cell infomations */
+
+    fprintf(fp,"data_%s\n",filename);
+    fprintf(fp,"_audit_creation_date              2007-10-11\n");
+    fprintf(fp,"_audit_creation_method            'Materials Studio'\n");
+
+    fprintf(fp,"_symmetry_space_group_name_H-M    'P1'\n");
+    fprintf(fp,"_symmetry_Int_Tables_number       1\n");
+    fprintf(fp,"_symmetry_cell_setting            triclinic\n");
+
+    fprintf(fp,"loop_\n");
+    fprintf(fp,"_symmetry_equiv_pos_as_xyz\n");
+    fprintf(fp,"  x,y,z\n");
+
+    fprintf(fp,"_cell_length_a%26.4f\n",lena*BohrR);
+    fprintf(fp,"_cell_length_b%26.4f\n",lenb*BohrR);
+    fprintf(fp,"_cell_length_c%26.4f\n",lenc*BohrR);
+
+    fprintf(fp,"_cell_angle_alpha%24.4f\n",alpha);
+    fprintf(fp,"_cell_angle_beta %24.4f\n",beta);
+    fprintf(fp,"_cell_angle_gamma%24.4f\n",gamma);
+
+    fprintf(fp,"loop_\n");
+    fprintf(fp,"_atom_site_label\n");
+    fprintf(fp,"_atom_site_type_symbol\n");
+    fprintf(fp,"_atom_site_fract_x\n");
+    fprintf(fp,"_atom_site_fract_y\n");
+    fprintf(fp,"_atom_site_fract_z\n");
+    fprintf(fp,"_atom_site_Uiso_or_equiv\n");
+    fprintf(fp,"_atom_site_adp_type\n");
+    fprintf(fp,"_atom_site_occupancy\n");
+
+    for (Gc_AN=1; Gc_AN<=atomnum; Gc_AN++){
+
+      /* The zero is taken as the origin of the unit cell. */
+
+      Cxyz[1] = Gxyz[Gc_AN][1];
+      Cxyz[2] = Gxyz[Gc_AN][2];
+      Cxyz[3] = Gxyz[Gc_AN][3];
+
+      Cell_Gxyz[Gc_AN][1] = Dot_Product(Cxyz,rtv[1])*0.5/PI;
+      Cell_Gxyz[Gc_AN][2] = Dot_Product(Cxyz,rtv[2])*0.5/PI;
+      Cell_Gxyz[Gc_AN][3] = Dot_Product(Cxyz,rtv[3])*0.5/PI;
+
+      /* The fractional coordinates are kept within 0 to 1. */
+
+      for (i=1; i<=3; i++){
+
+	itmp = (int)Cell_Gxyz[Gc_AN][i]; 
+
+	if (1.0<Cell_Gxyz[Gc_AN][i]){
+	  Cell_Gxyz[Gc_AN][i] = fabs(Cell_Gxyz[Gc_AN][i] - (double)itmp);
+	}
+	else if (Cell_Gxyz[Gc_AN][i]<-1.0e-13){
+	  Cell_Gxyz[Gc_AN][i] = fabs(Cell_Gxyz[Gc_AN][i] + (double)(abs(itmp)+1));
+	}
+      }
+
+      k = WhatSpecies[Gc_AN];
+      j = Spe_WhatAtom[k];
+
+      fprintf(fp,"A%-6d%-3s%10.5f%10.5f%10.5f%10.5f  Uiso   1.00\n",
+              Gc_AN, 
+	      Atom_Symbol[j],
+	      Cell_Gxyz[Gc_AN][1],
+	      Cell_Gxyz[Gc_AN][2],
+	      Cell_Gxyz[Gc_AN][3],
+              0.0);
+    }
+
+    fclose(fp);
+  }
+
+  /********************************************
+              making of a XYZ file
+  *********************************************/
+
+  sprintf(fname1,"%s%s.xyz",filepath,filename);
+
+  if ((fp = fopen(fname1,"w")) != NULL){
+
+    fprintf(fp,"%d\n\n",atomnum);
+
+    for (Gc_AN=1; Gc_AN<=atomnum; Gc_AN++){
+
+      k = WhatSpecies[Gc_AN];
+      j = Spe_WhatAtom[k];
+
+      fprintf(fp,"%4s  %18.14f %18.14f %18.14f\n",
+	      Atom_Symbol[j],
+	      Gxyz[Gc_AN][1]*BohrR,
+	      Gxyz[Gc_AN][2]*BohrR,
+	      Gxyz[Gc_AN][3]*BohrR);
+    }
+
+    fclose(fp);
+  }
+
+}
