@@ -22,6 +22,13 @@
 #include "mpi.h"
 #endif
 
+#ifdef noomp
+#include "mimic_omp.h"
+#else
+#include <omp.h>
+#endif
+
+
 double Set_Aden_Grid(int init_density)
 {
   /****************************************************
@@ -57,6 +64,9 @@ double Set_Aden_Grid(int init_density)
   MPI_Status stat;
   MPI_Request request;
 
+  /* for OpenMP */
+  int OMPID,Nthrds,Nprocs;
+
   /* MPI */
   MPI_Comm_size(mpi_comm_level1,&numprocs);
   MPI_Comm_rank(mpi_comm_level1,&myid);
@@ -147,30 +157,41 @@ double Set_Aden_Grid(int init_density)
 
     Gc_AN = M2G[Mc_AN];    
     Cwan = WhatSpecies[Gc_AN];
-      
-    for (Nc=0; Nc<GridN_Atom[Gc_AN]; Nc++){
-      
-      GNc = GridListAtom[Mc_AN][Nc];
-      GRc = CellListAtom[Mc_AN][Nc];
-      
-      Get_Grid_XYZ(GNc,Cxyz);
-      dx = Cxyz[1] + atv[GRc][1] - Gxyz[Gc_AN][1];
-      dy = Cxyz[2] + atv[GRc][2] - Gxyz[Gc_AN][2];
-      dz = Cxyz[3] + atv[GRc][3] - Gxyz[Gc_AN][3];
-      
-      r = sqrt(dx*dx + dy*dy + dz*dz); 
+ 
+#pragma omp parallel shared(PCCDen_Grid,PCC_switch,AtomDen_Grid,Cwan,Gxyz,atv,Mc_AN,CellListAtom,GridListAtom,GridN_Atom,Gc_AN) private(OMPID,Nthrds,Nprocs,Nc,GNc,GRc,Cxyz,dx,dy,dz,r,DenA)
+    {
 
-      /* AtomicDenF */
-      DenA = AtomicDenF(Cwan,r);
-      AtomDen_Grid[Mc_AN][Nc] = DenA;
+      /* get info. on OpenMP */ 
+
+      OMPID = omp_get_thread_num();
+      Nthrds = omp_get_num_threads();
+      Nprocs = omp_get_num_procs();
+
+      for (Nc=OMPID*GridN_Atom[Gc_AN]/Nthrds; Nc<(OMPID+1)*GridN_Atom[Gc_AN]/Nthrds; Nc++){
       
-      /*  partial core correction */
-      if (PCC_switch==1) {
-        DenPCC= AtomicPCCF(Cwan,r);
-        PCCDen_Grid[Mc_AN][Nc] = DenPCC;
+	GNc = GridListAtom[Mc_AN][Nc];
+	GRc = CellListAtom[Mc_AN][Nc];
+      
+	Get_Grid_XYZ(GNc,Cxyz);
+	dx = Cxyz[1] + atv[GRc][1] - Gxyz[Gc_AN][1];
+	dy = Cxyz[2] + atv[GRc][2] - Gxyz[Gc_AN][2];
+	dz = Cxyz[3] + atv[GRc][3] - Gxyz[Gc_AN][3];
+      
+	r = sqrt(dx*dx + dy*dy + dz*dz); 
+
+	/* AtomicDenF */
+
+	DenA = AtomicDenF(Cwan,r);
+	AtomDen_Grid[Mc_AN][Nc] = DenA;
+      
+	/*  partial core correction */
+	if (PCC_switch==1) {
+	  PCCDen_Grid[Mc_AN][Nc] = AtomicPCCF(Cwan,r);
+	}
       }
-    }
-    
+
+    } /* #pragma omp parallel */
+
     dtime(&Etime_atom);
     time_per_atom[Gc_AN] += Etime_atom - Stime_atom;
   }
@@ -401,50 +422,54 @@ double Set_Aden_Grid(int init_density)
     }
 
     for (Nc=0; Nc<GridN_Atom[Gc_AN]; Nc++){
+
       MN = MGridListAtom[Mc_AN][Nc];
-      if (0<=MN){ 
+      GRc = CellListAtom[Mc_AN][Nc];
 
-        DenA = AtomDen_Grid[Mc_AN][Nc];
+      if ( (0<=MN && Solver!=4) || (0<=MN && Solver==4 && atv_ijk[GRc][1]==0) ){
 
-        /* spin collinear */
-        if ( init_density==1 && SpinP_switch==1 ){
-          Density_Grid[0][MN] += ocupcy_u*DenA;
-          Density_Grid[1][MN] += ocupcy_d*DenA;
-        } 
+	DenA = AtomDen_Grid[Mc_AN][Nc];
 
-        /* spin non-collinear */
-        else if ( init_density==1 && SpinP_switch==3 ){
+	/* spin collinear */
+	if ( init_density==1 && SpinP_switch==1 ){
+	  Density_Grid[0][MN] += ocupcy_u*DenA;
+	  Density_Grid[1][MN] += ocupcy_d*DenA;
+	} 
 
-          theta = Angle0_Spin[Gc_AN];
-          phi   = Angle1_Spin[Gc_AN];
+	/* spin non-collinear */
+	else if ( init_density==1 && SpinP_switch==3 ){
+
+	  theta = Angle0_Spin[Gc_AN];
+	  phi   = Angle1_Spin[Gc_AN];
           
-          rho = DenA;
-          mag = (ocupcy_u - ocupcy_d)*DenA;             
-          magx = mag*sin(theta)*cos(phi);
-          magy = mag*sin(theta)*sin(phi);
-          magz = mag*cos(theta);
+	  rho = DenA;
+	  mag = (ocupcy_u - ocupcy_d)*DenA;             
+	  magx = mag*sin(theta)*cos(phi);
+	  magy = mag*sin(theta)*sin(phi);
+	  magz = mag*cos(theta);
 
-          Density_Grid[0][MN] += rho;
-          Density_Grid[1][MN] += magx;
-          Density_Grid[2][MN] += magy;
-          Density_Grid[3][MN] += magz;
-        } 
+	  Density_Grid[0][MN] += rho;
+	  Density_Grid[1][MN] += magx;
+	  Density_Grid[2][MN] += magy;
+	  Density_Grid[3][MN] += magz;
+	} 
 
-        else if (init_density==1){
-          Density_Grid[0][MN] += 0.5*DenA;
-        }
-        else{
-          ADensity_Grid[MN] += DenA;
-        }
+	else if (init_density==1){
+	  Density_Grid[0][MN] += 0.5*DenA;
+	}
+	else{
+	  ADensity_Grid[MN] += DenA;
+	}
 
-        /*  partial core correction  */
-        if (PCC_switch==1) {
-          DenPCC = PCCDen_Grid[Mc_AN][Nc];
-          /* later add this in Set_XC_Grid */
-          PCCDensity_Grid[MN] += 0.5*DenPCC;
-        }
-      }
-    }
+	/*  partial core correction  */
+	if (PCC_switch==1) {
+	  DenPCC = PCCDen_Grid[Mc_AN][Nc];
+	  /* later add this in Set_XC_Grid */
+	  PCCDensity_Grid[MN] += 0.5*DenPCC;
+	}
+
+      } 
+    } /* Nc */
 
     dtime(&Etime_atom);
     time_per_atom[Gc_AN] += Etime_atom - Stime_atom;
@@ -508,41 +533,54 @@ double Set_Aden_Grid(int init_density)
   /* Density_Grid += AtomDen2_Grid */
 
   for (i=0; i<FNAN2_Grid; i++){
+
     DenA = AtomDen2_Grid[i];
     MN    = Rcv_FNAN2_MN[i];
     Gc_AN = Rcv_FNAN2_GA[i];
+    GRc   = Rcv_FNAN2_GRc[i];
     Nele = InitN_USpin[Gc_AN] + InitN_DSpin[Gc_AN];
     Nu = InitN_USpin[Gc_AN];
     Nd = InitN_DSpin[Gc_AN];
-    ocupcy_u = Nu/Nele;
-    ocupcy_d = Nd/Nele;
 
-    /* spin collinear */
-    if ( init_density==1 && SpinP_switch==1 ){
-      Density_Grid[0][MN] += ocupcy_u*DenA;
-      Density_Grid[1][MN] += ocupcy_d*DenA;
-    } 
+    if (1.0e-15<Nele){
+      ocupcy_u = Nu/Nele;
+      ocupcy_d = Nd/Nele;
+    }
+    else{
+      ocupcy_u = 0.0;
+      ocupcy_d = 0.0;
+    }
 
-    /* spin non-collinear */
-    else if ( init_density==1 && SpinP_switch==3 ){
+    if (Solver!=4 || (Solver==4 && atv_ijk[GRc][1]==0 )){
 
-      theta = Angle0_Spin[Gc_AN];
-      phi   = Angle1_Spin[Gc_AN];
+      /* spin collinear */
+      if ( init_density==1 && SpinP_switch==1 ){
+	Density_Grid[0][MN] += ocupcy_u*DenA;
+	Density_Grid[1][MN] += ocupcy_d*DenA;
+      } 
 
-      rho = DenA;
-      mag = (ocupcy_u - ocupcy_d)*DenA;
-      magx = mag*sin(theta)*cos(phi);
-      magy = mag*sin(theta)*sin(phi);
-      magz = mag*cos(theta);
+      /* spin non-collinear */
+      else if ( init_density==1 && SpinP_switch==3 ){
 
-      Density_Grid[0][MN] += rho;
-      Density_Grid[1][MN] += magx;
-      Density_Grid[2][MN] += magy;
-      Density_Grid[3][MN] += magz;
-    } 
+	theta = Angle0_Spin[Gc_AN];
+	phi   = Angle1_Spin[Gc_AN];
+
+	rho = DenA;
+	mag = (ocupcy_u - ocupcy_d)*DenA;
+	magx = mag*sin(theta)*cos(phi);
+	magy = mag*sin(theta)*sin(phi);
+	magz = mag*cos(theta);
+
+	Density_Grid[0][MN] += rho;
+	Density_Grid[1][MN] += magx;
+	Density_Grid[2][MN] += magy;
+	Density_Grid[3][MN] += magz;
+      } 
+
+      else if (init_density==1){
+	Density_Grid[0][MN] += 0.5*DenA;
+      }
 
-    else if (init_density==1){
-      Density_Grid[0][MN] += 0.5*DenA;
     }
   }
 
