@@ -1608,6 +1608,14 @@ void Conventional_Allocation(int MD_iter, int isw, int NL_switch)
     printf("%d: MatomN, WMatomN=%d %f\n",myid, MatomN[myid] , WMatomN[myid] );
 #endif
 
+    /* find Max Matomnum */
+
+    Max_Matomnum = 0;
+    for (ID=0; ID<numprocs; ID++){
+      if (Max_Matomnum<MatomN[ID]) Max_Matomnum = MatomN[ID];
+    }     
+
+    /* print the information */
 
     if (myid==Host_ID && 1<=MD_iter){
       printf("\n");
@@ -1797,59 +1805,63 @@ void Estimate_NL()
 
   BCR2 = BCR*BCR;
 
-  for (i=1; i<=Matomnum; i++){
+#pragma omp parallel shared(level_stdout,NOHS_C,cell_atv,Gxyz,TN_Rn,atomnum,SNAN,FNAN,Spe_Total_CNO,Spe_Atom_Cut1,WhatSpecies,M2G,BCR2,Matomnum) private(i,ct_AN,wanA,rcutA,tno1,j,wanB,rcutB,rcut,rcut2,tno2,Rn,dx,dy,dz,r2)
+  {
+
+    for (i=1; i<=Matomnum; i++){
         
-    ct_AN = M2G[i];
-    wanA = WhatSpecies[ct_AN];
-    rcutA = Spe_Atom_Cut1[wanA];
-    tno1 = Spe_Total_CNO[wanA];
+      ct_AN = M2G[i];
+      wanA = WhatSpecies[ct_AN];
+      rcutA = Spe_Atom_Cut1[wanA];
+      tno1 = Spe_Total_CNO[wanA];
 
-    FNAN[ct_AN] = 0;
-    SNAN[ct_AN] = 0;
+      FNAN[ct_AN] = 0;
+      SNAN[ct_AN] = 0;
 
-    for (j=1; j<=atomnum; j++){
+      for (j=1; j<=atomnum; j++){
 
-      wanB = WhatSpecies[j];
-      rcutB = Spe_Atom_Cut1[wanB];
-      rcut = rcutA + rcutB;
-      rcut2 = rcut*rcut;
-      tno2 = Spe_Total_CNO[wanB];
+	wanB = WhatSpecies[j];
+	rcutB = Spe_Atom_Cut1[wanB];
+	rcut = rcutA + rcutB;
+	rcut2 = rcut*rcut;
+	tno2 = Spe_Total_CNO[wanB];
 
-      for (Rn=0; Rn<TN_Rn; Rn++){
+	for (Rn=0; Rn<TN_Rn; Rn++){
 
-	if ((ct_AN==j) && Rn==0){
+	  if ((ct_AN==j) && Rn==0){
 
-          FNAN[ct_AN] += tno1*tno2;  
-          SNAN[ct_AN] += tno2;
-        }
+	    FNAN[ct_AN] += tno1*tno2;  
+	    SNAN[ct_AN] += tno2;
+	  }
  
-	else{
+	  else{
 
-	  if (((Rn==0)&&(ct_AN!=j)) || Rn!=0){
-	    dx = fabs(Gxyz[ct_AN][1] - Gxyz[j][1] - cell_atv[Rn][1]);
-	    dy = fabs(Gxyz[ct_AN][2] - Gxyz[j][2] - cell_atv[Rn][2]);
-	    dz = fabs(Gxyz[ct_AN][3] - Gxyz[j][3] - cell_atv[Rn][3]);
-	    r2 = dx*dx + dy*dy + dz*dz;
-	  }
+	    if (((Rn==0)&&(ct_AN!=j)) || Rn!=0){
+	      dx = fabs(Gxyz[ct_AN][1] - Gxyz[j][1] - cell_atv[Rn][1]);
+	      dy = fabs(Gxyz[ct_AN][2] - Gxyz[j][2] - cell_atv[Rn][2]);
+	      dz = fabs(Gxyz[ct_AN][3] - Gxyz[j][3] - cell_atv[Rn][3]);
+	      r2 = dx*dx + dy*dy + dz*dz;
+	    }
 
-	  if (0.01<r2 && r2<=rcut2)    FNAN[ct_AN] += tno1*tno2;
-          if (NOHS_C==1){
-  	    if (0.01<r2 && r2<=rcut2)  SNAN[ct_AN] += tno2;
-	  }
-          else{
-  	    if (0.01<r2 && r2<=BCR2)   SNAN[ct_AN] += tno2;
-	  }
+	    if (0.01<r2 && r2<=rcut2)    FNAN[ct_AN] += tno1*tno2;
+	    if (NOHS_C==1){
+	      if (0.01<r2 && r2<=rcut2)  SNAN[ct_AN] += tno2;
+	    }
+	    else{
+	      if (0.01<r2 && r2<=BCR2)   SNAN[ct_AN] += tno2;
+	    }
 
-        }
+	  }
+	}
       }
-    }
 
+      if (2<=level_stdout){
+	printf("<Estimate_NL>  ct_AN=%2d FNAN SNAN %2d %2d\n",ct_AN,FNAN[ct_AN],SNAN[ct_AN]);
+      }
 
-    if (2<=level_stdout){
-      printf("<Estimate_NL>  ct_AN=%2d FNAN SNAN %2d %2d\n",ct_AN,FNAN[ct_AN],SNAN[ct_AN]);
-    }
+    } /* i */
 
-  }
+  } /* #pragma omp parallel */
 
   /* freeing of cell_atv */
   for (Rn=0; Rn<TN_Rn; Rn++){
