@@ -24,6 +24,13 @@
 #include "mpi.h"
 #endif
 
+#ifdef noomp
+#include "mimic_omp.h"
+#else
+#include <omp.h>
+#endif
+
+
 
 
 double Set_Density_Grid(int Cnt_kind, int Calc_CntOrbital_ON, double *****CDM)
@@ -55,7 +62,6 @@ double Set_Density_Grid(int Cnt_kind, int Calc_CntOrbital_ON, double *****CDM)
   double *orbs1_0,*orbs1_1,*orbs1_2,*orbs1_3;
 
   double ***tmp_CDM;
-  double Dcut=1.0e-14; 
   int *Snd_Size,*Rcv_Size;
   int numprocs,myid,tag=999,ID,IDS,IDR;
   double Stime_atom, Etime_atom;
@@ -63,9 +69,11 @@ double Set_Density_Grid(int Cnt_kind, int Calc_CntOrbital_ON, double *****CDM)
   MPI_Status stat;
   MPI_Request request;
 
+  /* for OpenMP */
+  int OMPID,Nthrds,Nprocs;
+
   /* snlxxx */
   double time1,time2,mflops;
-  double numop;
   /**/
   /* MPI */
   MPI_Comm_size(mpi_comm_level1,&numprocs);
@@ -77,51 +85,6 @@ double Set_Density_Grid(int Cnt_kind, int Calc_CntOrbital_ON, double *****CDM)
   
   dtime(&TStime);
 
-  /****************************************************
-    allocation of arrays:
-
-    double Tmp_Den_Grid
-                   [SpinP_switch+1]
-                   [Matomnum+MatomnumF+1]
-                   [GridN_Atom[Gc_AN]]
-    double orbs0[List_YOUSO[7]]
-    double orbs1[List_YOUSO[7]]
-
-    double tmp_CDM[SpinP_switch+1]
-                  [List_YOUSO[7]]
-                  [List_YOUSO[7]]  
-  ****************************************************/
-
-  Tmp_Den_Grid = (double***)malloc(sizeof(double**)*(SpinP_switch+1)); 
-  for (i=0; i<(SpinP_switch+1); i++){
-    Tmp_Den_Grid[i] = (double**)malloc(sizeof(double*)*(Matomnum+MatomnumF+1)); 
-    Tmp_Den_Grid[i][0] = (double*)malloc(sizeof(double)*1); 
-    for (Mc_AN=1; Mc_AN<=(Matomnum+MatomnumF); Mc_AN++){
-      Gc_AN = F_M2G[Mc_AN];
-      Tmp_Den_Grid[i][Mc_AN] = (double*)malloc(sizeof(double)*GridN_Atom[Gc_AN]);
-    }
-  }
-
-  orbs0 = (double*)malloc(sizeof(double)*List_YOUSO[7]);
-  orbs1 = (double*)malloc(sizeof(double)*List_YOUSO[7]);
-
-  orbs0_0 = (double*)malloc(sizeof(double)*List_YOUSO[7]);
-  orbs0_1 = (double*)malloc(sizeof(double)*List_YOUSO[7]);
-  orbs0_2 = (double*)malloc(sizeof(double)*List_YOUSO[7]);
-  orbs0_3 = (double*)malloc(sizeof(double)*List_YOUSO[7]);
-  orbs1_0 = (double*)malloc(sizeof(double)*List_YOUSO[7]);
-  orbs1_1 = (double*)malloc(sizeof(double)*List_YOUSO[7]);
-  orbs1_2 = (double*)malloc(sizeof(double)*List_YOUSO[7]);
-  orbs1_3 = (double*)malloc(sizeof(double)*List_YOUSO[7]);
-
-  tmp_CDM = (double***)malloc(sizeof(double**)*(SpinP_switch+1)); 
-  for (i=0; i<(SpinP_switch+1); i++){
-    tmp_CDM[i] = (double**)malloc(sizeof(double*)*List_YOUSO[7]); 
-    for (j=0; j<List_YOUSO[7]; j++){
-      tmp_CDM[i][j] = (double*)malloc(sizeof(double)*List_YOUSO[7]); 
-    }
-  }
-
   /****************************************************
              initialize electron densities
   ****************************************************/
@@ -307,189 +270,250 @@ double Set_Density_Grid(int Cnt_kind, int Calc_CntOrbital_ON, double *****CDM)
               calculate Tmp_Den_Grid
   ***********************************************/
     
-  /* snlxxx */
   dtime(&time1);
-  numop=0;
-  /**/
-  for (Mc_AN=1; Mc_AN<=Matomnum; Mc_AN++){
-    
-    dtime(&Stime_atom);
-    
-    Gc_AN = M2G[Mc_AN];
-    Cwan = WhatSpecies[Gc_AN];
-    NO0 = Spe_Total_CNO[Cwan]; 
-    
-    for (spin=0; spin<=SpinP_switch; spin++){
-      for (Nc=0; Nc<GridN_Atom[Gc_AN]; Nc++){
-        Tmp_Den_Grid[spin][Mc_AN][Nc] = 0.0;
+
+  /* allocation of Tmp_Den_Grid */
+
+  Tmp_Den_Grid = (double***)malloc(sizeof(double**)*(SpinP_switch+1)); 
+  for (i=0; i<(SpinP_switch+1); i++){
+    Tmp_Den_Grid[i] = (double**)malloc(sizeof(double*)*(Matomnum+MatomnumF+1)); 
+    Tmp_Den_Grid[i][0] = (double*)malloc(sizeof(double)*1); 
+    for (Mc_AN=1; Mc_AN<=(Matomnum+MatomnumF); Mc_AN++){
+      Gc_AN = F_M2G[Mc_AN];
+      Tmp_Den_Grid[i][Mc_AN] = (double*)malloc(sizeof(double)*GridN_Atom[Gc_AN]);
+    }
+  }
+
+#pragma omp parallel shared(List_YOUSO,time_per_atom,Tmp_Den_Grid,Orbs_Grid,COrbs_Grid,Cnt_switch,Cnt_kind,GListTAtoms2,GListTAtoms1,NumOLG,CDM,SpinP_switch,WhatSpecies,ncn,F_G2M,natn,Spe_Total_CNO,M2G) private(OMPID,Nthrds,Nprocs,Mc_AN,h_AN,Stime_atom,Etime_atom,Gc_AN,Cwan,NO0,Gh_AN,Mh_AN,Rnh,Hwan,NO1,spin,i,j,tmp_CDM,Nog,Nc_0,Nc_1,Nc_2,Nc_3,Nh_0,Nh_1,Nh_2,Nh_3,orbs0_0,orbs0_1,orbs0_2,orbs0_3,orbs1_0,orbs1_1,orbs1_2,orbs1_3,sum_0,sum_1,sum_2,sum_3,tmp0_0,tmp0_1,tmp0_2,tmp0_3,mm,Nc,Nh,orbs0,orbs1,sum,tmp0)
+  {
+
+    orbs0 = (double*)malloc(sizeof(double)*List_YOUSO[7]);
+    orbs1 = (double*)malloc(sizeof(double)*List_YOUSO[7]);
+
+    orbs0_0 = (double*)malloc(sizeof(double)*List_YOUSO[7]);
+    orbs0_1 = (double*)malloc(sizeof(double)*List_YOUSO[7]);
+    orbs0_2 = (double*)malloc(sizeof(double)*List_YOUSO[7]);
+    orbs0_3 = (double*)malloc(sizeof(double)*List_YOUSO[7]);
+    orbs1_0 = (double*)malloc(sizeof(double)*List_YOUSO[7]);
+    orbs1_1 = (double*)malloc(sizeof(double)*List_YOUSO[7]);
+    orbs1_2 = (double*)malloc(sizeof(double)*List_YOUSO[7]);
+    orbs1_3 = (double*)malloc(sizeof(double)*List_YOUSO[7]);
+
+    tmp_CDM = (double***)malloc(sizeof(double**)*(SpinP_switch+1)); 
+    for (i=0; i<(SpinP_switch+1); i++){
+      tmp_CDM[i] = (double**)malloc(sizeof(double*)*List_YOUSO[7]); 
+      for (j=0; j<List_YOUSO[7]; j++){
+	tmp_CDM[i][j] = (double*)malloc(sizeof(double)*List_YOUSO[7]); 
       }
     }
-      
-    for (h_AN=0; h_AN<=FNAN[Gc_AN]; h_AN++){
-      Gh_AN = natn[Gc_AN][h_AN];
-      Mh_AN = F_G2M[Gh_AN];
-      Rnh = ncn[Gc_AN][h_AN];
-      Hwan = WhatSpecies[Gh_AN];
-      NO1 = Spe_Total_CNO[Hwan];
 
-      /* store CDM into tmp_CDM */
+    /* get info. on OpenMP */ 
+
+    OMPID = omp_get_thread_num();
+    Nthrds = omp_get_num_threads();
+    Nprocs = omp_get_num_procs();
+
+    for (Mc_AN=(OMPID*Matomnum/Nthrds+1); Mc_AN<((OMPID+1)*Matomnum/Nthrds+1); Mc_AN++){
+
+      dtime(&Stime_atom);
+
+      /* set data on Mc_AN */
+
+      Gc_AN = M2G[Mc_AN];
+      Cwan = WhatSpecies[Gc_AN];
+      NO0 = Spe_Total_CNO[Cwan]; 
 
       for (spin=0; spin<=SpinP_switch; spin++){
-        for (i=0; i<NO0; i++){
-          for (j=0; j<NO1; j++){
-            tmp_CDM[spin][i][j] = CDM[spin][Mc_AN][h_AN][i][j];
-	  }
+	for (Nc=0; Nc<GridN_Atom[Gc_AN]; Nc++){
+	  Tmp_Den_Grid[spin][Mc_AN][Nc] = 0.0;
 	}
       }
 
-      /* summation of non-zero elements */
+      for (h_AN=0; h_AN<=FNAN[Gc_AN]; h_AN++){
 
-      for (Nog=0; Nog<NumOLG[Mc_AN][h_AN]-3; Nog+=4){
+	/* set data on h_AN */
+    
+	Gh_AN = natn[Gc_AN][h_AN];
+	Mh_AN = F_G2M[Gh_AN];
+	Rnh = ncn[Gc_AN][h_AN];
+	Hwan = WhatSpecies[Gh_AN];
+	NO1 = Spe_Total_CNO[Hwan];
 
-        Nc_0 = GListTAtoms1[Mc_AN][h_AN][Nog];
-        Nc_1 = GListTAtoms1[Mc_AN][h_AN][Nog+1];
-        Nc_2 = GListTAtoms1[Mc_AN][h_AN][Nog+2];
-        Nc_3 = GListTAtoms1[Mc_AN][h_AN][Nog+3];
+	/* store CDM into tmp_CDM */
 
-        Nh_0 = GListTAtoms2[Mc_AN][h_AN][Nog];
-        Nh_1 = GListTAtoms2[Mc_AN][h_AN][Nog+1];
-        Nh_2 = GListTAtoms2[Mc_AN][h_AN][Nog+2];
-        Nh_3 = GListTAtoms2[Mc_AN][h_AN][Nog+3];
-
-        /* Now under the orbital optimization */
-        if (Cnt_kind==0 && Cnt_switch==1){
+	for (spin=0; spin<=SpinP_switch; spin++){
 	  for (i=0; i<NO0; i++){
-            orbs0_0[i] = COrbs_Grid[Mc_AN][i][Nc_0];
-            orbs0_1[i] = COrbs_Grid[Mc_AN][i][Nc_1];
-            orbs0_2[i] = COrbs_Grid[Mc_AN][i][Nc_2];
-            orbs0_3[i] = COrbs_Grid[Mc_AN][i][Nc_3];
-          }
-          for (j=0; j<NO1; j++){
-            orbs1_0[j] = COrbs_Grid[Mh_AN][j][Nh_0];
-            orbs1_1[j] = COrbs_Grid[Mh_AN][j][Nh_1];
-            orbs1_2[j] = COrbs_Grid[Mh_AN][j][Nh_2];
-            orbs1_3[j] = COrbs_Grid[Mh_AN][j][Nh_3];
-          }
-	}
-        /* else if ! "now under the orbital optimization" */
-        else{
-	  for (i=0; i<NO0; i++){
-            orbs0_0[i] = Orbs_Grid[Mc_AN][i][Nc_0];
-            orbs0_1[i] = Orbs_Grid[Mc_AN][i][Nc_1];
-            orbs0_2[i] = Orbs_Grid[Mc_AN][i][Nc_2];
-            orbs0_3[i] = Orbs_Grid[Mc_AN][i][Nc_3];
-          }
-          for (j=0; j<NO1; j++){
-            orbs1_0[j] = Orbs_Grid[Mh_AN][j][Nh_0];
-            orbs1_1[j] = Orbs_Grid[Mh_AN][j][Nh_1];
-            orbs1_2[j] = Orbs_Grid[Mh_AN][j][Nh_2];
-            orbs1_3[j] = Orbs_Grid[Mh_AN][j][Nh_3];
-          }
-        }
-
-        for (spin=0; spin<=SpinP_switch; spin++){
-
-          /* Tmp_Den_Grid */
-
-          sum_0 = 0.0;
-          sum_1 = 0.0;
-          sum_2 = 0.0;
-          sum_3 = 0.0;
-          for (i=0; i<NO0; i++){
-            tmp0_0 = 0.0;
-            tmp0_1 = 0.0;
-            tmp0_2 = 0.0;
-            tmp0_3 = 0.0;
 	    for (j=0; j<NO1; j++){
-              tmp0_0 += orbs1_0[j]*tmp_CDM[spin][i][j];
-              tmp0_1 += orbs1_1[j]*tmp_CDM[spin][i][j];
-              tmp0_2 += orbs1_2[j]*tmp_CDM[spin][i][j];
-              tmp0_3 += orbs1_3[j]*tmp_CDM[spin][i][j];
+	      tmp_CDM[spin][i][j] = CDM[spin][Mc_AN][h_AN][i][j];
 	    }
-	    /* snlxxx */
-            numop += 4*2*NO1;
-	    /**/
-            sum_0 += orbs0_0[i]*tmp0_0;
-            sum_1 += orbs0_1[i]*tmp0_1;
-            sum_2 += orbs0_2[i]*tmp0_2;
-            sum_3 += orbs0_3[i]*tmp0_3;
 	  }
-	  /* snlxxx */
-	  numop += 4*2*NO0;
-	  /**/
-
-          Tmp_Den_Grid[spin][Mc_AN][Nc_0] += sum_0;
-          Tmp_Den_Grid[spin][Mc_AN][Nc_1] += sum_1;
-          Tmp_Den_Grid[spin][Mc_AN][Nc_2] += sum_2;
-          Tmp_Den_Grid[spin][Mc_AN][Nc_3] += sum_3;
 	}
-	/* snlxxx */
-        numop += 4*(SpinP_switch+1);
-	/**/
-      }
-      mm=NumOLG[Mc_AN][h_AN]-(NumOLG[Mc_AN][h_AN]/4)*4;
 
-      if(mm != 0){
-        for(Nog=NumOLG[Mc_AN][h_AN]-mm; Nog<NumOLG[Mc_AN][h_AN]; Nog++){
- 
-          Nc = GListTAtoms1[Mc_AN][h_AN][Nog];
-          Nh = GListTAtoms2[Mc_AN][h_AN][Nog]; 
- 
+	/* summation of non-zero elements */
+
+	for (Nog=0; Nog<NumOLG[Mc_AN][h_AN]-3; Nog+=4){
+
+	  Nc_0 = GListTAtoms1[Mc_AN][h_AN][Nog];
+	  Nc_1 = GListTAtoms1[Mc_AN][h_AN][Nog+1];
+	  Nc_2 = GListTAtoms1[Mc_AN][h_AN][Nog+2];
+	  Nc_3 = GListTAtoms1[Mc_AN][h_AN][Nog+3];
+
+	  Nh_0 = GListTAtoms2[Mc_AN][h_AN][Nog];
+	  Nh_1 = GListTAtoms2[Mc_AN][h_AN][Nog+1];
+	  Nh_2 = GListTAtoms2[Mc_AN][h_AN][Nog+2];
+	  Nh_3 = GListTAtoms2[Mc_AN][h_AN][Nog+3];
+
 	  /* Now under the orbital optimization */
-          if (Cnt_kind==0 && Cnt_switch==1){
-            for (i=0; i<NO0; i++){
-              orbs0[i] = COrbs_Grid[Mc_AN][i][Nc];
-            }
-            for (j=0; j<NO1; j++){
-              orbs1[j] = COrbs_Grid[Mh_AN][j][Nh];
-            }
-          }
+	  if (Cnt_kind==0 && Cnt_switch==1){
+	    for (i=0; i<NO0; i++){
+	      orbs0_0[i] = COrbs_Grid[Mc_AN][i][Nc_0];
+	      orbs0_1[i] = COrbs_Grid[Mc_AN][i][Nc_1];
+	      orbs0_2[i] = COrbs_Grid[Mc_AN][i][Nc_2];
+	      orbs0_3[i] = COrbs_Grid[Mc_AN][i][Nc_3];
+	    }
+	    for (j=0; j<NO1; j++){
+	      orbs1_0[j] = COrbs_Grid[Mh_AN][j][Nh_0];
+	      orbs1_1[j] = COrbs_Grid[Mh_AN][j][Nh_1];
+	      orbs1_2[j] = COrbs_Grid[Mh_AN][j][Nh_2];
+	      orbs1_3[j] = COrbs_Grid[Mh_AN][j][Nh_3];
+	    }
+	  }
 	  /* else if ! "now under the orbital optimization" */
-          else{
-            for (i=0; i<NO0; i++){
-              orbs0[i] = Orbs_Grid[Mc_AN][i][Nc];
-            }
-            for (j=0; j<NO1; j++){
-              orbs1[j] = Orbs_Grid[Mh_AN][j][Nh];
-            }
-          }
+	  else{
+	    for (i=0; i<NO0; i++){
+	      orbs0_0[i] = Orbs_Grid[Mc_AN][i][Nc_0];
+	      orbs0_1[i] = Orbs_Grid[Mc_AN][i][Nc_1];
+	      orbs0_2[i] = Orbs_Grid[Mc_AN][i][Nc_2];
+	      orbs0_3[i] = Orbs_Grid[Mc_AN][i][Nc_3];
+	    }
+	    for (j=0; j<NO1; j++){
+	      orbs1_0[j] = Orbs_Grid[Mh_AN][j][Nh_0];
+	      orbs1_1[j] = Orbs_Grid[Mh_AN][j][Nh_1];
+	      orbs1_2[j] = Orbs_Grid[Mh_AN][j][Nh_2];
+	      orbs1_3[j] = Orbs_Grid[Mh_AN][j][Nh_3];
+	    }
+	  }
+
+	  for (spin=0; spin<=SpinP_switch; spin++){
 
-          for (spin=0; spin<=SpinP_switch; spin++){
- 
 	    /* Tmp_Den_Grid */
+
+	    sum_0 = 0.0;
+	    sum_1 = 0.0;
+	    sum_2 = 0.0;
+	    sum_3 = 0.0;
+
+	    for (i=0; i<NO0; i++){
+
+	      tmp0_0 = 0.0;
+	      tmp0_1 = 0.0;
+	      tmp0_2 = 0.0;
+	      tmp0_3 = 0.0;
+
+	      for (j=0; j<NO1; j++){
+		tmp0_0 += orbs1_0[j]*tmp_CDM[spin][i][j];
+		tmp0_1 += orbs1_1[j]*tmp_CDM[spin][i][j];
+		tmp0_2 += orbs1_2[j]*tmp_CDM[spin][i][j];
+		tmp0_3 += orbs1_3[j]*tmp_CDM[spin][i][j];
+	      }
+
+	      sum_0 += orbs0_0[i]*tmp0_0;
+	      sum_1 += orbs0_1[i]*tmp0_1;
+	      sum_2 += orbs0_2[i]*tmp0_2;
+	      sum_3 += orbs0_3[i]*tmp0_3;
+	    }
+
+	    Tmp_Den_Grid[spin][Mc_AN][Nc_0] += sum_0;
+	    Tmp_Den_Grid[spin][Mc_AN][Nc_1] += sum_1;
+	    Tmp_Den_Grid[spin][Mc_AN][Nc_2] += sum_2;
+	    Tmp_Den_Grid[spin][Mc_AN][Nc_3] += sum_3;
+
+	  }
+	}
+
+	mm = NumOLG[Mc_AN][h_AN]-(NumOLG[Mc_AN][h_AN]/4)*4;
+
+	if (mm!=0){
+
+	  for (Nog=NumOLG[Mc_AN][h_AN]-mm; Nog<NumOLG[Mc_AN][h_AN]; Nog++){
  
-            sum = 0.0;
-            for (i=0; i<NO0; i++){
-              tmp0 = 0.0;
-              for (j=0; j<NO1; j++){
-                tmp0 += orbs1[j]*tmp_CDM[spin][i][j];
-              }
-	      /* snlxxx */
-              numop += 2*NO1;
-	      /**/
-              sum += orbs0[i]*tmp0;
-            }
-	    /* snlxxx */
-	    numop += 2*NO0;
-	    /**/
+	    Nc = GListTAtoms1[Mc_AN][h_AN][Nog];
+	    Nh = GListTAtoms2[Mc_AN][h_AN][Nog]; 
  
-            Tmp_Den_Grid[spin][Mc_AN][Nc] += sum;
-          }
-	  /* snlxxx */
-          numop += (SpinP_switch+1);
-	  /**/
-        }
+	    /* Now under the orbital optimization */
+	    if (Cnt_kind==0 && Cnt_switch==1){
+	      for (i=0; i<NO0; i++){
+		orbs0[i] = COrbs_Grid[Mc_AN][i][Nc];
+	      }
+	      for (j=0; j<NO1; j++){
+		orbs1[j] = COrbs_Grid[Mh_AN][j][Nh];
+	      }
+	    }
+	    /* else if ! "now under the orbital optimization" */
+	    else{
+	      for (i=0; i<NO0; i++){
+		orbs0[i] = Orbs_Grid[Mc_AN][i][Nc];
+	      }
+	      for (j=0; j<NO1; j++){
+		orbs1[j] = Orbs_Grid[Mh_AN][j][Nh];
+	      }
+	    }
+
+	    for (spin=0; spin<=SpinP_switch; spin++){
+ 
+	      /* Tmp_Den_Grid */
+ 
+	      sum = 0.0;
+	      for (i=0; i<NO0; i++){
+		tmp0 = 0.0;
+		for (j=0; j<NO1; j++){
+		  tmp0 += orbs1[j]*tmp_CDM[spin][i][j];
+		}
+		sum += orbs0[i]*tmp0;
+	      }
+ 
+	      Tmp_Den_Grid[spin][Mc_AN][Nc] += sum;
+	    }
+	  }
+	}
+
+      } /* h_AN */
+
+      dtime(&Etime_atom);
+      time_per_atom[Gc_AN] += Etime_atom - Stime_atom;
+
+    } /* Mc_AN */
+
+    /* freeing of arrays */ 
+
+    free(orbs0);
+    free(orbs1);
+
+    free(orbs0_0);
+    free(orbs0_1);
+    free(orbs0_2);
+    free(orbs0_3);
+    free(orbs1_0);
+    free(orbs1_1);
+    free(orbs1_2);
+    free(orbs1_3);
+
+    for (i=0; i<(SpinP_switch+1); i++){
+      for (j=0; j<List_YOUSO[7]; j++){
+	free(tmp_CDM[i][j]);
       }
+      free(tmp_CDM[i]);
     }
-    dtime(&Etime_atom);
-    time_per_atom[Gc_AN] += Etime_atom - Stime_atom;
-  }
-  /* snlxxx */
+    free(tmp_CDM);
+
+#pragma omp flush(Tmp_Den_Grid)
+
+  } /* #pragma omp parallel */
+
   dtime(&time2);
-  mflops=numop/(time2-time1)*1.0e-6;
-  if(myid==0 && measure_time)
-    printf("Time for Part1=%18.5f mflops=%12.2f\n",(time2-time1),mflops);fflush(stdout);
-  /**/
+  if(myid==0 && measure_time){
+    printf("Time for Part1=%18.5f\n",(time2-time1));fflush(stdout);
+  }
 
   /******************************************************
    MPI:
@@ -618,12 +642,13 @@ double Set_Density_Grid(int Cnt_kind, int Calc_CntOrbital_ON, double *****CDM)
   free(Rcv_Size);
 
   /**********************************************
-                calc Density_Grid
+                 calc Density_Grid
   ***********************************************/
 
   /* snlxxx */
   dtime(&time1);
   /**/
+
   for (Mc_AN=1; Mc_AN<=(Matomnum+MatomnumF); Mc_AN++){
 
     dtime(&Stime_atom);
@@ -633,16 +658,20 @@ double Set_Density_Grid(int Cnt_kind, int Calc_CntOrbital_ON, double *****CDM)
 
     for (Nc=0; Nc<GridN_Atom[Gc_AN]; Nc++){
       MN = MGridListAtom[Mc_AN][Nc];
-      if (0<=MN){ 
-        for (spin=0; spin<=SpinP_switch; spin++){
-          Density_Grid[spin][MN] += Tmp_Den_Grid[spin][Mc_AN][Nc];
-        }
+      GRc = CellListAtom[Mc_AN][Nc];
+
+      if ( (0<=MN && Solver!=4) || (0<=MN && Solver==4 && atv_ijk[GRc][1]==0) ){ 
+
+	for (spin=0; spin<=SpinP_switch; spin++){
+	  Density_Grid[spin][MN] += Tmp_Den_Grid[spin][Mc_AN][Nc];
+	}
       }
     }
 
     dtime(&Etime_atom);
     time_per_atom[Gc_AN] += Etime_atom - Stime_atom;
   }
+
   /* snlxxx */
   dtime(&time2);
   if(myid==0 && measure_time)
@@ -653,6 +682,7 @@ double Set_Density_Grid(int Cnt_kind, int Calc_CntOrbital_ON, double *****CDM)
          add Tmp_Den2_Grid in terms of FNAN2 
   ******************************************************/
 
+  /* allocation of array */
   Tmp_Den2_Grid = (double**)malloc(sizeof(double*)*(SpinP_switch+1));
   for (i=0; i<(SpinP_switch+1); i++){
     Tmp_Den2_Grid[i] = (double*)malloc(sizeof(double)*FNAN2_Grid);
@@ -708,15 +738,27 @@ double Set_Density_Grid(int Cnt_kind, int Calc_CntOrbital_ON, double *****CDM)
         }
       }
     }
-  }
+  } /* spin */
 
   for (spin=0; spin<=SpinP_switch; spin++){
 
     /* Density_Grid += Tmp_Den2_Grid */ 
 
-    for (i=0; i<FNAN2_Grid; i++){
-      MN = Rcv_FNAN2_MN[i];
-      Density_Grid[spin][MN] += Tmp_Den2_Grid[spin][i];
+    if (Solver!=4){
+      for (i=0; i<FNAN2_Grid; i++){
+	MN = Rcv_FNAN2_MN[i];
+	Density_Grid[spin][MN] += Tmp_Den2_Grid[spin][i];
+      }
+    }
+    else if (Solver==4){
+      for (i=0; i<FNAN2_Grid; i++){
+	MN = Rcv_FNAN2_MN[i];
+        GRc = Rcv_FNAN2_GRc[i];
+
+        if (atv_ijk[GRc][1]==0){
+  	  Density_Grid[spin][MN] += Tmp_Den2_Grid[spin][i];
+	}
+      }
     }
   }
 
@@ -740,15 +782,36 @@ double Set_Density_Grid(int Cnt_kind, int Calc_CntOrbital_ON, double *****CDM)
     }
   }
 
-  /****************************************************
-    freeing of arrays:
 
-    double Tmp_Den_Grid[SpinP_switch+1]
-                       [Matomnum+MatomnumF+1]
-                       [GridN_Atom[Gc_AN]]
-    double orbs0[List_YOUSO[7]]
-    double orbs1[List_YOUSO[7]]
-  ****************************************************/
+  /*
+  {
+
+    int nn1,nn0,MN1,MN2;
+
+  n2 = Ngrid2/2;
+  n3 = Ngrid3/2;
+
+  for (n1=Start_Grid1[myid]; n1<=End_Grid1[myid]; n1++){
+    nn1 = My_Cell0[n1];
+    nn0 = n1 - Start_Grid1[myid]; 
+    MN1 = nn1*Ngrid2*Ngrid3;
+    MN2 = n2*Ngrid3;
+    MN = MN1 + MN2 + n3;
+    printf("%2d %18.15f %18.15f\n",n1,Density_Grid[0][MN],Density_Grid[1][MN]);
+  }
+
+  }
+  */
+
+
+
+
+  /* freeing of arrays */
+
+  for (i=0; i<(SpinP_switch+1); i++){
+    free(Tmp_Den2_Grid[i]);
+  }
+  free(Tmp_Den2_Grid);
 
   for (i=0; i<(SpinP_switch+1); i++){
     for (Mc_AN=0; Mc_AN<=(Matomnum+MatomnumF); Mc_AN++){
@@ -758,31 +821,6 @@ double Set_Density_Grid(int Cnt_kind, int Calc_CntOrbital_ON, double *****CDM)
   }
   free(Tmp_Den_Grid);
 
-  free(orbs0);
-  free(orbs1);
-
-  free(orbs0_0);
-  free(orbs0_1);
-  free(orbs0_2);
-  free(orbs0_3);
-  free(orbs1_0);
-  free(orbs1_1);
-  free(orbs1_2);
-  free(orbs1_3);
-
-  for (i=0; i<(SpinP_switch+1); i++){
-    for (j=0; j<List_YOUSO[7]; j++){
-      free(tmp_CDM[i][j]);
-    }
-    free(tmp_CDM[i]);
-  }
-  free(tmp_CDM);
-
-  for (i=0; i<(SpinP_switch+1); i++){
-    free(Tmp_Den2_Grid[i]);
-  }
-  free(Tmp_Den2_Grid);
-
   /* elapsed time */
   dtime(&TEtime);
   time0 = TEtime - TStime;
@@ -790,6 +828,7 @@ double Set_Density_Grid(int Cnt_kind, int Calc_CntOrbital_ON, double *****CDM)
   if(myid==0 && measure_time)
     printf(">>>>>>>Set_Density_Grid RETURN time=%18.5f\n",time0);
   /**/
+
   return time0;
 }
 
@@ -802,31 +841,37 @@ void diagonalize_nc_density()
   double Re11,Re22,Re12,Im12;
   double phi[2],theta[2],sit,cot,sip,cop;
   double d1,d2,d3,x,y,z,Cxyz[4];
-  double Dcut=1.0e-14; 
   double Nup[2],Ndown[2];
 
-  for (MN=0; MN<My_NumGrid1; MN++){
+  /* for OpenMP */
+  int OMPID,Nthrds,Nprocs;
 
-    Re11 = Density_Grid[0][MN];
-    Re22 = Density_Grid[1][MN];
-    Re12 = Density_Grid[2][MN];
-    Im12 = Density_Grid[3][MN];
+#pragma omp parallel shared(Density_Grid,My_NumGrid1) private(OMPID,Nthrds,Nprocs,MN,Re11,Re22,Re12,Im12,Nup,Ndown,theta,phi)
+  {
 
-    EulerAngle_Spin( 1, Re11, Re22, Re12, Im12, Re12, -Im12, Nup, Ndown, theta, phi );
+    /* get info. on OpenMP */ 
 
-    Density_Grid[0][MN] = Nup[0];
-    Density_Grid[1][MN] = Ndown[0];
-    Density_Grid[2][MN] = theta[0];
-    Density_Grid[3][MN] = phi[0];
+    OMPID = omp_get_thread_num();
+    Nthrds = omp_get_num_threads();
+    Nprocs = omp_get_num_procs();
 
-    /*
-    printf("MN=%4d %15.12f %15.12f %15.12f %15.12f\n",
-           MN,Density_Grid[0][MN],Density_Grid[1][MN],
-	   Density_Grid[2][MN]/PI*180.0,
-	   Density_Grid[3][MN]/PI*180.0
-           );
-    */
+    for (MN=OMPID*My_NumGrid1/Nthrds; MN<(OMPID+1)*My_NumGrid1/Nthrds; MN++){
 
-  }
+      Re11 = Density_Grid[0][MN];
+      Re22 = Density_Grid[1][MN];
+      Re12 = Density_Grid[2][MN];
+      Im12 = Density_Grid[3][MN];
+
+      EulerAngle_Spin( 1, Re11, Re22, Re12, Im12, Re12, -Im12, Nup, Ndown, theta, phi );
+
+      Density_Grid[0][MN] = Nup[0];
+      Density_Grid[1][MN] = Ndown[0];
+      Density_Grid[2][MN] = theta[0];
+      Density_Grid[3][MN] = phi[0];
+    }
+
+#pragma omp flush(Density_Grid)
+
+  } /* #pragma omp parallel */
 
 }
