@@ -25,6 +25,13 @@
 #include "mpi.h"
 #endif
 
+#ifdef noomp
+#include "mimic_omp.h"
+#else
+#include <omp.h>
+#endif
+
+
 void simple_mixing_H(double *****Hcnt);
 
 
@@ -44,11 +51,11 @@ double Set_Hamiltonian(char *mode,
   ***************************************************************/
 
   double time0;
-  int Mc_AN,Gc_AN,Mh_AN,h_AN,Gh_AN;
+  int Mc_AN,Gc_AN,Mh_AN,h_AN,h_ANs,h_ANe,Gh_AN;
   int i,j,Cwan,Hwan,NO0,NO1,spinmax;
   int Rnh,Rnk,spin,N,NumC[4];
   int n1,n2,n3,L0,Mul0,M0,L1,Mul1,M1;
-  int Nc,GNc,GRc,Nog,Nh,MN,XC_P_switch;
+  int Nc,Ncs,GNc,GRc,Nog,Nh,MN,XC_P_switch;
   int Nc_0,Nc_1,Nc_2,Nc_3,Nh_0,Nh_1,Nh_2,Nh_3;
   int mm;
 
@@ -57,13 +64,10 @@ double Set_Hamiltonian(char *mode,
   double tmp0_0,tmp0_1,tmp0_2,tmp0_3;
 
   double xo,yo,zo,S_coordinate[3];
-  double *Chi0,*Chi1;
   double ***ChiV0;
-  double **tmp_ChiV0;
   double **tmp_ChiV0_0,**tmp_ChiV0_1,**tmp_ChiV0_2,**tmp_ChiV0_3;
-
-  double *tmp_Orbs_Grid;
   double *tmp_Orbs_Grid_0,*tmp_Orbs_Grid_1,*tmp_Orbs_Grid_2,*tmp_Orbs_Grid_3;
+  double *Chi0;
 
   double ***tmp_H;
   double TStime,TEtime;
@@ -75,8 +79,22 @@ double Set_Hamiltonian(char *mode,
   double Stime_atom, Etime_atom;
   /* snlxxx */
   double time1,time2,mflops;
-  double numop;
-  /**/
+
+  int GNh_0,GNh_1,GNh_2,GNh_3;
+  int GRh_0,GRh_1,GRh_2,GRh_3;
+  double Cxyz_0[4],Cxyz_1[4],Cxyz_2[4],Cxyz_3[4];
+  double x_0,y_0,z_0;
+  double x_1,y_1,z_1;
+  double x_2,y_2,z_2;
+  double x_3,y_3,z_3;
+  double dx_0,dy_0,dz_0;
+  double dx_1,dy_1,dz_1;
+  double dx_2,dy_2,dz_2;
+  double dx_3,dy_3,dz_3;
+
+  /* for OpenMP */
+  int OMPID,Nthrds,Nprocs;
+  int Nloop,OneD_Nloop,*OneD2Mc_AN,*OneD2h_AN;
 
   MPI_Comm_size(mpi_comm_level1,&numprocs);
   MPI_Comm_rank(mpi_comm_level1,&myid);
@@ -93,66 +111,6 @@ double Set_Hamiltonian(char *mode,
     printf("<Set_Hamiltonian>  Hamiltonian matrix for VNA+dVH+Vxc...\n");fflush(stdout);
   }
 
-  /****************************************************
-    allocation of arrays:
-
-    double Chi0[List_YOUSO[7]];
-    double Chi1[List_YOUSO[7]];
-    double ChiV0[4][List_YOUSO[7]][List_YOUSO[11]];
-  ****************************************************/
-
-  Chi0 = (double*)malloc(sizeof(double)*List_YOUSO[7]);
-  Chi1 = (double*)malloc(sizeof(double)*List_YOUSO[7]);
-
-  ChiV0 = (double***)malloc(sizeof(double**)*4);
-  for (i=0; i<4; i++){
-    ChiV0[i] = (double**)malloc(sizeof(double*)*List_YOUSO[7]);
-    for (j=0; j<List_YOUSO[7]; j++){
-      ChiV0[i][j] = (double*)malloc(sizeof(double)*List_YOUSO[11]);
-    }
-  }
-
-  tmp_ChiV0 = (double**)malloc(sizeof(double*)*4);
-  for (i=0; i<4; i++){
-    tmp_ChiV0[i] = (double*)malloc(sizeof(double)*List_YOUSO[7]);
-  }
-
-  tmp_ChiV0_0 = (double**)malloc(sizeof(double*)*4);
-  for (i=0; i<4; i++){
-    tmp_ChiV0_0[i] = (double*)malloc(sizeof(double)*List_YOUSO[7]);
-  }
-
-  tmp_ChiV0_1 = (double**)malloc(sizeof(double*)*4);
-  for (i=0; i<4; i++){
-    tmp_ChiV0_1[i] = (double*)malloc(sizeof(double)*List_YOUSO[7]);
-  }
-
-  tmp_ChiV0_2 = (double**)malloc(sizeof(double*)*4);
-  for (i=0; i<4; i++){
-    tmp_ChiV0_2[i] = (double*)malloc(sizeof(double)*List_YOUSO[7]);
-  }
-
-  tmp_ChiV0_3 = (double**)malloc(sizeof(double*)*4);
-  for (i=0; i<4; i++){
-    tmp_ChiV0_3[i] = (double*)malloc(sizeof(double)*List_YOUSO[7]);
-  }
-
-
-  tmp_Orbs_Grid = (double*)malloc(sizeof(double)*List_YOUSO[7]);
-
-  tmp_Orbs_Grid_0 = (double*)malloc(sizeof(double)*List_YOUSO[7]);
-  tmp_Orbs_Grid_1 = (double*)malloc(sizeof(double)*List_YOUSO[7]);
-  tmp_Orbs_Grid_2 = (double*)malloc(sizeof(double)*List_YOUSO[7]);
-  tmp_Orbs_Grid_3 = (double*)malloc(sizeof(double)*List_YOUSO[7]);
-
-  tmp_H = (double***)malloc(sizeof(double**)*4);
-  for (i=0; i<4; i++){
-    tmp_H[i] = (double**)malloc(sizeof(double*)*List_YOUSO[7]);
-    for (j=0; j<List_YOUSO[7]; j++){
-      tmp_H[i][j] = (double*)malloc(sizeof(double)*List_YOUSO[7]);
-    }
-  }
-
   /*****************************************************
                   adding H0+HNL to H 
   *****************************************************/
@@ -258,264 +216,337 @@ double Set_Hamiltonian(char *mode,
   /* snlxxx */
   dtime(&time1);
   /**/
+
   Set_Vpot(SCF_iter,XC_P_switch,CDM);
+
   /* snlxxx */
   dtime(&time2);
   if(myid==0 && measure_time) 
     printf("Set_Vpot  time=%18.5f\n",time2-time1);fflush(stdout);
   /**/
 
-  /* snlxxx */
   dtime(&time1);
-  numop=0;
-  /**/
+
+  /* one-dimensionalize the Mc_AN and h_AN loops */
+
+  OneD_Nloop = 0;
   for (Mc_AN=1; Mc_AN<=Matomnum; Mc_AN++){
-
-    dtime(&Stime_atom);
-
     Gc_AN = M2G[Mc_AN];    
-    Cwan = WhatSpecies[Gc_AN];
+    for (h_AN=0; h_AN<=FNAN[Gc_AN]; h_AN++){
+      OneD_Nloop++;
+    }
+  }  
 
-    if (Cnt_kind==0) NO0 = Spe_Total_NO[Cwan];
-    else             NO0 = Spe_Total_CNO[Cwan];
-    
-    for (spin=0; spin<=SpinP_switch; spin++){
-      for (i=0; i<NO0; i++){
-        for (Nc=0; Nc<GridN_Atom[Gc_AN]; Nc++){
-          MN = MGridListAtom[Mc_AN][Nc];
-          ChiV0[spin][i][Nc] = Orbs_Grid[Mc_AN][i][Nc]*Vpot_Grid[spin][MN];
-        }
-	/* snlxxx */
-        numop += GridN_Atom[Gc_AN];
-	/**/
+  OneD2Mc_AN = (int*)malloc(sizeof(int)*(OneD_Nloop+1));
+  OneD2h_AN = (int*)malloc(sizeof(int)*(OneD_Nloop+1));
+
+  OneD_Nloop = 0;
+  for (Mc_AN=1; Mc_AN<=Matomnum; Mc_AN++){
+    Gc_AN = M2G[Mc_AN];    
+    for (h_AN=0; h_AN<=FNAN[Gc_AN]; h_AN++){
+      OneD2Mc_AN[OneD_Nloop] = Mc_AN; 
+      OneD2h_AN[OneD_Nloop] = h_AN; 
+      OneD_Nloop++;
+    }
+  }
+
+  /* OpenMP */
+
+#pragma omp parallel shared(gtv,Gxyz,atv,GridListAtom,CellListAtom,OneD2h_AN,OneD2Mc_AN,OneD_Nloop,GridN_Atom,H,CntH,List_YOUSO,GridVol,Orbs_Grid,Vpot_Grid,MGridListAtom,GListTAtoms2,GListTAtoms1,Matomnum,NumOLG,SpinP_switch,Spe_Total_CNO,Spe_Total_NO,Cnt_kind,WhatSpecies,ncn,natn,F_G2M,FNAN)  private(OMPID,Nthrds,Nprocs,h_AN,Gh_AN,Mh_AN,Rnh,Hwan,NO1,spin,i,j,tmp_H,Nog,Nc_0,Nc_1,Nc_2,Nc_3,Nh_0,Nh_1,Nh_2,Nh_3,tmp_ChiV0_0,tmp_ChiV0_1,tmp_ChiV0_2,tmp_ChiV0_3,tmp_Orbs_Grid_0,tmp_Orbs_Grid_1,tmp_Orbs_Grid_2,tmp_Orbs_Grid_3,mm,Nc,Nh,tmp0,tmp0_0,tmp0_1,tmp0_2,tmp0_3,Mc_AN,Gc_AN,Cwan,NO0,Ncs,ChiV0,Chi0,Nloop)
+  {
+    /* allocation of arrays */ 
+
+    Chi0 = (double*)malloc(sizeof(double)*List_YOUSO[7]);
+
+    ChiV0 = (double***)malloc(sizeof(double**)*4);
+    for (i=0; i<4; i++){
+      ChiV0[i] = (double**)malloc(sizeof(double*)*List_YOUSO[7]);
+      for (j=0; j<List_YOUSO[7]; j++){
+	ChiV0[i][j] = (double*)malloc(sizeof(double)*List_YOUSO[11]);
       }
     }
+
+    tmp_H = (double***)malloc(sizeof(double**)*4);
+    for (i=0; i<4; i++){
+      tmp_H[i] = (double**)malloc(sizeof(double*)*List_YOUSO[7]);
+      for (j=0; j<List_YOUSO[7]; j++){
+	tmp_H[i][j] = (double*)malloc(sizeof(double)*List_YOUSO[7]);
+      }
+    }
+
+    tmp_ChiV0_0 = (double**)malloc(sizeof(double*)*4);
+    for (i=0; i<4; i++){
+      tmp_ChiV0_0[i] = (double*)malloc(sizeof(double)*List_YOUSO[7]);
+    }
+
+    tmp_ChiV0_1 = (double**)malloc(sizeof(double*)*4);
+    for (i=0; i<4; i++){
+      tmp_ChiV0_1[i] = (double*)malloc(sizeof(double)*List_YOUSO[7]);
+    }
     
-    for (h_AN=0; h_AN<=FNAN[Gc_AN]; h_AN++){
+    tmp_ChiV0_2 = (double**)malloc(sizeof(double*)*4);
+    for (i=0; i<4; i++){
+      tmp_ChiV0_2[i] = (double*)malloc(sizeof(double)*List_YOUSO[7]);
+    }
+    
+    tmp_ChiV0_3 = (double**)malloc(sizeof(double*)*4);
+    for (i=0; i<4; i++){
+      tmp_ChiV0_3[i] = (double*)malloc(sizeof(double)*List_YOUSO[7]);
+    }
+    
+    tmp_Orbs_Grid_0 = (double*)malloc(sizeof(double)*List_YOUSO[7]);
+    tmp_Orbs_Grid_1 = (double*)malloc(sizeof(double)*List_YOUSO[7]);
+    tmp_Orbs_Grid_2 = (double*)malloc(sizeof(double)*List_YOUSO[7]);
+    tmp_Orbs_Grid_3 = (double*)malloc(sizeof(double)*List_YOUSO[7]);
+    
+    /* get info. on OpenMP */ 
+     
+    OMPID = omp_get_thread_num();
+    Nthrds = omp_get_num_threads();
+    Nprocs = omp_get_num_procs();
+
+    /* one-dimensionalized loop */
+
+    for (Nloop=OMPID*OneD_Nloop/Nthrds; Nloop<(OMPID+1)*OneD_Nloop/Nthrds; Nloop++){
+
+      /* get Mc_AN and h_AN */
+
+      Mc_AN = OneD2Mc_AN[Nloop];
+      h_AN  = OneD2h_AN[Nloop];
+
+      /* set data on Mc_AN */
+
+      Gc_AN = M2G[Mc_AN];    
+      Cwan = WhatSpecies[Gc_AN];
+
+      if (Cnt_kind==0) NO0 = Spe_Total_NO[Cwan];
+      else             NO0 = Spe_Total_CNO[Cwan];
+    
+      for (spin=0; spin<=SpinP_switch; spin++){
+	for (i=0; i<NO0; i++){
+
+	  for (Nc=0; Nc<GridN_Atom[Gc_AN]; Nc++){
+	    ChiV0[spin][i][Nc] = Orbs_Grid[Mc_AN][i][Nc]*Vpot_Grid[spin][MGridListAtom[Mc_AN][Nc]];
+	  }
+
+	  Ncs = GridN_Atom[Gc_AN] - GridN_Atom[Gc_AN]%4; 
+	  for (Nc=Ncs; Nc<GridN_Atom[Gc_AN]; Nc++){
+	    ChiV0[spin][i][Nc] = Orbs_Grid[Mc_AN][i][Nc]*Vpot_Grid[spin][MGridListAtom[Mc_AN][Nc]];
+	  }
+	}
+      }
+
+      /* set data on h_AN */
 
       Gh_AN = natn[Gc_AN][h_AN];
       Mh_AN = F_G2M[Gh_AN];
 
       Rnh = ncn[Gc_AN][h_AN];
       Hwan = WhatSpecies[Gh_AN];
+
       if (Cnt_kind==0)
-        NO1 = Spe_Total_NO[Hwan];
+	NO1 = Spe_Total_NO[Hwan];
       else   
-        NO1 = Spe_Total_CNO[Hwan];
+	NO1 = Spe_Total_CNO[Hwan];
 
       /* initialize tmp_H */
 
       for (spin=0; spin<=SpinP_switch; spin++){
-        for (i=0; i<NO0; i++){
-          for (j=0; j<NO1; j++){
-            tmp_H[spin][i][j] = 0.0;                          
-          }
+	for (i=0; i<NO0; i++){
+	  for (j=0; j<NO1; j++){
+	    tmp_H[spin][i][j] = 0.0;                          
+	  }
 	}
       }
 
       /* summation of non-zero elements */
 
       for (Nog=0; Nog<NumOLG[Mc_AN][h_AN]-3; Nog+=4){
- 
-        Nc_0 = GListTAtoms1[Mc_AN][h_AN][Nog];
+
+        Nc_0 = GListTAtoms1[Mc_AN][h_AN][Nog+0];
         Nc_1 = GListTAtoms1[Mc_AN][h_AN][Nog+1];
         Nc_2 = GListTAtoms1[Mc_AN][h_AN][Nog+2];
         Nc_3 = GListTAtoms1[Mc_AN][h_AN][Nog+3];
 
-        Nh_0 = GListTAtoms2[Mc_AN][h_AN][Nog];
+        Nh_0 = GListTAtoms2[Mc_AN][h_AN][Nog+0];
         Nh_1 = GListTAtoms2[Mc_AN][h_AN][Nog+1];
         Nh_2 = GListTAtoms2[Mc_AN][h_AN][Nog+2];
         Nh_3 = GListTAtoms2[Mc_AN][h_AN][Nog+3];
+
+	/* store ChiV0 in tmp_ChiV0 */
  
-        /* store ChiV0 in tmp_ChiV0 */
+	for (spin=0; spin<=SpinP_switch; spin++){
+	  for (i=0; i<NO0; i++){
+	    tmp_ChiV0_0[spin][i] = ChiV0[spin][i][Nc_0];
+	    tmp_ChiV0_1[spin][i] = ChiV0[spin][i][Nc_1];
+	    tmp_ChiV0_2[spin][i] = ChiV0[spin][i][Nc_2];
+	    tmp_ChiV0_3[spin][i] = ChiV0[spin][i][Nc_3];
+	  }
+	}
+
+	/* store Orbs_Grid in tmp_Orbs_Grid */
+
+	for (j=0; j<NO1; j++){
+	  tmp_Orbs_Grid_0[j] = Orbs_Grid[Mh_AN][j][Nh_0];
+	  tmp_Orbs_Grid_1[j] = Orbs_Grid[Mh_AN][j][Nh_1];
+	  tmp_Orbs_Grid_2[j] = Orbs_Grid[Mh_AN][j][Nh_2];
+	  tmp_Orbs_Grid_3[j] = Orbs_Grid[Mh_AN][j][Nh_3];
+	}
+
+	/* integration */
  
-        for (spin=0; spin<=SpinP_switch; spin++){
-          for (i=0; i<NO0; i++){
-            tmp_ChiV0_0[spin][i] = ChiV0[spin][i][Nc_0];
-            tmp_ChiV0_1[spin][i] = ChiV0[spin][i][Nc_1];
-            tmp_ChiV0_2[spin][i] = ChiV0[spin][i][Nc_2];
-            tmp_ChiV0_3[spin][i] = ChiV0[spin][i][Nc_3];
-          }
-        }
- 
-        /* store Orbs_Grid in tmp_Orbs_Grid */
- 
-        for (j=0; j<NO1; j++){
-          tmp_Orbs_Grid_0[j] = Orbs_Grid[Mh_AN][j][Nh_0];
-          tmp_Orbs_Grid_1[j] = Orbs_Grid[Mh_AN][j][Nh_1];
-          tmp_Orbs_Grid_2[j] = Orbs_Grid[Mh_AN][j][Nh_2];
-          tmp_Orbs_Grid_3[j] = Orbs_Grid[Mh_AN][j][Nh_3];
-        }
- 
-        /* integration */
- 
-        for (spin=0; spin<=SpinP_switch; spin++){
-          for (i=0; i<NO0; i++){
-            tmp0_0 = tmp_ChiV0_0[spin][i];
-            tmp0_1 = tmp_ChiV0_1[spin][i];
-            tmp0_2 = tmp_ChiV0_2[spin][i];
-            tmp0_3 = tmp_ChiV0_3[spin][i];
-            for (j=0; j<NO1; j++){
-              tmp_H[spin][i][j] += ( tmp0_0*tmp_Orbs_Grid_0[j]
-				    +tmp0_1*tmp_Orbs_Grid_1[j]
-				    +tmp0_2*tmp_Orbs_Grid_2[j]
-				    +tmp0_3*tmp_Orbs_Grid_3[j]);
-            }
-	    /* snlxxx */
-            numop += 4*2*NO1;
-	    /**/
-          }
-        }
+	for (spin=0; spin<=SpinP_switch; spin++){
+	  for (i=0; i<NO0; i++){
+
+	    tmp0_0 = tmp_ChiV0_0[spin][i];
+	    tmp0_1 = tmp_ChiV0_1[spin][i];
+	    tmp0_2 = tmp_ChiV0_2[spin][i];
+	    tmp0_3 = tmp_ChiV0_3[spin][i];
+
+	    for (j=0; j<NO1; j++){
+	      tmp_H[spin][i][j] += (  tmp0_0*tmp_Orbs_Grid_0[j]
+			             +tmp0_1*tmp_Orbs_Grid_1[j]
+				     +tmp0_2*tmp_Orbs_Grid_2[j]
+				     +tmp0_3*tmp_Orbs_Grid_3[j]);
+	    }
+
+	  }
+	}
       }
 
-      mm=NumOLG[Mc_AN][h_AN]-(NumOLG[Mc_AN][h_AN]/4)*4;
+      mm = NumOLG[Mc_AN][h_AN]-(NumOLG[Mc_AN][h_AN]/4)*4;
+
       if(mm != 0){
  
-        for (Nog=NumOLG[Mc_AN][h_AN]-mm; Nog<NumOLG[Mc_AN][h_AN]; Nog++){
+	for (Nog=NumOLG[Mc_AN][h_AN]-mm; Nog<NumOLG[Mc_AN][h_AN]; Nog++){
  
-          Nc = GListTAtoms1[Mc_AN][h_AN][Nog];
-          Nh = GListTAtoms2[Mc_AN][h_AN][Nog];
+	  Nc = GListTAtoms1[Mc_AN][h_AN][Nog];
+	  Nh = GListTAtoms2[Mc_AN][h_AN][Nog];
  
-	  /* store ChiV0 in tmp_ChiV0 */
+	  /* store ChiV0 in tmp_ChiV0_0 */
  
-          for (spin=0; spin<=SpinP_switch; spin++){
-            for (i=0; i<NO0; i++){
-              tmp_ChiV0[spin][i] = ChiV0[spin][i][Nc];
-            }
-          }
+	  for (spin=0; spin<=SpinP_switch; spin++){
+	    for (i=0; i<NO0; i++){
+	      tmp_ChiV0_0[spin][i] = ChiV0[spin][i][Nc];
+	    }
+	  }
  
 	  /* store Orbs_Grid in tmp_Orbs_Grid */
  
-          for (j=0; j<NO1; j++){
-            tmp_Orbs_Grid[j] = Orbs_Grid[Mh_AN][j][Nh];
-          }
+	  for (j=0; j<NO1; j++){
+	    tmp_Orbs_Grid_0[j] = Orbs_Grid[Mh_AN][j][Nh];
+	  }
  
 	  /* integration */
- 
-          for (spin=0; spin<=SpinP_switch; spin++){
-            for (i=0; i<NO0; i++){
-              tmp0 = tmp_ChiV0[spin][i];
-              for (j=0; j<NO1; j++){
-                tmp_H[spin][i][j] += tmp0*tmp_Orbs_Grid[j];
-              }
-            }
-	    /* snlxxx */
-            numop += 2*NO0*NO1;
-	    /**/
-          }
-        }
+
+	  for (spin=0; spin<=SpinP_switch; spin++){
+
+	    for (i=0; i<NO0; i++){
+	      tmp0 = tmp_ChiV0_0[spin][i];
+	      for (j=0; j<NO1; j++){
+		tmp_H[spin][i][j] += tmp0*tmp_Orbs_Grid_0[j];
+	      }
+	    }
+	  }
+	}
       }
 
       /* add tmp_H to H or CntH */
- 
+
       if (Cnt_kind==0){
-        for (spin=0; spin<=SpinP_switch; spin++){
-          for (i=0; i<NO0; i++){
-            for (j=0; j<NO1; j++){
-              H[spin][Mc_AN][h_AN][i][j] += tmp_H[spin][i][j]*GridVol;
-            }
-          }
-	  /* snlxxx */
-          numop += 2*NO0*NO1;
-	  /**/
-        }
+
+	for (spin=0; spin<=SpinP_switch; spin++){
+	  for (i=0; i<NO0; i++){
+	    for (j=0; j<NO1; j++){
+	      H[spin][Mc_AN][h_AN][i][j] += tmp_H[spin][i][j]*GridVol;
+	    }
+	  }
+	}
       }
  
       else{
-        for (spin=0; spin<=SpinP_switch; spin++){
-          for (i=0; i<NO0; i++){
-            for (j=0; j<NO1; j++){
-              CntH[spin][Mc_AN][h_AN][i][j] += tmp_H[spin][i][j]*GridVol;
-            }
-          }
-	  /* snlxxx */
-          numop += 2*NO0*NO1;
-	  /**/
-        }
+	for (spin=0; spin<=SpinP_switch; spin++){
+	  for (i=0; i<NO0; i++){
+	    for (j=0; j<NO1; j++){
+	      CntH[spin][Mc_AN][h_AN][i][j] += tmp_H[spin][i][j]*GridVol;
+	    }
+	  }
+	}
       }
-    }
 
-    dtime(&Etime_atom);
-    time_per_atom[Gc_AN] += Etime_atom - Stime_atom;
-  }
-  /* snlxxx */
+    } /* Nloop */
+
+    /* freeing of arrays */ 
+
+    free(tmp_Orbs_Grid_3);
+    free(tmp_Orbs_Grid_2);
+    free(tmp_Orbs_Grid_1);
+    free(tmp_Orbs_Grid_0);
+
+    for (i=0; i<4; i++){
+      free(tmp_ChiV0_3[i]);
+    }
+    free(tmp_ChiV0_3);
+
+    for (i=0; i<4; i++){
+      free(tmp_ChiV0_2[i]);
+    }
+    free(tmp_ChiV0_2);
+
+    for (i=0; i<4; i++){
+      free(tmp_ChiV0_1[i]);
+    }
+    free(tmp_ChiV0_1);
+
+    for (i=0; i<4; i++){
+      free(tmp_ChiV0_0[i]);
+    }
+    free(tmp_ChiV0_0);
+
+    for (i=0; i<4; i++){
+      for (j=0; j<List_YOUSO[7]; j++){
+	free(tmp_H[i][j]);
+      }
+      free(tmp_H[i]);
+    }
+    free(tmp_H);
+
+    for (i=0; i<4; i++){
+      for (j=0; j<List_YOUSO[7]; j++){
+	free(ChiV0[i][j]);
+      }
+      free(ChiV0[i]);
+    }
+    free(ChiV0);
+
+    free(Chi0);
+
+#pragma omp flush(H,CntH)
+
+  } /* #pragma omp parallel */
+
   dtime(&time2);
-  mflops=numop/(time2-time1)*1.0e-6;
-  if(myid==0 && measure_time) 
-    printf("Time for Part2=%18.5f mflops=%12.3f\n",(time2-time1),mflops);fflush(stdout);
-  /**/
+  if(myid==0 && measure_time){
+    printf("Time for Part2=%18.5f\n",(time2-time1));fflush(stdout);
+  }
   
-  /****************************************************
-    freeing of arrays:
-
-    double Chi0[List_YOUSO[7]];
-    double Chi1[List_YOUSO[7]];
-    double ChiV0[4][List_YOUSO[7]][List_YOUSO[11]];
-  ****************************************************/
-
-  free(Chi0);
-  free(Chi1);
-
-  for (i=0; i<4; i++){
-    for (j=0; j<List_YOUSO[7]; j++){
-      free(ChiV0[i][j]);
-    }
-    free(ChiV0[i]);
-  }
-  free(ChiV0);
-
-  for (i=0; i<4; i++){
-    free(tmp_ChiV0[i]);
-  }
-  free(tmp_ChiV0);
-
-  for (i=0; i<4; i++){
-    free(tmp_ChiV0_0[i]);
-  }
-  free(tmp_ChiV0_0);
-
-  for (i=0; i<4; i++){
-    free(tmp_ChiV0_1[i]);
-  }
-  free(tmp_ChiV0_1);
-
-  for (i=0; i<4; i++){
-    free(tmp_ChiV0_2[i]);
-  }
-  free(tmp_ChiV0_2);
-
-  for (i=0; i<4; i++){
-    free(tmp_ChiV0_3[i]);
-  }
-  free(tmp_ChiV0_3);
-
-  free(tmp_Orbs_Grid);
-
-  free(tmp_Orbs_Grid_0);
-  free(tmp_Orbs_Grid_1);
-  free(tmp_Orbs_Grid_2);
-  free(tmp_Orbs_Grid_3);
-
-  for (i=0; i<4; i++){
-    for (j=0; j<List_YOUSO[7]; j++){
-      free(tmp_H[i][j]);
-    }
-    free(tmp_H[i]);
-  }
-  free(tmp_H);
-
   /*****************************************************
    When the restart file is used, a simple mixing of the
    Hamiltonian matrix is made at SCF_iter==2 
   *****************************************************/
 
   /*
-  if (SucceedReadingDMfile==1 && SCF_iter==2){
+    if (SucceedReadingDMfile==1 && SCF_iter==2){
     if (Cnt_kind==0)  simple_mixing_H(H);
     else              simple_mixing_H(CntH);
-  }
+    }
   */
 
+  /* freeing of arrays */ 
+
+  free(OneD2h_AN);
+  free(OneD2Mc_AN);
+
   /* for time */
 
   MPI_Barrier(mpi_comm_level1);
@@ -599,7 +630,7 @@ void simple_mixing_H(double *****Hcnt)
            i_vec[7]!=TNO1           
          )
       {
-        printf("Failed (1) in roading the restart file %s\n",fileHKS); fflush(stdout);     
+        printf("Failed (1) in reading the restart file %s\n",fileHKS); fflush(stdout);     
         my_check = 0;
       }
 
@@ -675,7 +706,7 @@ void simple_mixing_H(double *****Hcnt)
       fclose(fp);
     }
     else{
-      printf("Failed (2) in roading the restart file %s\n",fileHKS); fflush(stdout);     
+      printf("Failed (2) in reading the restart file %s\n",fileHKS); fflush(stdout);     
       my_check = 0;
     }
   }
