@@ -1,3 +1,15 @@
+/**********************************************************************
+  TRAN_Calc_SurfGreen_direct.c:
+
+  TRAN_Calc_SurfGreen_direct.c is a subroutine to calculate the  
+  surface Green's function.
+
+  Log of TRAN_Calc_SurfGreen_direct.o
+
+     11/Dec/2005   Released by H.Kino
+
+***********************************************************************/
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -15,20 +27,6 @@
 
 
 
-/* calculate surface green function */
-void TRAN_Calc_SurfGreen_direct(
-                                dcomplex w,
-                                int n,
-                                dcomplex *h00,
-                                dcomplex *h01,
-                                dcomplex *s00,
-                                dcomplex *s01,
-                                int iteration_max,
-                                double eps,
-                                dcomplex *gr /* output */
-                                );
-
-
 static void TRAN_Calc_SurfGreen_Normal(
                                         /* input */
 				dcomplex w,
@@ -72,24 +70,6 @@ static void TRAN_Calc_SurfGreen_transfer(
 
 
 
-
-/* fortran interface */
-void tran_calc_surfgreen_direct(
-                                dcomplex *w,
-                                int *n,
-                                dcomplex *h00,
-                                dcomplex *h01,
-                                dcomplex *s00,
-                                dcomplex *s01,
-                                int *iteration_max,
-                                double *eps,
-                                dcomplex *gr /* output */
-                                )
-{
-      TRAN_Calc_SurfGreen_direct(*w, *n, h00,h01,s00,s01, *iteration_max, *eps, gr);
-
-}
-
 /*
  * calculate surface green function
  *
@@ -98,7 +78,7 @@ void tran_calc_surfgreen_direct(
 */
 
 void TRAN_Calc_SurfGreen_direct(
-                                        /* input */
+                                /* input */
 				dcomplex w,
 				int n, 
 				dcomplex *h00, 
@@ -107,7 +87,8 @@ void TRAN_Calc_SurfGreen_direct(
 				dcomplex *s01,
                                 int iteration_max,
                                 double eps,
-	 			dcomplex *gr /* output */
+                                /* output */
+	 			dcomplex *gr 
 				)
 {
 
@@ -157,7 +138,6 @@ void TRAN_Calc_SurfGreen_Normal(
 #define gt_ref(i,j) gt[ n*((j)-1)+(i)-1 ]  
 
 {
-  static char *thisprogram="TRAN_Calc_SurfGreen_direct";
   int i,j,iter;
   dcomplex a,b;
   double rms2,val;
@@ -181,7 +161,6 @@ void TRAN_Calc_SurfGreen_Normal(
   gr02 = (dcomplex*)malloc(sizeof(dcomplex)*n*n) ;
   gt = (dcomplex*)malloc(sizeof(dcomplex)*n*n) ;
 
-
   for (i=1;i<=n;i++) {    
     for (j=1;j<=n;j++) {
       es0_ref(i,j).r = w.r*s00_ref(i,j).r - w.i*s00_ref(i,j).i - h00_ref(i,j).r;
@@ -391,7 +370,6 @@ void TRAN_Calc_SurfGreen_Multiple_Inverse(
 #define tmpv1_ref(i,j) tmpv1[ n*((j)-1)+(i)-1 ]
 #define tmpv2_ref(i,j) tmpv2[ n*((j)-1)+(i)-1 ]
 {
-  static char *thisprogram="TRAN_Calc_SurfGreen_tranfermatrix";
   int i,j,iter;
   dcomplex a,b;
   double rms2,val;
@@ -625,7 +603,6 @@ void TRAN_Calc_SurfGreen_transfer(
 #define tt3_ref(i,j) tt3[ n*((j)-1)+(i)-1 ]
 
 {
-  static char *thisprogram="TRAN_Calc_SurfGreen_tranfermatrix";
   int i,j,iter;
   dcomplex a,b,cval;
   double rms2,val;
@@ -805,7 +782,6 @@ void TRAN_Calc_SurfGreen_transfer(
 
   Lapack_LU_Zinverse(n,G00); /* (53) */
 
-
   free(H01);
   free(H10);
   free(gr00);
@@ -820,5 +796,4 @@ void TRAN_Calc_SurfGreen_transfer(
   free(bar_t_i);
   free(t_i);
 
-
 }
