@@ -8,11 +8,10 @@
   Log of TRAN_DFT.c:
 
      11/Dec/2005   Released by H.Kino
-     06/June/2005  Modified by T.Ozaki
+     24/July/2008  Modified by T.Ozaki
 
 ***********************************************************************/
 
-#define WRITE_DENSITY 0
 #define MEASURE_TIME  0
 
 #include <stdio.h> 
@@ -26,11 +25,63 @@
 #include <mpi.h>
 #endif
 
+#ifdef noomp
+#include "mimic_omp.h"
+#else
+#include <omp.h>
+#endif
+
 #include "tran_prototypes.h"
 #include "tran_variables.h"
- 
+
 void dtime(double *);
 
+void k_inversion(int i,  int j,  int k, 
+                 int mi, int mj, int mk, 
+                 int *ii, int *ij, int *ik ); 
+
+double Band_DFT_Col(int SCF_iter,
+                    int knum_i, int knum_j, int knum_k,
+		    int SpinP_switch,
+		    double *****nh,
+		    double *****ImNL,
+		    double ****CntOLP,
+		    double *****CDM,
+		    double *****EDM,
+		    double Eele0[2], 
+                    double Eele1[2], 
+		    int *MP,
+		    int *order_GA,
+		    double *ko,
+		    double *koS,
+		    double ***EIGEN,
+		    double *H1,
+		    double *S1,
+		    double *CDM1,
+		    double *EDM1,
+		    dcomplex **H,
+		    dcomplex **S,
+		    dcomplex **C,
+                    dcomplex *BLAS_S,
+		    int ***k_op,
+		    int *T_k_op,
+		    int **T_k_ID,
+		    double *T_KGrids1,
+		    double *T_KGrids2,
+		    double *T_KGrids3,
+                    int myworld1,
+		    int *NPROCS_ID1,
+		    int *Comm_World1,
+		    int *NPROCS_WD1,
+		    int *Comm_World_StartID1,
+		    MPI_Comm *MPI_CommWD1,
+                    int myworld2,
+		    int *NPROCS_ID2,
+		    int *NPROCS_WD2,
+		    int *Comm_World2,
+		    int *Comm_World_StartID2,
+		    MPI_Comm *MPI_CommWD2);
+
 void Make_Comm_Worlds(
    MPI_Comm MPI_Curret_Comm_WD,   
    int myid0,
@@ -57,6 +108,34 @@ static void TRAN_Add_MAT(
     dcomplex *out
     );
 
+static double TRAN_DFT_Original(
+		/* input */
+                MPI_Comm comm1,
+                int level_stdout,
+		int iter, 
+		int SpinP_switch,
+		double *****nh,  /* H */
+		double *****ImNL, /* not used, s-o coupling */
+		double ****CntOLP, 
+		int atomnum,
+		int Matomnum,
+		int *WhatSpecies,
+		int *Spe_Total_CNO,
+		int *FNAN,
+		int **natn, 
+		int **ncn,
+		int *M2G, 
+		int *G2ID,
+                int *F_G2M, 
+		int **atv_ijk,
+		int *List_YOUSO,
+		/* output */
+		double *****CDM,  /* output, density matrix */
+		double *****EDM,  /* not used */
+                double ***TRAN_DecMulP, /* output, partial DecMulP */
+		double Eele0[2], double Eele1[2], 
+                double ChemP_e0[2]); 
+
 
 static void TRAN_DFT_Kdependent(
 			  /* input */
@@ -95,10 +174,430 @@ static void TRAN_DFT_Kdependent(
 
 
 
-
-
-
 double TRAN_DFT(
+		/* input */
+                MPI_Comm comm1,
+                int SucceedReadingDMfile, 
+                int level_stdout,
+		int iter, 
+		int SpinP_switch,
+		double *****nh,  /* H */
+		double *****ImNL, /* not used, s-o coupling */
+		double ****CntOLP, 
+		int atomnum,
+		int Matomnum,
+		int *WhatSpecies,
+		int *Spe_Total_CNO,
+		int *FNAN,
+		int **natn, 
+		int **ncn,
+		int *M2G, 
+		int *G2ID,
+                int *F_G2M, 
+		int **atv_ijk,
+		int *List_YOUSO,
+		/* output */
+		double *****CDM,  /* output, density matrix */
+		double *****EDM,  /* not used */
+                double ***TRAN_DecMulP, /* output, partial DecMulP */
+		double Eele0[2], double Eele1[2], 
+                double ChemP_e0[2]) 
+{
+  double TStime,TEtime,time5;
+
+  dtime(&TStime);
+
+  /*************************************************
+    if 3<SCF_iter, employ TRAN_DFT
+  *************************************************/
+
+  if (3<iter || SucceedReadingDMfile==1){
+   
+    TRAN_DFT_Original( comm1,
+		       level_stdout,
+		       iter, 
+		       SpinP_switch,
+		       nh,  /* H */
+		       ImNL, /* not used, s-o coupling */
+		       CntOLP, 
+		       atomnum,
+		       Matomnum,
+		       WhatSpecies,
+		       Spe_Total_CNO,
+		       FNAN,
+		       natn, 
+		       ncn,
+		       M2G, 
+		       G2ID,
+		       F_G2M, 
+		       atv_ijk,
+		       List_YOUSO,
+		       /* output */
+		       CDM,  /* output, density matrix */
+		       EDM,  /* not used */
+		       TRAN_DecMulP, /* output, partial DecMulP */
+		       Eele0, Eele1, 
+		       ChemP_e0); 
+  }
+
+  /*************************************************
+    if SCF_iter<=3, employ the band diagonalization
+  *************************************************/
+
+  else {
+
+    int i,j,k,n,n2,wanA;
+    int ii,ij,ik,T_knum,size_H1;
+    int Kspace_grid1,Kspace_grid2,Kspace_grid3;
+    int *MP;
+    int *order_GA;
+    int *My_NZeros;
+    int *SP_NZeros;
+    int *SP_Atoms;
+    double *ko;
+    double *koS;
+    double tmp;
+    dcomplex **H_Band_Col;
+    dcomplex **S_Band;
+    dcomplex **C_Band_Col;
+    dcomplex *BLAS_S;
+    double *H1_Band_Col;
+    double *S1_Band_Col;
+    double *CDM1_Band_Col;
+    double *EDM1_Band_Col;
+    int ***k_op;
+    int *T_k_op;
+    int **T_k_ID;
+    double *T_KGrids1,*T_KGrids2,*T_KGrids3;
+    double ***EIGEN_Band_Col;
+
+    int numprocs0,myid0;
+    int numprocs1,myid1;
+
+    int Num_Comm_World1;
+    int Num_Comm_World2;
+
+    int myworld1;
+    int myworld2;
+
+    int *NPROCS_ID1;
+    int *Comm_World1;
+    int *NPROCS_WD1;
+    int *Comm_World_StartID1;
+    MPI_Comm *MPI_CommWD1;
+
+    int *NPROCS_ID2;
+    int *NPROCS_WD2;
+    int *Comm_World2;
+    int *Comm_World_StartID2;
+    MPI_Comm *MPI_CommWD2;
+
+    /* MPI */ 
+    MPI_Comm_size(comm1,&numprocs0);
+    MPI_Comm_rank(comm1,&myid0);
+
+    Kspace_grid1 = 1;
+    Kspace_grid2 = TRAN_Kspace_grid2;
+    Kspace_grid3 = TRAN_Kspace_grid3;
+    
+    n = 0;
+    for (i=1; i<=atomnum; i++){
+      wanA = WhatSpecies[i];
+      n += Spe_Total_CNO[wanA];
+    }
+    n2 = n + 2;
+
+    MP = (int*)malloc(sizeof(int)*List_YOUSO[1]);
+    order_GA = (int*)malloc(sizeof(int)*(List_YOUSO[1]+1));
+
+    My_NZeros = (int*)malloc(sizeof(int)*numprocs0);
+    SP_NZeros = (int*)malloc(sizeof(int)*numprocs0);
+    SP_Atoms = (int*)malloc(sizeof(int)*numprocs0);
+
+    ko = (double*)malloc(sizeof(double)*(n+1));
+    koS = (double*)malloc(sizeof(double)*(n+1));
+
+    H_Band_Col = (dcomplex**)malloc(sizeof(dcomplex*)*(n+1));
+    for (j=0; j<n+1; j++){
+      H_Band_Col[j] = (dcomplex*)malloc(sizeof(dcomplex)*(n+1));
+    }
+
+    S_Band = (dcomplex**)malloc(sizeof(dcomplex*)*(n+1));
+    for (i=0; i<n+1; i++){
+      S_Band[i] = (dcomplex*)malloc(sizeof(dcomplex)*(n+1));
+    }
+
+    C_Band_Col = (dcomplex**)malloc(sizeof(dcomplex*)*(n+1));
+    for (j=0; j<n+1; j++){
+      C_Band_Col[j] = (dcomplex*)malloc(sizeof(dcomplex)*(n+1));
+    }
+
+    BLAS_S = (dcomplex*)malloc(sizeof(dcomplex)*n*n);
+
+    /* find size_H1 */
+
+    size_H1 = Get_OneD_HS_Col(0, nh[0], &tmp, MP, order_GA, My_NZeros, SP_NZeros, SP_Atoms);
+
+    H1_Band_Col   = (double*)malloc(sizeof(double)*size_H1);
+    S1_Band_Col   = (double*)malloc(sizeof(double)*size_H1);
+    CDM1_Band_Col = (double*)malloc(sizeof(double)*size_H1);
+    EDM1_Band_Col = (double*)malloc(sizeof(double)*size_H1);
+
+    k_op = (int***)malloc(sizeof(int**)*Kspace_grid1);
+    for (i=0;i<Kspace_grid1; i++) {
+      k_op[i] = (int**)malloc(sizeof(int*)*Kspace_grid2);
+      for (j=0;j<Kspace_grid2; j++) {
+	k_op[i][j] = (int*)malloc(sizeof(int)*Kspace_grid3);
+      }
+    }
+
+    for (i=0; i<Kspace_grid1; i++) {
+      for (j=0; j<Kspace_grid2; j++) {
+	for (k=0; k<Kspace_grid3; k++) {
+	  k_op[i][j][k] = -999;
+	}
+      }
+    }
+
+    for (i=0; i<Kspace_grid1; i++) {
+      for (j=0; j<Kspace_grid2; j++) {
+	for (k=0; k<Kspace_grid3; k++) {
+
+	  if ( k_op[i][j][k]==-999 ) {
+	    k_inversion(i,j,k,Kspace_grid1,Kspace_grid2,Kspace_grid3,&ii,&ij,&ik);
+	    if ( i==ii && j==ij && k==ik ) {
+	      k_op[i][j][k]    = 1;
+	    }
+
+	    else {
+	      k_op[i][j][k]    = 2;
+	      k_op[ii][ij][ik] = 0;
+	    }
+	  }
+	} /* k */
+      } /* j */
+    } /* i */
+  
+    /* find T_knum */
+
+    T_knum = 0;
+    for (i=0; i<Kspace_grid1; i++) {
+      for (j=0; j<Kspace_grid2; j++) {
+	for (k=0; k<Kspace_grid3; k++) {
+	  if (0<k_op[i][j][k]){
+	    T_knum++;
+	  }
+	}
+      }
+    }
+
+    T_KGrids1 = (double*)malloc(sizeof(double)*T_knum);
+    T_KGrids2 = (double*)malloc(sizeof(double)*T_knum);
+    T_KGrids3 = (double*)malloc(sizeof(double)*T_knum);
+    T_k_op    = (int*)malloc(sizeof(int)*T_knum);
+
+    T_k_ID    = (int**)malloc(sizeof(int*)*2);
+    for (i=0; i<2; i++){
+      T_k_ID[i] = (int*)malloc(sizeof(int)*T_knum);
+    }
+
+    EIGEN_Band_Col  = (double***)malloc(sizeof(double**)*List_YOUSO[23]);
+    for (i=0; i<List_YOUSO[23]; i++){
+      EIGEN_Band_Col[i] = (double**)malloc(sizeof(double*)*T_knum);
+      for (j=0; j<T_knum; j++){
+	EIGEN_Band_Col[i][j] = (double*)malloc(sizeof(double)*(n+1));
+	for (k=0; k<(n+1); k++) EIGEN_Band_Col[i][j][k] = 1.0e+5;
+      }
+    }
+
+    /***********************************************
+      allocation of arrays for the first world 
+      and 
+      make the first level worlds 
+    ***********************************************/
+
+    Num_Comm_World1 = SpinP_switch + 1; 
+
+    NPROCS_ID1 = (int*)malloc(sizeof(int)*numprocs0); 
+    Comm_World1 = (int*)malloc(sizeof(int)*numprocs0); 
+    NPROCS_WD1 = (int*)malloc(sizeof(int)*Num_Comm_World1); 
+    Comm_World_StartID1 = (int*)malloc(sizeof(int)*Num_Comm_World1); 
+    MPI_CommWD1 = (MPI_Comm*)malloc(sizeof(MPI_Comm)*Num_Comm_World1);
+
+    Make_Comm_Worlds(comm1, myid0, numprocs0, Num_Comm_World1, &myworld1, MPI_CommWD1, 
+		     NPROCS_ID1, Comm_World1, NPROCS_WD1, Comm_World_StartID1);
+
+    MPI_Comm_size(MPI_CommWD1[myworld1],&numprocs1);
+    MPI_Comm_rank(MPI_CommWD1[myworld1],&myid1);
+
+    /***********************************************
+        allocation of arrays for the second world 
+        and 
+        make the second level worlds 
+    ***********************************************/
+
+    if (T_knum<=numprocs1){
+
+      Num_Comm_World2 = T_knum;
+
+      NPROCS_ID2 = (int*)malloc(sizeof(int)*numprocs1);
+      Comm_World2 = (int*)malloc(sizeof(int)*numprocs1);
+      NPROCS_WD2 = (int*)malloc(sizeof(int)*Num_Comm_World2);
+      Comm_World_StartID2 = (int*)malloc(sizeof(int)*Num_Comm_World2);
+      MPI_CommWD2 = (MPI_Comm*)malloc(sizeof(MPI_Comm)*Num_Comm_World2);
+
+      Make_Comm_Worlds(MPI_CommWD1[myworld1], myid1, numprocs1, Num_Comm_World2, &myworld2, MPI_CommWD2, 
+		       NPROCS_ID2, Comm_World2, NPROCS_WD2, Comm_World_StartID2);
+    }
+
+    /***********************************************
+                     call Band_DFT_Col
+     ***********************************************/
+    
+    time5 = Band_DFT_Col( 1,
+			  Kspace_grid1, Kspace_grid2, Kspace_grid3,
+			  SpinP_switch,
+			  nh,
+			  ImNL,
+			  CntOLP,
+			  CDM,
+			  EDM,
+			  Eele0,
+			  Eele1, 
+			  MP,order_GA,ko,koS,EIGEN_Band_Col,
+			  H1_Band_Col,S1_Band_Col,
+			  CDM1_Band_Col,EDM1_Band_Col,
+			  H_Band_Col,S_Band,C_Band_Col, 
+			  BLAS_S,
+			  k_op,T_k_op,T_k_ID,
+			  T_KGrids1,T_KGrids2,T_KGrids3,
+			  myworld1,
+			  NPROCS_ID1,
+			  Comm_World1,
+			  NPROCS_WD1,
+			  Comm_World_StartID1,
+			  MPI_CommWD1,
+			  myworld2,
+			  NPROCS_ID2,
+			  Comm_World2,
+			  NPROCS_WD2,
+			  Comm_World_StartID2,
+			  MPI_CommWD2);
+
+    Eele0[0] = 0.0;
+    Eele0[1] = 0.0;
+ 
+    Eele1[0] = 0.0;
+    Eele1[1] = 0.0;
+
+    /***********************************************
+       freeing of arrays
+    ***********************************************/
+
+    n = 0;
+    for (i=1; i<=atomnum; i++){
+      wanA  = WhatSpecies[i];
+      n += Spe_Total_CNO[wanA];
+    }
+    n2 = n + 2;
+
+    free(MP);
+    free(order_GA);
+
+    free(My_NZeros);
+    free(SP_NZeros);
+    free(SP_Atoms);
+
+    free(ko);
+    free(koS);
+
+    for (j=0; j<n+1; j++){
+      free(H_Band_Col[j]);
+    }
+    free(H_Band_Col);
+
+    for (i=0; i<n+1; i++){
+      free(S_Band[i]);
+    }
+    free(S_Band);
+
+    for (j=0; j<n+1; j++){
+      free(C_Band_Col[j]);
+    }
+    free(C_Band_Col);
+
+    free(BLAS_S);
+
+    free(H1_Band_Col);
+    free(S1_Band_Col);
+    free(CDM1_Band_Col);
+    free(EDM1_Band_Col);
+
+    for (i=0; i<Kspace_grid1; i++) {
+      for (j=0; j<Kspace_grid2; j++) {
+	free(k_op[i][j]);
+      }
+      free(k_op[i]);
+    }
+    free(k_op);
+
+    free(T_KGrids1);
+    free(T_KGrids2);
+    free(T_KGrids3);
+    free(T_k_op);
+
+    for (i=0; i<2; i++){
+      free(T_k_ID[i]);
+    }
+    free(T_k_ID);
+
+    for (i=0; i<List_YOUSO[23]; i++){
+      for (j=0; j<T_knum; j++){
+	free(EIGEN_Band_Col[i][j]);
+      }
+      free(EIGEN_Band_Col[i]);
+    }
+    free(EIGEN_Band_Col);
+
+    /* freeing of arrays for the second world */
+
+    if (T_knum<=numprocs1){
+
+      if (Num_Comm_World2<=numprocs1){
+        MPI_Comm_free(&MPI_CommWD2[myworld2]);
+      }
+
+      free(MPI_CommWD2);
+      free(Comm_World_StartID2);
+      free(NPROCS_WD2);
+      free(Comm_World2);
+      free(NPROCS_ID2);
+    }
+
+    /* freeing of arrays for the first world */
+
+    if (Num_Comm_World1<=numprocs0){
+      MPI_Comm_free(&MPI_CommWD1[myworld1]);
+    }
+
+    free(MPI_CommWD1);
+    free(Comm_World_StartID1);
+    free(NPROCS_WD1);
+    free(Comm_World1);
+    free(NPROCS_ID1);
+  }
+
+  /* for elapsed time */
+  dtime(&TEtime);
+  return TEtime - TStime;
+
+}
+
+
+
+
+
+double TRAN_DFT_Original(
 		/* input */
                 MPI_Comm comm1,
                 int level_stdout,
@@ -165,9 +664,9 @@ double TRAN_DFT(
     printf("<TRAN_DFT>\n");
   }
 
-  /***********************************
-  ChemP_e0 will be used in outputfile1
-  ************************************/
+  /***************************************
+    ChemP_e0 will be used in outputfile1
+  ***************************************/
 
   ChemP_e0[0] = ChemP_e[0];
   ChemP_e0[1] = ChemP_e[1];
@@ -260,6 +759,30 @@ double TRAN_DFT(
     }
   }
 
+  /***************************************
+    print out     
+  ***************************************/
+
+  if (myid0==Host_ID){
+
+    printf(" KGrids2: ");fflush(stdout);
+
+    for (i=0;i<TRAN_Kspace_grid2;i++){
+      if (TRAN_Kspace_grid2==1)  k2 = 0.0;
+      else                       k2 = -0.5 + (2.0*(double)i+1.0)/(2.0*(double)TRAN_Kspace_grid2) + Shift_K_Point;
+      printf("%9.5f ",k2);fflush(stdout);
+    }
+    printf("\n");fflush(stdout);
+
+    printf(" KGrids3: ");fflush(stdout);
+    for (i=0;i<TRAN_Kspace_grid3;i++){
+      if (TRAN_Kspace_grid3==1)  k3 = 0.0;
+      else                       k3 = -0.5 + (2.0*(double)i+1.0)/(2.0*(double)TRAN_Kspace_grid3) - Shift_K_Point;
+      printf("%9.5f ",k3);fflush(stdout);
+    }
+    printf("\n");fflush(stdout);
+  }
+
   /***************************************************
    allocate calculations of k-points into processors 
   ***************************************************/
@@ -338,7 +861,7 @@ double TRAN_DFT(
   SP_NZeros = (int*)malloc(sizeof(int)*numprocs0);
   SP_Atoms = (int*)malloc(sizeof(int)*numprocs0);
 
-  size_H1 = Get_OneD_HS_Col(0, nh[0], S1, MP, order_GA, My_NZeros, SP_NZeros, SP_Atoms);
+  size_H1 = Get_OneD_HS_Col(0, nh[0], &tmp, MP, order_GA, My_NZeros, SP_NZeros, SP_Atoms);
 
   DM1 = (double**)malloc(sizeof(double*)*(SpinP_switch+1));
   for (k=0; k<(SpinP_switch+1); k++){
@@ -422,14 +945,6 @@ double TRAN_DFT(
 	RnB = ncn[GA_AN][LB_AN];
 	wanB = WhatSpecies[GB_AN];
 	tnoB = Spe_Total_CNO[wanB];
-	Bnum = MP[GB_AN];
-	l1 = atv_ijk[RnB][1];
-	l2 = atv_ijk[RnB][2];
-	l3 = atv_ijk[RnB][3];
-
-	kRn = k2*(double)l2 + k3*(double)l3;
-	si = (double)k_op*sin(2.0*PI*kRn);
-	co = (double)k_op*cos(2.0*PI*kRn);
 
 	for (i=0;i<tnoA;i++) {
 	  for (j=0;j<tnoB;j++) {
@@ -469,6 +984,7 @@ double TRAN_DFT(
         l3 = atv_ijk[RnB][3];
 
         /* DM between C-L or C-R */
+
         if (l1==-1 || l1==1){
 	  for (i=0; i<tnoA; i++){
 	    for (j=0; j<tnoB; j++){
@@ -476,6 +992,7 @@ double TRAN_DFT(
 	    }
 	  }
         }
+
       }
     }
   }
@@ -651,6 +1168,10 @@ double TRAN_DFT(
 
   /* for elapsed time */
   dtime(&TEtime);
+  if (myid0==Host_ID){
+    printf("<TRAN_DFT> time=%lf\n", TEtime-TStime);fflush(stdout);
+  }
+
   return TEtime - TStime;
 }
 
@@ -706,54 +1227,30 @@ static void TRAN_DFT_Kdependent(
 #define Gless_ref(i,j) Gless[ NUM_c*((j)-1) + (i)-1 ]
 
 {
-  int i,j,k,iside; 
+  int i,j,k,iside,spinsize; 
   int *MP;
   int  iw,iw_method;
   dcomplex w, w_weight;
-  dcomplex *GC,*GRL,*GRR,*SigmaL, *SigmaR; 
-  dcomplex *v1,*Gless,*GCLorR;
+  dcomplex *GC_Ad;
+  dcomplex *GC,*GRL,*GRR;
+  dcomplex *SigmaL, *SigmaR; 
+  dcomplex *SigmaL_Ad, *SigmaR_Ad; 
+  dcomplex *work1,*work2,*Gless;
   dcomplex **v2;
   double dum;
   double TStime,TEtime;
   int MA_AN, GA_AN, wanA, tnoA, Anum;
   int LB_AN, GB_AN, wanB, tnoB, Bnum; 
 
-  /* debug */
-  double **Density;
-  double density_sum;
-  /* end debug*/
-  
   static int ID;
-  int **iwIdx, Miwmax, Miw,iw0; 
+  int Miw,iw0; 
   double time_a0, time_a1, time_a2; 
-  
-  /* parallel setup */
-
-  iwIdx=(int**)malloc(sizeof(int*)*numprocs);
-  Miwmax = (tran_omega_n_scf)/numprocs+1;
-  for (i=0; i<numprocs; i++) {
-    iwIdx[i]=(int*)malloc(sizeof(int)*Miwmax);
-  }
-
-  TRAN_Distribute_Node_Idx(0, tran_omega_n_scf-1, numprocs, Miwmax,
-                           iwIdx); /* output */
 
   /* setup MP */
-  TRAN_Set_MP(0, atomnum, WhatSpecies, Spe_Total_CNO, &NUM_c, MP);
+  TRAN_Set_MP(0, atomnum, WhatSpecies, Spe_Total_CNO, &NUM_c, &i);
   MP = (int*)malloc(sizeof(int)*(NUM_c+1));
   TRAN_Set_MP(1, atomnum, WhatSpecies, Spe_Total_CNO, &NUM_c, MP);
   
-  /*debug */
-
-  if (WRITE_DENSITY){
-    Density = (double**)malloc(sizeof(double*)*(SpinP_switch+1));
-    for (k=0;k<=SpinP_switch;k++) {
-      Density[k] = (double*)malloc(sizeof(double)*NUM_c);
-      for (i=0;i<NUM_c;i++) { Density[k][i]=0.0; }
-    }
-  }
-  /*end debug */
-  
   /* initialize */
   TRAN_Set_Value_double(SCC,NUM_c*NUM_c,    0.0,0.0);
   TRAN_Set_Value_double(SCL,NUM_c*NUM_e[0], 0.0,0.0);
@@ -792,7 +1289,6 @@ static void TRAN_DFT_Kdependent(
                           ncn,
                           atv_ijk);
 
-
   if (MEASURE_TIME){
     dtime(&time_a0);
   }
@@ -807,167 +1303,162 @@ static void TRAN_DFT_Kdependent(
   }
   
   GC = (dcomplex*)malloc(sizeof(dcomplex)*NUM_c* NUM_c);
+  GC_Ad = (dcomplex*)malloc(sizeof(dcomplex)*NUM_c* NUM_c);
+
   GRL = (dcomplex*)malloc(sizeof(dcomplex)*NUM_e[0]* NUM_e[0]);
   GRR = (dcomplex*)malloc(sizeof(dcomplex)*NUM_e[1]* NUM_e[1]);
+
   SigmaL = (dcomplex*)malloc(sizeof(dcomplex)*NUM_c* NUM_c);
   SigmaR = (dcomplex*)malloc(sizeof(dcomplex)*NUM_c* NUM_c);
-  v1 = (dcomplex*)malloc(sizeof(dcomplex)*NUM_c* NUM_c);
+
+  SigmaL_Ad = (dcomplex*)malloc(sizeof(dcomplex)*NUM_c* NUM_c);
+  SigmaR_Ad = (dcomplex*)malloc(sizeof(dcomplex)*NUM_c* NUM_c);
+
+  work1 = (dcomplex*)malloc(sizeof(dcomplex)*NUM_c* NUM_c);
+  work2 = (dcomplex*)malloc(sizeof(dcomplex)*NUM_c* NUM_c);
+
   Gless = (dcomplex*)malloc(sizeof(dcomplex)*NUM_c* NUM_c); 
-  GCLorR = (dcomplex*)malloc(sizeof(dcomplex)*NUM_c* NUM_c); 
-  
+
   if (2<=level_stdout){
     printf("NUM_c=%d, NUM_e= %d %d\n",NUM_c, NUM_e[0], NUM_e[1]);
     printf("# of freq. to calculate G =%d\n",tran_omega_n_scf);
   }
 
-  /*parallel global iw 0:tran_omega_n_scf-1 */
-  /*parallel local  Miw 0:Miwmax-1 */
-  /*parllel variable iw=iwIdx[myid][Miw] */
+  if      (SpinP_switch==0) spinsize = 1;
+  else if (SpinP_switch==1) spinsize = 2;
+  else if (SpinP_switch==3) spinsize = 1;
 
-  for (Miw=0; Miw<Miwmax; Miw++) {
+  /**************************************************************
+             calculation of Green functions at k and iw
+  **************************************************************/
 
-    iw = iwIdx[myid][Miw];
+  for ( Miw=myid; Miw<tran_omega_n_scf*spinsize; Miw+=numprocs ) {
 
-    if (iw>=0) {
-      w = tran_omega_scf[iw];
-      w_weight  = tran_omega_weight_scf[iw];
-      iw_method = tran_integ_method_scf[iw]; 
-    }
+    k = Miw/tran_omega_n_scf; 
+    iw = Miw - k*tran_omega_n_scf;
+
+    w = tran_omega_scf[iw];
+    w_weight  = tran_omega_weight_scf[iw];
+    iw_method = tran_integ_method_scf[iw]; 
     
     /*
     printf("Miwmax=%3d Miw=%3d iw=%d of %d w=%le %le weight=%le %le method=%d\n",
             Miwmax,Miw,iw,tran_omega_n_scf, w.r,w.i, w_weight.r, w_weight.i, iw_method);
     */
 
-    for (k=0; k<=SpinP_switch; k++) {
+    /* calculation of surface Green's function and self energy from the LEFT lead */
 
-      if (iw>=0) {
+    iside = 0;
 
-        iside=0;
+    TRAN_Calc_SurfGreen_direct(w, NUM_e[iside], H00_e[iside][k], H01_e[iside][k],
+			       S00_e[iside], S01_e[iside], tran_surfgreen_iteration_max,
+			       tran_surfgreen_eps, GRL);
 
-        TRAN_Calc_SurfGreen_direct(w,NUM_e[iside], H00_e[iside][k],H01_e[iside][k],
-				   S00_e[iside], S01_e[iside], tran_surfgreen_iteration_max,
-                                   tran_surfgreen_eps, GRL);
+    TRAN_Calc_SelfEnergy(w, NUM_e[iside], GRL, NUM_c, HCL[k], SCL, SigmaL);
 
-        TRAN_Calc_SelfEnergy(w, NUM_e[iside], GRL, NUM_c, HCL[k], SCL, SigmaL);
+    /* calculation of surface Green's function and self energy from the RIGHT lead */
 
-        iside=1;
+    iside = 1;
 
-        TRAN_Calc_SurfGreen_direct(w,NUM_e[iside], H00_e[iside][k],H01_e[iside][k],
-				   S00_e[iside], S01_e[iside], tran_surfgreen_iteration_max,
-                                   tran_surfgreen_eps, GRR);
+    TRAN_Calc_SurfGreen_direct(w, NUM_e[iside], H00_e[iside][k], H01_e[iside][k],
+			       S00_e[iside], S01_e[iside], tran_surfgreen_iteration_max,
+			       tran_surfgreen_eps, GRR);
 
-        TRAN_Calc_SelfEnergy(w, NUM_e[iside], GRR, NUM_c, HCR[k], SCR, SigmaR);
+    TRAN_Calc_SelfEnergy(w, NUM_e[iside], GRR, NUM_c, HCR[k], SCR, SigmaR);
 
-        TRAN_Calc_CentGreen(w, NUM_c, SigmaL,SigmaR, HCC[k], SCC, GC);
+    /* calculation of central retarded Green's function */
 
-        /***********************************************
-                             G_{C} 
-        ***********************************************/
+    TRAN_Calc_CentGreen(w, NUM_c, SigmaL, SigmaR, HCC[k], SCC, GC);
 
-	if (iw_method==1) {
+    /***********************************************
+       The non-equilibrium part is calculated by 
+        using the lesser Green's function.
+    ***********************************************/
 
-	  /* start debug */
+    if (TRAN_Kspace_grid2==1 && TRAN_Kspace_grid3==1 && iw_method==2){
 
-          if (WRITE_DENSITY){
-  	    for (i=0;i<NUM_c;i++) {
-	      /* imag( GC * weight ) */
-	      Density[k][i] += GC_ref(i+1,i+1).r*w_weight.i + GC_ref(i+1,i+1).i*w_weight.r;
-	    }
-	  }
+      for (i=0; i<NUM_c*NUM_c; i++) {
 
-	  /* end debug */
+	SigmaL_Ad[i].r = SigmaL[i].r;
+	SigmaL_Ad[i].i =-SigmaL[i].i;
 
-	}  /* iw_method */
+	SigmaR_Ad[i].r = SigmaR[i].r;
+	SigmaR_Ad[i].i =-SigmaR[i].i;
 
-        /***********************************************
-              based on the lesser Green's function
-        ***********************************************/
+	GC_Ad[i].r = GC[i].r;  
+	GC_Ad[i].i =-GC[i].i;  
+      }    
 
-        else if (iw_method==2)  {
+      TRAN_Calc_CentGreenLesser( w, ChemP_e, NUM_c, 
+				 Order_Lead_Side,  
+				 SigmaL, SigmaL_Ad, 
+				 SigmaR, SigmaR_Ad, 
+				 GC, GC_Ad,        
+				 HCC[k], SCC,
+				 work1, work2, Gless);
+    }
 
-          if (2<=level_stdout){
-            printf("G_Lesser, iw=%d (of %d) w=%le %le\n",iw,tran_omega_n_scf,w.r, w.i);
-	  }
+    /* in case of TRAN_Kspace_grid2!=1 || TRAN_Kspace_grid3!=1 */ 
+    /* calculation of advanced Green's function */ 
+         
+    else if (iw_method==2){
 
-          TRAN_Calc_CentGreenLesser(w, ChemP_e, NUM_c, SigmaL,SigmaR,GC,v1, Gless);
+      /* conjugate complex */
+      w.i = -w.i;
 
-#ifdef DEBUG
-	  TRAN_Print2_dcomplex("Gless",NUM_c,NUM_c,Gless);
-	  printf("exit after TRAN_Print2_dcomplex Gless\n");
-	  exit(0);
-#endif
+      /* calculation of surface Green's function and self energy from the LEFT lead */
 
-          if (WRITE_DENSITY){
-	    /* real( Gless * w_weight )  */
-	    for (i=0;i<NUM_c;i++) {
-	      Density[k][i] += Gless_ref(i+1,i+1).r*w_weight.r - Gless_ref(i+1,i+1).i*w_weight.i;
-	    }
+      iside = 0;
 
-	    /*        printf("iw=%d w=%lf %lf weight=%lf %lf GC=%lf %lf, val=%lf\n",
-	     *          iw,w.r , w.i, w_weight.r, w_weight.i, Density[k][0]);
-	     */
-	    /*end debug */
-	  }
-	}
+      TRAN_Calc_SurfGreen_direct(w, NUM_e[iside], H00_e[iside][k], H01_e[iside][k],
+				 S00_e[iside], S01_e[iside], tran_surfgreen_iteration_max,
+				 tran_surfgreen_eps, GRL);
 
-        /***********************************************
-              G_{C_L} in the non-equilibrium case
-                  based on the GaussHG method
-        ***********************************************/
+      TRAN_Calc_SelfEnergy(w, NUM_e[iside], GRL, NUM_c, HCL[k], SCL, SigmaL_Ad);
 
-        else if (iw_method==3){
-	  TRAN_Calc_GC_LorR(iw_method, w, ChemP_e, NUM_c, NUM_e, SigmaL, GC, HCC[k], SCC, v1, GCLorR);
-        }
+      /* calculation of surface Green's function and self energy from the RIGHT lead */
 
-        /***********************************************
-              G_{C_R} in the non-equilibrium case
-                  based on the GaussHG method
-        ***********************************************/
+      iside = 1;
 
-        else if (iw_method==4){
-	  TRAN_Calc_GC_LorR(iw_method, w, ChemP_e, NUM_c, NUM_e, SigmaR, GC, HCC[k], SCC, v1, GCLorR);
-        }
+      TRAN_Calc_SurfGreen_direct(w, NUM_e[iside], H00_e[iside][k], H01_e[iside][k],
+				 S00_e[iside], S01_e[iside], tran_surfgreen_iteration_max,
+				 tran_surfgreen_eps, GRR);
 
-	else {
-	  printf("error, iw_method=%d",iw_method);
-	  exit(10);
-	}
+      TRAN_Calc_SelfEnergy(w, NUM_e[iside], GRR, NUM_c, HCR[k], SCR, SigmaR_Ad);
 
-        /***********************************************
+      /* calculation of central advanced Green's function */
+
+      TRAN_Calc_CentGreen(w, NUM_c, SigmaL_Ad, SigmaR_Ad, HCC[k], SCC, GC_Ad);
+
+      /* conjugate complex */
+      w.i = -w.i;
+
+      /* calculation of central lesser Green's function */
+
+      TRAN_Calc_CentGreenLesser( w, ChemP_e, NUM_c, 
+				 Order_Lead_Side, 
+				 SigmaL, SigmaL_Ad, 
+				 SigmaR, SigmaR_Ad, 
+				 GC, GC_Ad, 
+				 HCC[k], SCC,
+				 work1, work2, Gless);
+    }
+
+    /***********************************************
             add it to construct the density matrix
-        ***********************************************/
+    ***********************************************/
 
-        if      (iw_method==1) {
-          TRAN_Add_MAT( 1, NUM_c, w_weight, GC,     v2[k]);
-        }
-        else if (iw_method==2) {
-          TRAN_Add_MAT( 2, NUM_c, w_weight, Gless,  v2[k]);
-        }
-        else if (iw_method==3) {
-          TRAN_Add_MAT( 1, NUM_c, w_weight, GCLorR, v2[k]);
-        }
-        else if (iw_method==4) {
-          TRAN_Add_MAT( 1, NUM_c, w_weight, GCLorR, v2[k]);
-        }
+    if (iw_method==1)
+      TRAN_Add_MAT( 1, NUM_c, w_weight, GC,     v2[k]);
+    else if (iw_method==2)
+      TRAN_Add_MAT( 1, NUM_c, w_weight, Gless,  v2[k]);
 
-      }  /* iw>=0 */
-    } /* for k */
   } /* iw */
 
   if (MEASURE_TIME){
     dtime(&time_a1);
   }
 
-  free(GCLorR);
-  free(Gless);
-  free(v1);
-  free(SigmaR);
-  free(SigmaL);
-  free(GRR);
-  free(GRL);
-  free(GC);
-
   /***********************************************
           calculation of density matrix
   ***********************************************/
@@ -996,8 +1487,8 @@ static void TRAN_DFT_Kdependent(
 	tnoB = Spe_Total_CNO[wanB];
 	Bnum = MP[GB_AN];
 
-	for (i=0;i<tnoA;i++) {
-	  for (j=0;j<tnoB;j++) {
+	for (i=0; i<tnoA; i++) {
+	  for (j=0; j<tnoB; j++) {
 	    size_v3++;
 	    size_v3++;
 	  }
@@ -1012,7 +1503,7 @@ static void TRAN_DFT_Kdependent(
 
     /* set up v3 */
 
-#define v_idx(i,j)   ( ((j)-1)*NUM_c + (i)-1 ) 
+#define v_idx(i,j)  ( ((j)-1)*NUM_c + (i)-1 ) 
 
     for (k=0; k<=SpinP_switch; k++) {
 
@@ -1031,8 +1522,8 @@ static void TRAN_DFT_Kdependent(
 	  tnoB = Spe_Total_CNO[wanB];
 	  Bnum = MP[GB_AN];
 
-	  for (i=0;i<tnoA;i++) {
-	    for (j=0;j<tnoB;j++) {
+	  for (i=0; i<tnoA; i++) {
+	    for (j=0; j<tnoB; j++) {
 	      my_v3[itot++] = v2[k][ v_idx( Anum+i, Bnum+j) ].r;
 	      my_v3[itot++] = v2[k][ v_idx( Anum+i, Bnum+j) ].i;
 	    }
@@ -1075,11 +1566,14 @@ static void TRAN_DFT_Kdependent(
 	  si = (double)k_op*sin(2.0*PI*kRn);
 	  co = (double)k_op*cos(2.0*PI*kRn);
 
-	  for (i=0;i<tnoA;i++) {
-	    for (j=0;j<tnoB;j++) {
+	  for (i=0; i<tnoA; i++) {
+	    for (j=0; j<tnoB; j++) {
 	      re = v3[itot++];
 	      im = v3[itot++];
+
+              /* divided by numprocs due to the later MPI_Allreduce  */
 	      DM1[k][itot0++] += (re*co + im*si)/(double)numprocs; 
+
 	    }
 	  }
 	}
@@ -1102,25 +1596,24 @@ static void TRAN_DFT_Kdependent(
 
   /* free arrays */
 
+  free(Gless);
+  free(work2);
+  free(work1);
+  free(SigmaL_Ad);
+  free(SigmaR_Ad);
+  free(SigmaR);
+  free(SigmaL);
+  free(GRR);
+  free(GRL);
+  free(GC_Ad);
+  free(GC);
+
   for (k=0; k<=SpinP_switch; k++) {
     free(v2[k]);
   }
   free(v2);
 
-  if (WRITE_DENSITY){
-    for (k=SpinP_switch; k>=0; k--) {
-      free(Density[k]);
-    }
-    free(Density);
-  }
-
   free(MP);
-
-  for (i=0;i<numprocs;i++) {
-    free(iwIdx[i]);
-  }
-  free(iwIdx);
-
 }
 
 
@@ -1138,25 +1631,18 @@ static void TRAN_Add_MAT(
 
   switch (mode) {
   case 0:
-    for (i=0;i<NUM_c*NUM_c;i++) {
+    for (i=0; i<NUM_c*NUM_c; i++) {
       out[i].r =  0.0;
       out[i].i =  0.0;
     }
     break;
   case 1:
-    for (i=0;i<NUM_c*NUM_c;i++) {
+    for (i=0; i<NUM_c*NUM_c; i++) {
       out[i].r += ( v[i].r*w_weight.r - v[i].i*w_weight.i );
       out[i].i += ( v[i].r*w_weight.i + v[i].i*w_weight.r );
     }
     break;
-  case 2:
-    for (i=0;i<NUM_c*NUM_c;i++) {
-      /* real(G^less*weight) */
-      dum = v[i].r*w_weight.r - v[i].i*w_weight.i;
-      out[i].r += dum;
-    }
-    break;
-
   }
+
 }
 
