@@ -420,7 +420,7 @@ void Voronoi_Orbital_Moment()
       dy = y - Gxyz[Gc_AN][2];
       dz = z - Gxyz[Gc_AN][3];
 
-      FuzzyW = Fuzzy_Weight(Gc_AN,0,x,y,z);
+      FuzzyW = Fuzzy_Weight(Gc_AN,Mc_AN,0,x,y,z);
       Orbitals_on_Grid(Cwan,Lmax,Nmul,dx,dy,dz,Chi0,RF,AF);
 
       for (L=0; L<=Lmax; L++){
