@@ -25,14 +25,9 @@ static void Eigen_lapack(double **a, double *ko, int n);
 static void EigenBand_lapack(dcomplex **A, double *W, int N);
 
 static void jx_cluster1(int argc, char *argv[]);
-static void jx_cluster2(int argc, char *argv[]);
 static void calc_J_cluster1(int First_Atom, int Second_Atom, int *MPF,
                             double ***C, double **ko,
                             int F_TNumOrbs, int F_TNumOrbs3);
-static void calc_J_cluster2(int First_Atom, int Second_Atom,
-                            int Fatom_ESM, int Satom_ESM,
-                            int *MPF, double ***C, double **ko,
-                            int F_TNumOrbs, int F_TNumOrbs3);
 
 static void jx_band1(int argc, char *argv[]);
 static void k_inversion(int i,  int j,  int k, 
@@ -907,402 +902,6 @@ void jx_cluster1(int argc, char *argv[])
 }  
 
 
-void jx_cluster2(int argc, char *argv[]) 
-{
-
-  static int ct_AN,h_AN,Gh_AN,i,j,TNO1,TNO2;  
-  static int spin,Rn;
-  static int po1,po2,Fatom_ESM,Satom_ESM;
-  int spin1,spin2;   /* dummy variables for loop    */
-  int ii,jj,ll;      /* dummy variables for loop    */
-  int II,JJ ;        /* dummy variables for loop    */
-  int Anum;          /* temporary variable          */
-  int Number_Choo ;  /* the number of Choosen atom  */
-  int TNumOrbs,TNumOrbs3;  
-  int F_TNumOrbs,F_TNumOrbs3;
-  int SPI,SPJ;
-  int optEV ;        /* variable to be used for option of eigenvector-printing */
-  int *Full_atom ;   /* array to store the glbal number of Full atoms */
-  int *MPF ;         /* Full system version of MP */
-  int First_Atom;    /* the first atom i in Jij  */
-  int Second_Atom;   /* the second atom j in Jij */
-  int end_switch;    /* switch in the evalusion loop of J */
-  double ***FullHks; /* full Hamiltonian */
-  double **FullOLP;  /* full overlap     */
-
-  /* variable & arrays for PART-2; same with that of Cluster_DFT.c */
-  static int l,n,n2,n1,i1,j1,k1,l1;
-  static double **ko, *M1;
-  static double **B, ***C, **D;
-  static double sum,sum1;                                                      
-
-  /************************************************************************* 
-     Calculation flow in the evaluation of J:
-
-     PART-1 : set full Hamiltonian and overlap
-     PART-2 : the generalized eigenvalue problem HC = ESC
-     PART-3 : Calculation of J
-  **************************************************************************/
-
-  /************************************************************************* 
-     PART-1 :  set full Hamiltonian and overlap
-  **************************************************************************/
-
-  printf(" \nEvaluation of J based on cluster calculation\n");
-
-  Full_atom = (int*)malloc(sizeof(int)*atomnum); 
-  MPF = (int*)malloc(sizeof(int)*(atomnum+1)); 
-
-  /* read Full_atom */
-  for (i=0; i<atomnum; i++){
-    Full_atom[i] = i+1;
-  }
-
-  /*********************************************
-   make an array MPF which specifies the starting
-   position of atom II in the martix such as
-   a full but small Hamiltonian
-  *********************************************/
-
-  Anum = 1;
-  for (i=0; i<atomnum; i++){
-    MPF[i+1] = Anum;
-    ct_AN = Full_atom[i];
-    Anum = Anum + Total_NumOrbs[ct_AN];    
-  }
-  F_TNumOrbs = Anum - 1;
-  F_TNumOrbs3 = F_TNumOrbs + 3;  
-
-  /*****************************************
-    allocation of arrays:
-
-    double ***FullHks ;  
-    double **FullOLP ;  
-  ******************************************/
-
-  FullHks = (double***)malloc(sizeof(double**)*(SpinP_switch+1)); 
-  for (spin=0; spin<=SpinP_switch; spin++){
-    FullHks[spin] = (double**)malloc(sizeof(double*)*F_TNumOrbs3); 
-    for (i=0; i<F_TNumOrbs3; i++){
-      FullHks[spin][i] = (double*)malloc(sizeof(double)*F_TNumOrbs3); 
-    }
-  }
-
-  FullOLP = (double**)malloc(sizeof(double*)*F_TNumOrbs3); 
-  for (i=0; i<F_TNumOrbs3; i++){
-    FullOLP[i] = (double*)malloc(sizeof(double)*F_TNumOrbs3); 
-  }
-
-  /*****************************************
-   rearragement of data:
-   Hks to FullHks
-   OLP to FullOLP
-
-   FullHks [1 to F_TNumOrbs] [1 to F_TNumOrbs]
-   FullOLP [1 to F_TNumOrbs] [1 to F_TNumOrbs]
-  ******************************************/
-
-  for (spin=0; spin<=SpinP_switch; spin++){
-    for (i=0; i<F_TNumOrbs3; i++){
-      for (j=0; j<F_TNumOrbs3; j++){
-        FullHks[spin][i][j] = 0.0;
-      }
-    }
-  }
-
-  for (i=0; i<F_TNumOrbs3; i++){
-    for (j=0; j<F_TNumOrbs3; j++){
-      FullOLP[i][j] = 0.0;
-    }
-  }
-
-  for (spin=0; spin<=SpinP_switch; spin++){
-    for (II=0; II<atomnum; II++){
-      SPI = MPF[II+1];  
-      for (JJ=0; JJ<atomnum; JJ++){
-        SPJ = MPF[JJ+1];  
-	ct_AN = Full_atom[II] ;
-	TNO1 = Total_NumOrbs[ct_AN];
-	for (h_AN=0; h_AN<=FNAN[ct_AN]; h_AN++){
-	  Gh_AN = natn[ct_AN][h_AN];
-	  if (Gh_AN == Full_atom[JJ]){
-	    Rn = ncn[ct_AN][h_AN];
-	    TNO2 = Total_NumOrbs[Gh_AN];
-	    for (i=0; i<TNO1; i++){
-              for (j=0; j<TNO2; j++){
-                FullHks[spin][i+SPI][j+SPJ] = Hks[spin][ct_AN][h_AN][i][j]; 
-                FullOLP[i+SPI][j+SPJ] = OLP[ct_AN][h_AN][i][j]; 
-	      }
-            }
-	  }
-	}
-      }
-    }
-  }
-  
-  /************************************************************************* 
-     PART-2 : solve the generalized eigenvalue problem HC = ESC
-  **************************************************************************/
-
-  /*******************************************
-   allocation of arrays:
-
-   double ko[SpinP_switch+1][F_TNumOrbs3];
-   double M1[F_TNumOrbs3];
-   double B[F_TNumOrbs3][F_TNumOrbs3];
-   double C[SpinP_switch+1][F_TNumOrbs3][F_TNumOrbs3];
-   double D[F_TNumOrbs3][F_TNumOrbs3];
-  ********************************************/
-
-  ko = (double**)malloc(sizeof(double*)*(SpinP_switch+1));
-  for (spin=0; spin<=SpinP_switch; spin++){
-    ko[spin] = (double*)malloc(sizeof(double)*F_TNumOrbs3); 
-  }
-  M1 = (double*)malloc(sizeof(double)*F_TNumOrbs3); 
-
-  B = (double**)malloc(sizeof(double*)*F_TNumOrbs3); 
-  for (i=0; i<F_TNumOrbs3; i++){
-    B[i] = (double*)malloc(sizeof(double)*F_TNumOrbs3); 
-  }
-
-  C = (double***)malloc(sizeof(double**)*(SpinP_switch+1)); 
-  for (spin=0; spin<=SpinP_switch; spin++){
-    C[spin] = (double**)malloc(sizeof(double*)*F_TNumOrbs3); 
-    for (i=0; i<F_TNumOrbs3; i++){
-      C[spin][i] = (double*)malloc(sizeof(double)*F_TNumOrbs3); 
-    }
-  }
-
-  D = (double**)malloc(sizeof(double*)*F_TNumOrbs3); 
-  for (i=0; i<F_TNumOrbs3; i++){
-    D[i] = (double*)malloc(sizeof(double)*F_TNumOrbs3); 
-  }
-
-  /*******************************************
-   diagonalize the overlap matrix
-
-   first 
-   FullOLP -> OLP matrix
-   after call Eigen_lapack
-   FullOLP -> eigenvectors of OLP matrix
-  ********************************************/
-
-  printf(" Diagonalize the overlap matrix\n");
-
-  Eigen_lapack(FullOLP,ko[0],F_TNumOrbs);
-
-  for (l=1; l<=F_TNumOrbs; l++){
-    printf("l ko %2d %15.12f\n",l,ko[0][l]);
-  }
-
-  /* check ill-conditioned eigenvalues */
-
-  for (l=1; l<=F_TNumOrbs; l++){
-    if (ko[0][l]<1.0e-14) {
-      printf("Found ill-conditioned eigenvalues\n");
-      printf("Stopped calculation\n");
-      exit(1);
-    }
-  }
-
-  for (l=1; l<=F_TNumOrbs; l++){
-    M1[l] = 1.0/sqrt(ko[0][l]);
-  }
-
-  /****************************************************
-   Calculations of eigenvalues for up and down spins
-  ****************************************************/
-
-  n = F_TNumOrbs;
-
-  for (spin=0; spin<=SpinP_switch; spin++){
-
-    printf(" Diagonalize the Hamiltonian for spin=%2d\n",spin);
-
-    for (i1=1; i1<=n; i1++){
-      for (j1=1; j1<=n; j1++){
-        sum = 0.0;
-        for (l=1; l<=n; l++){
-	  sum = sum + FullHks[spin][i1][l]*FullOLP[l][j1]*M1[j1]; 
-        }
-        C[spin][i1][j1] = sum;
-      }
-    }
-
-    for (i1=1; i1<=n; i1++){
-      for (j1=1; j1<=n; j1++){
-        sum = 0.0;
-        for (l=1; l<=n; l++){
-	  sum = sum + M1[i1]*FullOLP[l][i1]*C[spin][l][j1];
-        }
-        B[i1][j1] = sum;
-      }
-    }
-
-    for (i1=1; i1<=n; i1++){
-      for (j1=1; j1<=n; j1++){
-        D[i1][j1] = B[i1][j1];    
-      }
-    }
-
-    Eigen_lapack(D,ko[spin],n);
-
-    /****************************************************
-        Transformation to the original eigen vectors.
-                          NOTE 244P
-    ****************************************************/
-
-    for (i1=1; i1<=n; i1++){
-      for (j1=1; j1<=n; j1++){
-        C[spin][i1][j1] = 0.0;
-      }
-    }
-
-    for (i1=1; i1<=n; i1++){
-      for (j1=1; j1<=n; j1++){
-        sum = 0.0;
-        for (l=1; l<=n; l++){
-          sum = sum + FullOLP[i1][l]*M1[l]*D[l][j1];       
-        }
-        C[spin][i1][j1] = sum;
-      }
-    }
-  } /* spin */
-
-  /* printing out eigenvalues and the eigenvectors */
-
-  if (printout==1){
-    for (spin=0; spin<=SpinP_switch; spin++){
-      printf("\nspin=%i \n",spin);                                       
-      for (i=1; i<=F_TNumOrbs; i++){
-	printf("%ith eigenvalue of HC=eSC: %15.12f\n",i,ko[spin][i]);
-      }
-    }
-    printf("\nDo you want eigenvectors also? (yes:1 / no:0)");
-    scanf("%i",&optEV);
-    if (optEV == 1){
-      for (spin=0; spin<=SpinP_switch; spin++){
-	printf("\nspin=%i \n",spin);                                     
-	for (i=1; i<=F_TNumOrbs; i++){
-	  printf("%ith eigenvector: ",i);
-	  printf("{");
-	  for (j=1; j<=F_TNumOrbs; j++){
-	    printf("%15.12f,",C[spin][i][j]);
-	  }
-	  printf("}\n");
-	}
-      }
-    }
-  }
-
-  /************************************************************************* 
-
-     PART-3 : Calculation of J
-     
-     1) V_i,alpha,beta = < i,alpha | V_i | j,beta >
-                       = 0.5 * ( H_i,alpha,i,beta(0) - H_i,alpha,i,beta(1) )
-
-     2) V'_i = SUM_alpha,beta{ C_i,alpha * V_alpha,beta * C_i,beta }
-
-     3) calculation of J from V'
-  **************************************************************************/
-
-  printf("\n");
-
-  end_switch = 0;
-  do {
-
-    /* specify two atoms */ 
-    printf(" Specify two atoms (e.g 1 2, quit: 0 0)  ");
-    scanf("%d %d",&First_Atom,&Second_Atom);
-
-    if      (First_Atom==0 && Second_Atom==0) end_switch = 1;       
-    else if (First_Atom==0 && Second_Atom!=0 || 
-             First_Atom!=0 && Second_Atom==0){
-      printf(" Invalid atom\n");
-    }
-    else if (atomnum<First_Atom || atomnum<Second_Atom){
-      printf(" Invalid atom\n");
-    }
-    else{
-
-      po1 = 0;
-      for (i=0; i<Num_ESM; i++){
-        if (ESM_Atoms[i]==First_Atom)  { po1 = 1; Fatom_ESM = i; }
-      }
-
-      po2 = 0;
-      for (i=0; i<Num_ESM; i++){
-        if (ESM_Atoms[i]==Second_Atom) { po2 = 1; Satom_ESM = i; }
-      }
-
-      if (po1*po2==1){
-        /* calculation of J */ 
-        calc_J_cluster2(First_Atom, Second_Atom, Fatom_ESM, Satom_ESM,
-                        MPF, C, ko, F_TNumOrbs, F_TNumOrbs3);
-      }
-      else{
-        printf(" no exchange splitting matrix for selected atom\n");
-      }
-    } 
-
-  } while(end_switch==0);
-
-  /*********************************************
-    freeing of arrays:
-
-    int *Full_atom;
-    int *MPF;
-    double ***FullHks ;  
-    double **FullOLP ;  
-    double **ko;
-    double *M1;
-    double **B;
-    double ***C;
-    double **D;
-  *********************************************/
-
-  free(Full_atom);
-  free(MPF);
-
-  for (spin=0; spin<=SpinP_switch; spin++){
-    for (i=0; i<F_TNumOrbs3; i++){
-      free(FullHks[spin][i]);
-    }
-    free(FullHks[spin]);
-  }
-  free(FullHks);
-
-  for (i=0; i<F_TNumOrbs3; i++){
-    free(FullOLP[i]);
-  }
-  free(FullOLP);
-
-  for (spin=0; spin<=SpinP_switch; spin++){
-    free(ko[spin]);
-  }
-  free(ko);
-
-  free(M1);
-
-  for (i=0; i<F_TNumOrbs3; i++){
-    free(B[i]);
-  }
-  free(B);
-
-  for (spin=0; spin<=SpinP_switch; spin++){
-    for (i=0; i<F_TNumOrbs3; i++){
-      free(C[spin][i]);
-    }
-    free(C[spin]);
-  }
-  free(C);
-
-  for (i=0; i<F_TNumOrbs3; i++){
-    free(D[i]);
-  }
-  free(D);
-}  
 
 
 
@@ -2139,237 +1738,6 @@ void calc_J_cluster1(int First_Atom, int Second_Atom, int *MPF, double ***C, dou
 
 
 
-void calc_J_cluster2(int First_Atom, int Second_Atom,
-                     int Fatom_ESM, int Satom_ESM,
-                     int *MPF, double ***C, double **ko,
-                     int F_TNumOrbs, int F_TNumOrbs3)
-{
-  static int ct_AN,h_AN,Gh_AN,i,j,TNO1,TNO2,q_AN,Gq_AN;  
-  static int spin;
-  static int spin1,spin2;   /* dummy variables for loop                               */
-  static int ii,jj,ll;      /* dummy variables for loop                               */
-  static int II,JJ ;        /* dummy variables for loop                               */
-  static int Number_Choo ;  /* the number of Choosen atom                             */
-  static int *MP ;          /* array which specify a head position of a full matrix   */
-  static int *Choo_atom ;   /* array to store the glbal number of choosen atoms       */
-  static int Anum;          /* temporary variable                                     */
-  static int TNumOrbs,TNumOrbs3;  
-  static int SPI,SPJ;
-  static int NOJ,NOI;
-
-  static double sum,sum1;                                                      
-  static double ****VVi, ****VVj ;   /* array for V_i,V_j and V' */
-  static double **Fftn;        /* Fermi function */
-  static double J_ij;          /* final exchange-interaction value  */
-  static double kB=0.000003166813628;  /* Boltzman constant (Hatree/K) */
-  static double dFftn;
-  static double dko;
-
-  /*********************************************
-    allocation of arrays:
-
-    int Choo_atom[Number_Choo];
-    int MP[Number_Choo];
-  *********************************************/
-
-  Number_Choo = 2;
-
-  Choo_atom = (int*)malloc(sizeof(int)*Number_Choo); 
-  MP = (int*)malloc(sizeof(int)*Number_Choo); 
-
-  Choo_atom[0] = First_Atom;
-  Choo_atom[1] = Second_Atom;
-
-  /*********************************************
-   make an array MP which specify the starting
-   position of atom II in the martix such as
-   a full but small Hamiltonian
-  *********************************************/
-
-  Anum = 1;
-  for (i=0; i<Number_Choo; i++){
-    MP[i] = Anum;
-    ct_AN = Choo_atom[i];
-    Anum = Anum + Total_NumOrbs[ct_AN];    
-  }
-  TNumOrbs = Anum - 1;
-  TNumOrbs3 = TNumOrbs + 3;  
-
-  II = Choo_atom[0];
-  NOI = Total_NumOrbs[II] ; /* this gives orbital number of i-atom (or 1st atom) */
-  JJ = Choo_atom[1];
-  NOJ = Total_NumOrbs[JJ] ; /* this gives orbital number of j-atom (or 2nd atom) */
-
-  /*********************************************
-           calculation of VVi and VVj
-  *********************************************/
-
-  /* memory allocation */
-  VVi = (double****)malloc(sizeof(double***)*(SpinP_switch+1)); 
-  for (spin1=0; spin1<=SpinP_switch; spin1++){
-    VVi[spin1] = (double***)malloc(sizeof(double**)*(SpinP_switch+1)); 
-    for (spin2=0; spin2<=1; spin2++){
-      VVi[spin1][spin2] = (double**)malloc(sizeof(double*)*(F_TNumOrbs3)); 
-      for (i=0; i<F_TNumOrbs3; i++){
-	VVi[spin1][spin2][i] = (double*)malloc(sizeof(double)*(F_TNumOrbs3)); 
-      }
-    }
-  }
-
-  VVj = (double****)malloc(sizeof(double***)*(SpinP_switch+1)); 
-  for (spin1=0; spin1<=SpinP_switch; spin1++){
-    VVj[spin1] = (double***)malloc(sizeof(double**)*(SpinP_switch+1)); 
-    for (spin2=0; spin2<=SpinP_switch; spin2++){
-      VVj[spin1][spin2] = (double**)malloc(sizeof(double*)*(F_TNumOrbs3)); 
-      for (i=0; i<(F_TNumOrbs3); i++){
-	VVj[spin1][spin2][i] = (double*)malloc(sizeof(double)*(F_TNumOrbs3)); 
-      }
-    }
-  }
-
-
-  /* calculation VVi */    
-  for(spin1=0; spin1<=SpinP_switch; spin1++){
-    for (spin2=0; spin2<=SpinP_switch; spin2++){
-      for(i=1; i<=F_TNumOrbs; i++){
-	for(j=1; j<=F_TNumOrbs; j++){
-
-	  sum1 = 0.0;
-          for (h_AN=0; h_AN<=FNAN[First_Atom]; h_AN++){
-            Gh_AN = natn[First_Atom][h_AN];
-  	    TNO1 = Total_NumOrbs[Gh_AN];
-            for (q_AN=0; q_AN<=FNAN[First_Atom]; q_AN++){
-              Gq_AN = natn[First_Atom][q_AN];
-      	      TNO2 = Total_NumOrbs[Gq_AN];
-              for (ii=0; ii<TNO1; ii++){
-                for (jj=0; jj<TNO2; jj++){
-                  sum1 += C[spin1][ MPF[First_Atom]+ii ][i]*
-                          ESM[Fatom_ESM][h_AN][q_AN][ii][jj]*
-                          C[spin2][ MPF[First_Atom]+jj ][j];
-		}
-	      }           
-            }  
-          }  
-
-	  VVi[spin1][spin2][i][j] = sum1 ;
-	}
-      }
-    }
-  }
-
-  /* calculation VVj */    
-  for(spin1=0; spin1<=SpinP_switch; spin1++){
-    for (spin2=0; spin2<=SpinP_switch; spin2++){
-      for(i=1; i<=F_TNumOrbs; i++){
-	for(j=1; j<=F_TNumOrbs; j++){
-
-	  sum1 = 0.0;
-          for (h_AN=0; h_AN<=FNAN[Second_Atom]; h_AN++){
-            Gh_AN = natn[Second_Atom][h_AN];
-  	    TNO1 = Total_NumOrbs[Gh_AN];
-            for (q_AN=0; q_AN<=FNAN[Second_Atom]; q_AN++){
-              Gq_AN = natn[Second_Atom][q_AN];
-      	      TNO2 = Total_NumOrbs[Gq_AN];
-              for (ii=0; ii<TNO1; ii++){
-                for (jj=0; jj<TNO2; jj++){
-                  sum1 += C[spin1][ MPF[Second_Atom]+ii ][i]*
-                          ESM[Satom_ESM][h_AN][q_AN][ii][jj]*
-                          C[spin2][ MPF[Second_Atom]+jj ][j];
-		}
-	      }           
-            }  
-          }  
-
-	  VVj[spin1][spin2][i][j] = sum1 ;
-	}
-      }
-    }
-  }
-
-  /*********************************************
-              calculation of J_ij
-  *********************************************/
-    
-  /* allocation of memory */
-  Fftn = (double**)malloc(sizeof(double*)*(SpinP_switch+1)); 
-  for (spin=0; spin<=SpinP_switch; spin++){
-    Fftn[spin] = (double*)malloc(sizeof(double)*F_TNumOrbs3); 
-  }
-
-  /* the Fftn-array for Fermi function */
-  for (spin=0; spin<=SpinP_switch; spin++){
-    for (i=1; i<=F_TNumOrbs; i++){
-      Fftn[spin][i] = 1.0/( exp( (ko[spin][i]- ChemP)/(kB*E_Temp) ) + 1.0 )  ;
-    }
-  }
-
-  /* J_ij calculation from VVi & VVj */
-
-  printf("F_TNumOrbs=%2d\n",F_TNumOrbs);
-  for (i=1; i<=F_TNumOrbs; i++){    
-    printf("i=%2d ko0 ko1 %15.12f %15.12f\n",i,ko[0][i],ko[1][i]);
-  }
-
-
-  J_ij = 0.0; 
-  for (i=1; i<=F_TNumOrbs; i++){    
-    for (j=1; j<=F_TNumOrbs; j++){    
-      dFftn = Fftn[0][i] - Fftn[1][j] ;
-      dko = ko[1][j] - ko[0][i] ;
-      J_ij = J_ij + 0.125*dFftn * VVj[0][1][i][j] * VVi[1][0][j][i] / dko ;
-    }
-  }
-
-  /* unit conversion: Hartree to cm^{-1} */
-
-  printf(" J_ij between %ith atom and %ith atom is %15.12f Hartree\n",
-          Choo_atom[0], Choo_atom[1], J_ij);
-
-  J_ij = 2.194746*100000.0*J_ij;
-
-  /* printing the final result : J_ij*/
-
-  printf(" J_ij between %ith atom and %ith atom is %15.12f cm^{-1}\n",
-          Choo_atom[0], Choo_atom[1], J_ij);
-
-  /*********************************************
-    freeing of arrays:
-
-    double ****VVi;
-    double ****VVj;
-    double **Fftn;
-  *********************************************/
-
-  free(MP); 
-  free(Choo_atom);
-
-  for (spin1=0; spin1<=SpinP_switch; spin1++){
-    for (spin2=0; spin2<=1; spin2++){
-      for (i=0; i<F_TNumOrbs3; i++){
-        free(VVi[spin1][spin2][i]);
-      }
-      free(VVi[spin1][spin2]);
-    }
-    free(VVi[spin1]);
-  }
-  free(VVi);
-
-  for (spin1=0; spin1<=SpinP_switch; spin1++){
-    for (spin2=0; spin2<=SpinP_switch; spin2++){
-      for (i=0; i<(F_TNumOrbs3); i++){
-	free(VVj[spin1][spin2][i]);
-      }
-      free(VVj[spin1][spin2]);
-    }
-    free(VVj[spin1]);
-  }
-  free(VVj);
-
-  for (spin=0; spin<=SpinP_switch; spin++){
-    free(Fftn[spin]);
-  }
-  free(Fftn);
-}    
 
 
 
