@@ -1093,6 +1093,9 @@ void Overlap_k1k2(int diag_flag,
   double tmp2r,tmp2i;
   double tmp3r,tmp3i;
   double si,co,kRn,k1da,k1db,k1dc;
+  double si1,co1,si2,co2;
+  double tmp2r1,tmp2i1;
+  double tmp2r2,tmp2i2;
   double dx,dy,dz;
   double dkx,dky,dkz;
   double dka,dkb,dkc;
@@ -1486,30 +1489,50 @@ void Overlap_k1k2(int diag_flag,
 	      l2 = atv_ijk[Rnh][2];
 	      l3 = atv_ijk[Rnh][3];
 
+              /*  exp[i(k2 dot Rn - dk dot taui ) ]  */
+
   	      kRn = 2.0*PI*(k2[1]*(double)l1 + k2[2]*(double)l2 + k2[3]*(double)l3);
               kRn -= (dkx*Gxyz[ct_AN][1] + dky*Gxyz[ct_AN][2] + dkz*Gxyz[ct_AN][3]);
 
-	      si = sin(kRn);  /* Imaginary part */
-              co = cos(kRn);  /* Real part      */
+	      si1 = sin(kRn);  /* Imaginary part */
+              co1 = cos(kRn);  /* Real part      */
+
+              /*  exp[i(-k1 dot Rn - dk dot taui ) ]  */
+
+  	      kRn = -2.0*PI*(k1[1]*(double)l1 + k1[2]*(double)l2 + k1[3]*(double)l3);
+              kRn -= (dkx*Gxyz[ct_AN][1] + dky*Gxyz[ct_AN][2] + dkz*Gxyz[ct_AN][3]);
+
+	      si2 = sin(kRn);  /* Imaginary part */
+              co2 = cos(kRn);  /* Real part      */
+
 
 	      for (i1=0; i1<tnoA; i1++){
 		for (j1=0; j1<tnoB; j1++){
 
                   tmp1r = Wk1[spin][mu1][Anum+i1].r*Wk2[spin][mu2][Bnum+j1].r
-		    + Wk1[spin][mu1][Anum+i1].i*Wk2[spin][mu2][Bnum+j1].i;
+		        + Wk1[spin][mu1][Anum+i1].i*Wk2[spin][mu2][Bnum+j1].i;
                   tmp1i = Wk1[spin][mu1][Anum+i1].r*Wk2[spin][mu2][Bnum+j1].i
-		    - Wk1[spin][mu1][Anum+i1].i*Wk2[spin][mu2][Bnum+j1].r;
+		        - Wk1[spin][mu1][Anum+i1].i*Wk2[spin][mu2][Bnum+j1].r;
 
-                  tmp2r = co*tmp1r - si*tmp1i;
-                  tmp2i = co*tmp1i + si*tmp1r;
+                  tmp2r1 = co1*tmp1r - si1*tmp1i;
+                  tmp2i1 = co1*tmp1i + si1*tmp1r;
+
+                  tmp1r = Wk1[spin][mu1][Bnum+j1].r*Wk2[spin][mu2][Anum+i1].r
+		        + Wk1[spin][mu1][Bnum+j1].i*Wk2[spin][mu2][Anum+i1].i;
+                  tmp1i = Wk1[spin][mu1][Bnum+j1].r*Wk2[spin][mu2][Anum+i1].i
+		        - Wk1[spin][mu1][Bnum+j1].i*Wk2[spin][mu2][Anum+i1].r;
+
+                  tmp2r2 = co2*tmp1r - si2*tmp1i;
+                  tmp2i2 = co2*tmp1i + si2*tmp1r;
  
                   tmp3r = OLP[ct_AN][h_AN][i1][j1];
                   tmp3i =-dkx*OLPpox[ct_AN][h_AN][i1][j1]
-		    -dky*OLPpoy[ct_AN][h_AN][i1][j1]
-		    -dkz*OLPpoz[ct_AN][h_AN][i1][j1];
+		         -dky*OLPpoy[ct_AN][h_AN][i1][j1]
+		         -dkz*OLPpoz[ct_AN][h_AN][i1][j1];
+
+                  sumr += 0.5*(tmp2r1+tmp2r2)*tmp3r - 0.5*(tmp2i1+tmp2i2)*tmp3i;
+                  sumi += 0.5*(tmp2r1+tmp2r2)*tmp3i + 0.5*(tmp2i1+tmp2i2)*tmp3r;
 
-                  sumr += tmp2r*tmp3r - tmp2i*tmp3i;
-                  sumi += tmp2r*tmp3i + tmp2i*tmp3r;
 		}
 	      }
 	      
@@ -1558,35 +1581,59 @@ void Overlap_k1k2(int diag_flag,
 	    l2 = atv_ijk[Rnh][2];
 	    l3 = atv_ijk[Rnh][3];
 
+            /*  exp[i(k2 dot Rn - dk dot taui ) ]  */
+
   	    kRn = 2.0*PI*(k2[1]*(double)l1 + k2[2]*(double)l2 + k2[3]*(double)l3);
             kRn -= (dkx*Gxyz[ct_AN][1] + dky*Gxyz[ct_AN][2] + dkz*Gxyz[ct_AN][3]);
 
-	    si = sin(kRn);  /* Imaginary part */
-            co = cos(kRn);  /* Real part      */
+	    si1 = sin(kRn);  /* Imaginary part */
+            co1 = cos(kRn);  /* Real part      */
+
+	    /*  exp[i(-k1 dot Rn - dk dot taui ) ]  */
+
+	    kRn = -2.0*PI*(k1[1]*(double)l1 + k1[2]*(double)l2 + k1[3]*(double)l3);
+	    kRn -= (dkx*Gxyz[ct_AN][1] + dky*Gxyz[ct_AN][2] + dkz*Gxyz[ct_AN][3]);
+
+	    si2 = sin(kRn);  /* Imaginary part */
+	    co2 = cos(kRn);  /* Real part      */
 
 	    for (i1=0; i1<tnoA; i1++){
 	      for (j1=0; j1<tnoB; j1++){
 
 		tmp1r = Wk1[0][mu1][Anum      +i1].r*Wk2[0][mu2][Bnum      +j1].r
-		  + Wk1[0][mu1][Anum      +i1].i*Wk2[0][mu2][Bnum      +j1].i
-		  + Wk1[0][mu1][Anum+fsize+i1].r*Wk2[0][mu2][Bnum+fsize+j1].r
-		  + Wk1[0][mu1][Anum+fsize+i1].i*Wk2[0][mu2][Bnum+fsize+j1].i;
+		      + Wk1[0][mu1][Anum      +i1].i*Wk2[0][mu2][Bnum      +j1].i
+		      + Wk1[0][mu1][Anum+fsize+i1].r*Wk2[0][mu2][Bnum+fsize+j1].r
+		      + Wk1[0][mu1][Anum+fsize+i1].i*Wk2[0][mu2][Bnum+fsize+j1].i;
 
 		tmp1i = Wk1[0][mu1][Anum      +i1].r*Wk2[0][mu2][Bnum      +j1].i
-		  - Wk1[0][mu1][Anum      +i1].i*Wk2[0][mu2][Bnum      +j1].r
-		  + Wk1[0][mu1][Anum+fsize+i1].r*Wk2[0][mu2][Bnum+fsize+j1].i
-		  - Wk1[0][mu1][Anum+fsize+i1].i*Wk2[0][mu2][Bnum+fsize+j1].r;
+		      - Wk1[0][mu1][Anum      +i1].i*Wk2[0][mu2][Bnum      +j1].r
+		      + Wk1[0][mu1][Anum+fsize+i1].r*Wk2[0][mu2][Bnum+fsize+j1].i
+		      - Wk1[0][mu1][Anum+fsize+i1].i*Wk2[0][mu2][Bnum+fsize+j1].r;
 
-		tmp2r = co*tmp1r - si*tmp1i;
-		tmp2i = co*tmp1i + si*tmp1r;
+		tmp2r1 = co1*tmp1r - si1*tmp1i;
+		tmp2i1 = co1*tmp1i + si1*tmp1r;
+
+		tmp1r = Wk1[0][mu1][Bnum      +j1].r*Wk2[0][mu2][Anum      +i1].r
+		      + Wk1[0][mu1][Bnum      +j1].i*Wk2[0][mu2][Anum      +i1].i
+		      + Wk1[0][mu1][Bnum+fsize+j1].r*Wk2[0][mu2][Anum+fsize+i1].r
+		      + Wk1[0][mu1][Bnum+fsize+j1].i*Wk2[0][mu2][Anum+fsize+i1].i;
+
+		tmp1i = Wk1[0][mu1][Bnum      +j1].r*Wk2[0][mu2][Anum      +i1].i
+		      - Wk1[0][mu1][Bnum      +j1].i*Wk2[0][mu2][Anum      +i1].r
+		      + Wk1[0][mu1][Bnum+fsize+j1].r*Wk2[0][mu2][Anum+fsize+i1].i
+		      - Wk1[0][mu1][Bnum+fsize+j1].i*Wk2[0][mu2][Anum+fsize+i1].r;
+
+		tmp2r2 = co2*tmp1r - si2*tmp1i;
+		tmp2i2 = co2*tmp1i + si2*tmp1r;
  
 		tmp3r = OLP[ct_AN][h_AN][i1][j1];
 		tmp3i =-dkx*OLPpox[ct_AN][h_AN][i1][j1]
-		  -dky*OLPpoy[ct_AN][h_AN][i1][j1]
-		  -dkz*OLPpoz[ct_AN][h_AN][i1][j1];
+		       -dky*OLPpoy[ct_AN][h_AN][i1][j1]
+		       -dkz*OLPpoz[ct_AN][h_AN][i1][j1];
+
+                sumr += 0.5*(tmp2r1+tmp2r2)*tmp3r - 0.5*(tmp2i1+tmp2i2)*tmp3i;
+                sumi += 0.5*(tmp2r1+tmp2r2)*tmp3i + 0.5*(tmp2i1+tmp2i2)*tmp3r;
 
-		sumr += tmp2r*tmp3r - tmp2i*tmp3i;
-		sumi += tmp2r*tmp3i + tmp2i*tmp3r;
 	      }
 	    }
 	      
