@@ -3,7 +3,7 @@
 #define zgemm_ f2c_zgemm
 #define zcopy_ f2c_zcopy
 #endif
-
+      
 #ifndef ___dcomplex_definition___
 typedef struct { double r,i; } dcomplex;
 #define ___dcomplex_definition___ 
@@ -64,11 +64,19 @@ void TRAN_Allocate_Lead_Region( MPI_Comm mpi_comm_level1 );
 /* in TRAN_Allocate.c  */
 void TRAN_Deallocate_Lead_Region();
 
+/* TRAN_Deallocate_Electrode_Grid.c */
+void TRAN_Deallocate_Electrode_Grid(int Ngrid2);
+
+/* TRAN_Deallocate_RestartFile.c */
+void TRAN_Deallocate_RestartFile(char *position);
+
 
 /* TRAN_Apply_Bias2e.c  */
 void  TRAN_Apply_Bias2e(
        MPI_Comm comm1,
+       int side,
        double voltage,  
+       double TRAN_eV2Hartree,
        int SpinP_switch,
        int atomnum,
        int *WhatSpecies,
@@ -84,11 +92,6 @@ void  TRAN_Apply_Bias2e(
         double *dVHart_Grid
 );
 
-/* TRAN_Calc_CentGreen.c  */
-void tran_calc_centgreen(
-   dcomplex *w, int *nc, dcomplex *sigmaL, dcomplex *sigmaR,
-         dcomplex *HCC, dcomplex *SCC, dcomplex *GC);
-
 /* TRAN_Calc_CentGreen.c  */
 void TRAN_Calc_CentGreen(
                       dcomplex w,
@@ -102,46 +105,43 @@ void TRAN_Calc_CentGreen(
 
 /* TRAN_Calc_CentGreenLesser.c  */
 void TRAN_Calc_CentGreenLesser(
+                      /* input */
                       dcomplex w,
                       double ChemP_e[2],
                       int nc, 
+                      int Order_Lead_Side[2],
                       dcomplex *SigmaL,
+                      dcomplex *SigmaL_Ad,
                       dcomplex *SigmaR, 
-                      dcomplex *GC,  
-                      dcomplex *v1,  
-                      dcomplex *Gless  
+                      dcomplex *SigmaR_Ad, 
+                      dcomplex *GC, 
+                      dcomplex *GC_Ad, 
+                      dcomplex *HCCk, 
+                      dcomplex *SCC, 
+
+                      /* work, nc*nc */
+                      dcomplex *v1, 
+                      dcomplex *v2,
+ 
+                      /*  output */ 
+                      dcomplex *Gless 
                       );
 
 
-/* Taisuke Ozaki Copyright (C) */
-/* TRAN_Calc_GC_LorR.c  */
-void TRAN_Calc_GC_LorR(
-		       int iw_method,       /*  input */
-                       dcomplex w,          /*  input */
-                       double ChemP_e[2],   /*  input */
-                       int nc,              /*  input */
-                       int ne[2],           /*  input */
-                       dcomplex *SigmaLorR, /*  input */
-                       dcomplex *GC,        /*  input */
-                       dcomplex *HCC,         /*  input */ 
-                       dcomplex *SCC,         /*  input */ 
-                       dcomplex *v1,        /* work, nc*nc */
-                       dcomplex *GCLorR     /*  output */
-		       );
  
 /* TRAN_Calc_GridBound.c  */
-void TRAN_Calc_GridBound(
-  MPI_Comm mpi_comm_level1,
-  int atomnum,
-  int *WhatSpecies,
-  double *Spe_Atom_Cut1,
-  int Ngrid1, 
-  double *Grid_Origin, 
-  double **Gxyz,
-  double tv[4][4],
-  double gtv[4][4],
-  double Right_tv[4][4]
-);
+void TRAN_Calc_GridBound(MPI_Comm mpi_comm_level1,
+			 int atomnum,
+			 int *WhatSpecies,
+			 double *Spe_Atom_Cut1,
+			 int Ngrid1, 
+			 double *Grid_Origin, 
+			 double **Gxyz,
+			 double tv[4][4],
+			 double gtv[4][4],
+			 double rgtv[4][4],
+			 double Left_tv[4][4],
+			 double Right_tv[4][4]);
 
 /* TRAN_Calc_OneTransmission.c  */
 void TRAN_Calc_OneTransmission(
@@ -173,17 +173,6 @@ void TRAN_Calc_OneTransmission2(
 				);
 
 
-/* TRAN_Calc_SelfEnergy.c  */
-void tran_calc_selfenergy(
-            dcomplex *w,
-            int *ne,
-            dcomplex *gr,  
-            int *nc,
-            dcomplex *hce,    
-            dcomplex *sce,    
-            dcomplex *sigma   
-          );
-
 /* TRAN_Calc_SelfEnergy.c  */
 void TRAN_Calc_SelfEnergy( 
             dcomplex w,  
@@ -195,18 +184,22 @@ void TRAN_Calc_SelfEnergy(
             dcomplex *sigma   
           );
 
-/* TRAN_Calc_SurfGreen.c  */
-void tran_calc_surfgreen_direct(
-                                dcomplex *w,
-                                int *n,
-				dcomplex *h00, 
-				dcomplex *h01,
-				dcomplex *s00,
-				dcomplex *s01,
-                                int *iteration_max,
-                                double *eps,
-                                dcomplex *gr  
-                                );
+
+/* TRAN_Calc_Hopping_G.c */
+void TRAN_Calc_Hopping_G(
+                         /* input */
+			 dcomplex w,
+			 int ne,          /* size of electrode */ 
+			 int nc,          /* size of central region */
+			 dcomplex *gs,    /* surface green function of electrode, size=ne*ne */
+			 dcomplex *gc,    /* green function of central region, size=nc*nc    */
+			 dcomplex *hce,   /* e.g., HCL,    size=nc*ne */
+			 dcomplex *sce,   /* e.g., SCL,    size=nc*ne */
+
+                         /* output */
+			 dcomplex *gh     /* e.g., GCL_R , size=nc*ne */
+			 );
+
 
 /* TRAN_Calc_SurfGreen.c  */
 void TRAN_Calc_SurfGreen_direct(
@@ -240,21 +233,6 @@ double TRAN_Calc_Transmission(
 		int *List_YOUSO
 );
 
-/* TRAN_Calc_VHartree_G0.c  */
-void TRAN_Calc_VHartree_G0(dcomplex vl, dcomplex vr,    
-                      double vl_c_r,  double vl_c_i,    
-                      double vr_c_r,  double vr_c_i,     
-                      int ix0, int ixlp1  , double gtv1[4],
-                      double *pot_r, double *pot_i   
-);
-
-/* TRAN_Calc_VHartree_Gnon0.c  */
-void TRAN_Calc_VHartree_Gnon0(
-			      int k2, int k3,   
-			      int ix0, int ixlp1, double gtv[4][4],
-			      double   v1_r, double v1_i, double v2_r, double v2_i, 
-			      dcomplex v1_c, dcomplex v2_c,
-			      double *ReV2, double *ImV2)  ;
 
 /* TRAN_Connect_Read_Density.c  */
 void TRAN_Connect_Read_Density(
@@ -314,6 +292,7 @@ void TRAN_Credit(MPI_Comm comm1);
 /* TRAN_DFT.c  */
 double TRAN_DFT(
                 MPI_Comm comm1,
+                int SucceedReadingDMfile, 
                 int level_stdout,
 		int iter, 
 		int SpinP_switch,
@@ -382,28 +361,6 @@ void TRAN_Distribute_Node_Idx(
    int **Idxlist
  );
 
-/* TRAN_FFT_interpolation3c.c  */
-void zfft1(dcomplex *data,   
-   int n_in, 
-   dcomplex *dataout,   
-   int isign0);
-
-/* TRAN_FFT_interpolation3c.c  */
-void      zfourier1a(dcomplex *in,   
-                     int n_in,       
-		     dcomplex *out,   
-                     int n_out,      
-		     int isign,
-		     double alpha,double beta,double gamma) ;
-
-/* TRAN_FFT_interpolation3c.c  */
-void TRAN_FFT_Dinterpolation3D(
-  MPI_Comm comm1,
-  int n_in[3],
-  double *din, 
-  int n_out[4], 
-  double *dout,  
-  double alpha[3],double beta[3],double gamma[3]  );
 
 /* TRAN_Input_std.c  */
 void TRAN_Input_std(
@@ -412,32 +369,15 @@ void TRAN_Input_std(
   int SpinP_switch,  
   char *filepath,
   double kBvalue,
-  double Electronic_Temperature
+  double TRAN_eV2Hartree,  
+  double Electronic_Temperature,
+  int *output_hks
 );
 
 /* TRAN_Input_std_Atoms.c  */
-void TRAN_Input_std_Atoms(
-   int Solver
-);
+void TRAN_Input_std_Atoms(  MPI_Comm comm1, int Solver );
+
 
-/* TRAN_Interp_ElectrodeDensity_Grid.c  */
-/* *static
- * void    Convert_Cell_tv(double tv[4][4], double pos[4],
- *    double xyz[4]);*/
-/* TRAN_Interp_ElectrodeDensity_Grid.c  */
-void TRAN_Interp_ElectrodeDensity_Grid(
-   MPI_Comm   comm1,
-   double offset[4],   
-   double *Density_Grid_e ,   
-   int Ngrid1_e, int Ngrid2_e, int Ngrid3_e,  
-   double tv_e[4][4],   
-   double gtv_e[4][4],   
-   double *Density_Grid,  
-   int Ngrid1, int Ngrid2, int Ngrid3,
-   int l1boundary[2],  
-   double tv[4][4],  
-   double gtv[4][4]  
-);
 
 /* TRAN_Output_HKS.c  */
 int TRAN_Output_HKS(char *fileHKS);
@@ -445,6 +385,7 @@ int TRAN_Output_HKS(char *fileHKS);
 /* TRAN_Output_HKS_Write_Grid.c  */
 void TRAN_Output_HKS_Write_Grid(
 				MPI_Comm comm1,
+                                int mode,
 				int My_NGrid1_Poisson,
 				int Ngrid2,
 				int Ngrid3,
@@ -457,12 +398,14 @@ void TRAN_Output_HKS_Write_Grid(
 				int *End_Grid1, 
 				double *data,
 				double *data1,
+				double *data2,
 				FILE *fp
 				);
 
 /* TRAN_Output_Trans_HS.c  */
 void  TRAN_Output_Trans_HS(
         MPI_Comm comm1,
+        int Solver,
         int SpinP_switch, 
         double ChemP ,
         double *****H,
@@ -488,8 +431,8 @@ void  TRAN_Output_Trans_HS(
 /* TRAN_Output_Transmission.c  */ 
 void TRAN_Output_Transmission(int SpinP_switch);
 
-/* TRAN_Overwrite_Densitygrid.c  */
-void TRAN_Overwrite_Densitygrid(
+/* TRAN_Add_Density_Lead.c  */
+void TRAN_Add_Density_Lead(
             MPI_Comm comm1,
             int SpinP_switch,
             int Ngrid1,
@@ -501,24 +444,31 @@ void TRAN_Overwrite_Densitygrid(
             double **Density_Grid 
 );
 
-/* TRAN_Overwrite_V2.c  */
-void  TRAN_Overwrite_V2(
-   int Ngrid1,
-   int Ngrid2,
-   int Ngrid3,
-   int l1boundary[2],
-   int My_NGrid2_Poisson, 
-   int Start2, 
-   dcomplex *Grid[2],
-   double ***ReV2,   
-   double ***ImV2 )  ;
+/* TRAN_Add_ADensity_Lead.c */
+void TRAN_Add_ADensity_Lead(
+            MPI_Comm comm1,
+            int SpinP_switch,
+            int Ngrid1,
+            int Ngrid2,
+            int Ngrid3,
+            int Num_Cells0, 
+            int *My_Cell0, 
+            int *My_Cell1,
+            double *ADensity_Grid 
+	    );
 
 /* TRAN_Poisson.c  */
-double TRAN_Poisson(
-                    int fft_charge_flag,
-		    double ***ReV1, double ***ImV1,
+double TRAN_Poisson(double ***ReV1, double ***ImV1,
 		    double ***ReV2, double ***ImV2);
 
+/* FFT2D_Density.c  */
+void FFT2D_Density(int den_flag,
+		   double ***ReV1, double ***ImV1,
+		   double ***ReV2, double ***ImV2);
+
+/* Get_Value_inReal2D.c  */
+void Get_Value_inReal2D(double ***ReV2, double ***ImV2, double ***ReV1, double ***ImV1, double *Value_Grid);
+
 /* TRAN_Print.c  */
 void TRAN_Print2_set_eps(double e1);
 
@@ -654,18 +604,19 @@ void TRAN_Set_CentOverlap(
 			   );
 
 /* TRAN_Set_Electrode_Grid.c  */
-void TRAN_Set_Electrode_Grid(
-                MPI_Comm comm1,
-                double *Grid_Origin,  
-                double tv[4][4],       
-                double Left_tv[4][4],  
-                double Right_tv[4][4],  
-                double gtv[4][4],       
-                int Ngrid1,
-                int Ngrid2,
-                int Ngrid3,      
-                double *Density_Grid   
-);
+void TRAN_Set_Electrode_Grid(MPI_Comm comm1,
+                             int *TRAN_Poisson_flag2,
+			     double *Grid_Origin,   /* origin of the grid */
+			     double tv[4][4],       /* unit vector of the cell*/
+			     double Left_tv[4][4],  /* unit vector  left */
+			     double Right_tv[4][4], /* unit vector right */
+			     double gtv[4][4],      /* unit vector of the grid point, which is gtv*integer */
+			     int Ngrid1,
+			     int Ngrid2,
+			     int Ngrid3,            /* # of c grid points */
+			     double *Density_Grid   /* work */
+			     );
+
 
 /* TRAN_Set_Electrode_Grid.c  */
 /* *static
@@ -677,7 +628,12 @@ void TRAN_Set_IntegPath_Square(void);
 void      TRAN_Set_IntegPath_ThermalArc(void);
 
 /* TRAN_Set_IntegPath.c  */
-void TRAN_Set_IntegPath( double kBvalue, double Electronic_Temperature );
+void TRAN_Set_IntegPath( MPI_Comm comm1,
+			 double TRAN_eV2Hartree,
+			 double kBvalue, double Electronic_Temperature );
+
+/* TRAN_Check_Input.c */
+void TRAN_Check_Input( MPI_Comm comm1, int Solver );
 
 /* TRAN_Set_MP.c  */
 void TRAN_Set_MP(
@@ -701,15 +657,6 @@ void TRAN_Set_PathEnergyStr_Square(
    int    tran_square_path_ene_fix[4]
 );
 
-/* TRAN_Set_PathEnergyStr.c  */
-void TRAN_Set_PathEnergyStr(
-   int tran_integ_pathtype, 
-   int m,
-   char **str,
-   double default_relative_ene[4],  
-   double tran_square_path_ene[4],  
-   int  tran_square_path_ene_fix[4]  
-);
 
 /* TRAN_Set_SurfOverlap.c  */
 void TRAN_Set_SurfOverlap( MPI_Comm comm1, char *position, double k2, double k3 );
@@ -717,9 +664,6 @@ void TRAN_Set_SurfOverlap( MPI_Comm comm1, char *position, double k2, double k3
 /* TRAN_Set_Value.c  */
 void TRAN_Set_Value_double(dcomplex *A, int n, double a, double b);
 
-/* TRAN_adjust_Grid_Origin.c  */
-void TRAN_adjust_Grid_Origin( MPI_Comm comm1, double Grid_Origin[4]);
-
 /* TRAN_adjust_Ngrid.c  */
 void  TRAN_adjust_Ngrid( MPI_Comm comm1, int *Ngrid1,int *Ngrid2, int *Ngrid3);
 
